package com.rnsdk.clvideo.download;

import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.IBinder;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import com.castlabs.android.SdkConsts;
import com.castlabs.android.drm.DrmConfiguration;
import com.castlabs.android.player.models.AudioTrack;
import com.castlabs.android.player.models.SubtitleTrack;
import com.castlabs.android.player.models.VideoTrackQuality;
import com.castlabs.sdk.downloader.Download;
import com.castlabs.sdk.downloader.DownloadService;
import com.castlabs.sdk.downloader.DownloadServiceBinder;
import com.castlabs.sdk.downloader.Downloader;
import com.castlabs.sdk.downloader.MessageHandler;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.rnsdk.clvideo.utils.JsonUtils;
import com.rnsdk.clvideo.video.VideoUtils;
import java.io.IOException;
import java.util.Objects;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * The module which is responsible for CLDownload methods and events.
 */
public class DownloadModule extends ReactContextBaseJavaModule {

  public static final String REACT_CLASS = "DownloadModule";
  public static final String EVENT_PREFIX = "CLDownloader";

  private DownloadServiceBinder downloadServiceBinder;
  private ServiceConnection serviceConnection;
  private BroadcastReceiver broadcastReceiver;

  /**
   * The constructor for download module.
   *
   * @param context the application context
   */
  public DownloadModule(ReactApplicationContext context) {
    super(context);
    initServiceConnection();
    bindDownloadService();
    registerReceiver();
  }

  @Override
  @NonNull
  public String getName() {
    return REACT_CLASS;
  }

  /**
   * Starts the download.
   *
   * @param instanceId The instanceId of the download
   * @param selection The selected tracks for download
   */
  @ReactMethod
  public void startDownload(String instanceId, ReadableMap selection) {
    Download download = DownloadInstanceManager.getInstance(instanceId);
    download.setSelectedVideoTrackQuality(selection.getInt("videoQuality"));
    download.setSelectedAudioTracks(getIntArray(selection.getArray("audioTracks")));
    download.setSelectedSubtitleTracks(getIntArray(selection.getArray("subtitleTracks")));
    try {
      downloadServiceBinder.createDownload(download, true);
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Resumes the download.
   *
   * @param instanceId The instanceId of the download
   */
  @ReactMethod
  public void resumeDownload(String instanceId) {
    downloadServiceBinder.resumeDownload(instanceId);
  }

  @ReactMethod
  public void pauseDownload(String instanceId) {
    downloadServiceBinder.pauseDownload(instanceId);
  }

  /**
   * Deletes the download.
   *
   * @param instanceId The instanceId of the download
   */
  @ReactMethod
  public void deleteDownload(String instanceId) {
    downloadServiceBinder.deleteDownload(instanceId);
    DownloadInstanceManager.remove(instanceId);
  }

  /**
   * Selects an existing download and returns the download progress.
   *
   * @param id The download id
   * @param promise The promise which is used to send the result
   */
  @ReactMethod
  public void prepareById(String id, Promise promise) {
    Download download = downloadServiceBinder.findDownloadById(id);
    if (download == null) {
      promise.reject("error", "Download not found!");
    } else {
      DownloadInstanceManager.setInstance(id, download);
      promise.resolve(getProgressMap(download));
    }
  }

  /**
   * Creates a new download instance according to the provided config.
   *
   * @param config The download config
   * @param promise The promise which is used to send the result
   */
  @ReactMethod
  public void prepareDownload(ReadableMap config, Promise promise) {
    final Context context = getReactApplicationContext();
    String instanceId = config.getString("id");

    Bundle bundle = new Bundle();
    bundle.putString(SdkConsts.INTENT_URL, config.getString("url"));
    bundle.putString(SdkConsts.INTENT_DOWNLOAD_ID, instanceId);
    bundle.putString(SdkConsts.INTENT_DOWNLOAD_FOLDER, getDownloadPath(instanceId));

    if (config.hasKey("drm")) {
      try {
        JSONObject drm = new JSONObject(Objects.requireNonNull(config.getString("drm")));
        DrmConfiguration drmConfiguration = JsonUtils.jsonToDrmConfiguration(drm);
        bundle.putParcelable(SdkConsts.INTENT_DRM_CONFIGURATION, drmConfiguration);
      } catch (JSONException e) {
        promise.reject("error", e.getMessage());
      }
    }

    downloadServiceBinder.prepareDownload(context, bundle, new Downloader.ModelReadyCallback() {
      @Override
      public void onError(@NonNull Exception e) {
        promise.reject("error", e.getMessage());
      }

      @Override
      public void onModelAvailable(@NonNull Download download) {
        DownloadInstanceManager.setInstance(instanceId, download);
        promise.resolve(getDownloadableTracks(download));
      }
    });
  }

  /**
   * Returns the current state of the download.
   *
   * @param instanceId The instanceId of the download
   * @param promise The promise which is used to send the result
   */
  @ReactMethod
  public void getState(String instanceId, Promise promise) {
    Download download = DownloadInstanceManager.getInstance(instanceId);
    if (download != null) {
      promise.resolve(download.getState());
    } else {
      promise.reject("error", "Download not found!");
    }
  }

  /**
   * Returns the location of the downloaded manifest file.
   *
   * @param instanceId The instanceId of the download
   * @param promise The promise which is used to send the result
   */
  @ReactMethod
  public void getLocalManifestUrl(String instanceId, Promise promise) {
    Download download = DownloadInstanceManager.getInstance(instanceId);
    if (download != null) {
      promise.resolve(download.getLocalManifestUrl());
    } else {
      promise.reject("error", "Download not found!");
    }
  }

  /**
   * Returns the available tracks for download.
   *
   * @param instanceId The instanceId of the download
   * @param promise The promise which is used to send the result
   */
  @ReactMethod
  public void getTracks(String instanceId, Promise promise) {
    Download download = DownloadInstanceManager.getInstance(instanceId);
    if (download != null) {
      promise.resolve(getDownloadableTracks(download));
    } else {
      promise.reject("error", "Download not found!");
    }
  }

  /**
   * Returns the download progress.
   *
   * @param instanceId The instanceId of the download
   * @param promise The promise which is used to send the result
   */
  @ReactMethod
  public void getProgress(String instanceId, Promise promise) {
    Download download = DownloadInstanceManager.getInstance(instanceId);
    if (download != null) {
      promise.resolve(getProgressMap(download));
    } else {
      promise.reject("error", "Download not found!");
    }
  }

  // Required for rn built in EventEmitter Calls.
  @ReactMethod
  public void addListener(String eventName) {
  }

  @ReactMethod
  public void removeListeners(Integer count) {
  }

  private void initServiceConnection() {
    serviceConnection = new ServiceConnection() {
      @Override
      public void onServiceConnected(ComponentName name, IBinder service) {
        downloadServiceBinder = (DownloadServiceBinder) service;
      }

      @Override
      public void onServiceDisconnected(ComponentName name) {
        downloadServiceBinder = null;
      }
    };
  }

  private void bindDownloadService() {
    Context context = getReactApplicationContext();
    Intent intent = new Intent(context, DownloadService.class);
    context.bindService(intent, serviceConnection, Context.BIND_AUTO_CREATE);
  }

  private void unbindDownloadService() {
    Context context = getReactApplicationContext();
    context.unbindService(serviceConnection);
  }

  private void registerReceiver() {
    final Context appContext = getReactApplicationContext();
    broadcastReceiver = new BroadcastReceiver() {
      @Override
      public void onReceive(Context context, Intent intent) {
        if (intent == null || intent.getAction() == null) {
          return;
        }
        String downloadId = intent.getStringExtra(MessageHandler.INTENT_DOWNLOAD_ID);
        switch (intent.getAction()) {
          case MessageHandler.ACTION_DOWNLOAD_STARTED: {
            sendStartEvent(downloadId);
            break;
          }
          case MessageHandler.ACTION_DOWNLOAD_STOPPED: {
            sendStopEvent(downloadId);
            break;
          }
          case MessageHandler.ACTION_DOWNLOAD_PROGRESS: {
            sendProgressEvent(downloadId);
            break;
          }
          case MessageHandler.ACTION_DOWNLOAD_ERROR: {
            sendErrorEvent(downloadId, intent);
            break;
          }
          case MessageHandler.ACTION_DOWNLOAD_COMPLETED: {
            sendCompleteEvent(downloadId);
            break;
          }
          default:
        }
      }
    };

    IntentFilter filter = new IntentFilter();
    filter.addCategory(MessageHandler.INTENT_DOWNLOAD_CATEGORY);
    filter.addAction(MessageHandler.ACTION_DOWNLOAD_NO_PENDING);
    filter.addAction(MessageHandler.ACTION_DOWNLOAD_PROGRESS);
    filter.addAction(MessageHandler.ACTION_DOWNLOAD_COMPLETED);
    filter.addAction(MessageHandler.ACTION_DOWNLOAD_CREATED);
    filter.addAction(MessageHandler.ACTION_DOWNLOAD_DELETED);
    filter.addAction(MessageHandler.ACTION_DOWNLOAD_ERROR);
    filter.addAction(MessageHandler.ACTION_DOWNLOAD_STARTED);
    filter.addAction(MessageHandler.ACTION_DOWNLOAD_STOPPED);
    filter.addAction(MessageHandler.ACTION_DOWNLOAD_STORAGE_LOW);
    filter.addAction(MessageHandler.ACTION_DOWNLOAD_STORAGE_OK);
    filter.addAction(MessageHandler.ACTION_DOWNLOAD_PATH_UPDATE);

    LocalBroadcastManager.getInstance(appContext).registerReceiver(broadcastReceiver, filter);
  }

  private void unregisterReceiver() {
    Context appContext = getReactApplicationContext();
    LocalBroadcastManager.getInstance(appContext).unregisterReceiver(broadcastReceiver);
  }

  private void sendStartEvent(String downloadId) {
    WritableMap event = Arguments.createMap();
    event.putString("downloadId", downloadId);

    String eventName = EVENT_PREFIX + "-Start";
    createEvent(event, eventName);
  }

  private void sendStopEvent(String downloadId) {
    WritableMap event = Arguments.createMap();
    event.putString("downloadId", downloadId);

    String eventName = EVENT_PREFIX + "-Stop";
    createEvent(event, eventName);
  }

  private void sendProgressEvent(String downloadId) {
    Download download = DownloadInstanceManager.getInstance(downloadId);
    if (download != null && download.getState() == Download.STATE_LOADING) {
      WritableMap event = getProgressMap(download);
      event.putString("downloadId", downloadId);

      String eventName = EVENT_PREFIX + "-Progress";
      createEvent(event, eventName);
    }
  }

  private void sendErrorEvent(String downloadId, Intent intent) {

    String error = intent.getStringExtra(MessageHandler.INTENT_DOWNLOAD_ERROR);
    WritableMap event = Arguments.createMap();
    event.putString("error", error);
    event.putString("downloadId", downloadId);

    String eventName = EVENT_PREFIX + "-Error";
    createEvent(event, eventName);
  }

  private void sendCompleteEvent(String downloadId) {
    WritableMap event = Arguments.createMap();
    event.putString("downloadId", downloadId);

    String eventName = EVENT_PREFIX + "-Complete";
    createEvent(event, eventName);
  }

  private ReadableMap getDownloadableTracks(@NonNull Download download) {

    WritableArray videoQualities = Arguments.createArray();
    if (download.getVideoTrackQualities() != null) {
      for (int i = 0; i < download.getVideoTrackQualities().length; i++) {
        VideoTrackQuality quality = download.getVideoTrackQualities()[i];
        videoQualities.pushMap(VideoUtils.videoQualityToMap(quality));
      }
    }

    WritableArray audioTracks = Arguments.createArray();
    if (download.getAudioTracks() != null) {
      for (int i = 0; i < download.getAudioTracks().length; i++) {
        AudioTrack track = download.getAudioTracks()[i];
        audioTracks.pushMap(VideoUtils.audioTrackToMap(track));
      }
    }

    WritableArray subtitleTracks = Arguments.createArray();
    if (download.getSubtitleTracks() != null) {
      for (int i = 0; i < download.getSubtitleTracks().length; i++) {
        SubtitleTrack track = download.getSubtitleTracks()[i];
        subtitleTracks.pushMap(VideoUtils.subtitleTrackToMap(track));
      }
    }

    WritableMap out = Arguments.createMap();
    out.putArray("videoQualities", videoQualities);
    out.putArray("audioTracks", audioTracks);
    out.putArray("subtitleTracks", subtitleTracks);
    return out;
  }

  private WritableMap getProgressMap(Download download) {
    double downloadedSize = (double) download.getDownloadedSize();
    double estimatedSize = (double) download.getEstimatedSize();
    double progress = 100.0 * downloadedSize / estimatedSize;
    if (progress < 0) {
      progress = 0;
    }
    if (progress > 100) {
      progress = 100;
    }
    WritableMap map = Arguments.createMap();
    map.putDouble("downloadedSize", downloadedSize);
    map.putDouble("estimatedSize", estimatedSize);
    map.putDouble("progress", progress);
    return map;
  }

  private String getDownloadPath(String downloadId) {
    Context context = getReactApplicationContext();
    String path = context.getExternalFilesDir(null).getAbsolutePath();
    path += "/downloads/" + downloadId + "/";
    return path;
  }

  private int[] getIntArray(@Nullable ReadableArray array) {
    if (array == null) {
      return null;
    }
    int[] out = new int[array.size()];
    for (int i = 0; i < array.size(); i++) {
      out[i] = array.getInt(i);
    }
    return out;
  }

  private void createEvent(WritableMap event, String name) {
    ReactContext reactContext = getReactApplicationContext();
    reactContext.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
        .emit(name, event);
  }
}
