package com.rnsdk.clvideo.download;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.util.Log;
import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import com.castlabs.sdk.downloader.Download;
import com.castlabs.sdk.downloader.DownloadNotificationProvider;
import com.castlabs.sdk.downloader.DownloadServiceBinder;
import com.castlabs.sdk.downloader.MessageHandler;
import com.rnsdk.clvideo.R;
import java.util.List;

/**
 * This class is responsible for the information shown on the download notification.
 */
public class NotificationProvider extends DownloadNotificationProvider {

  private static final String TAG = "NotificationProvider";

  private static final int DOWNLOAD_NOTIFICATION_ID = 1234;
  private String lastAction = "";
  private Notification.Builder builder;
  private boolean storageLow = false;

  public NotificationProvider() {
    super(DOWNLOAD_NOTIFICATION_ID);
  }

  /**
   * Provides the {@link Notification}.
   *
   * @param downloadServiceBinder binder instance to get Download-related info
   * @param context the application context
   * @return the Notification to show
   */
  @NonNull
  public Notification getNotification(@NonNull DownloadServiceBinder downloadServiceBinder,
                                      @NonNull Context context) {
    // Setup the notification builder. We enable channel support on O
    if (builder == null) {
      String id = "content_download_channel";
      if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
        createNotificationChannel(context, id);
        builder = new Notification.Builder(context, id);
      } else {
        builder = new Notification.Builder(context);
      }
    }

    // Set the basic properties of the notification
    builder.setAutoCancel(false).setSmallIcon(R.mipmap.download);

    List<Download> downloads;
    try {
      downloads = downloadServiceBinder.getDownloads();
    } catch (Exception e) {
      Log.e(TAG, "Error while getting Downloads", e);
      return builder.build();
    }

    Download activeDownload = getActiveDownload(downloads);
    if (activeDownload != null) {
      builder.setContentTitle("Downloading #" + activeDownload.getId());
      builder.setContentText(getNotificationText(downloads));
      double downloadedRatio = (double) activeDownload.getDownloadedSize()
          / (double) activeDownload.getEstimatedSize();
      int progress = Math.max(0, Math.min(100, (int) (downloadedRatio * 100.0)));
      builder.setProgress(100, progress, false);
      // Set the state to ongoing so the notification can not be removed by the user since
      // we need to notification in place while downloading. This is especially important
      // since Android O because only the notification allows us to keep the download service
      // running even if the App is in background
      builder.setOngoing(true);
    } else {
      // If we have no currently active download we can mark the notification as not-ongoing
      // and allow the user to remove it
      builder.setOngoing(false);
      // Make sure the progress indicator is hidden
      builder.setProgress(0, 0, false);
      // Build a description
      if (storageLow) {
        // if the last action that we got was a storage low message, inform the user
        // that all downloads are paused because we do not have enough space.
        builder.setContentTitle("Not enough space!");
        builder.setContentText("Free up space to continue downloading");
      } else {
        builder.setContentTitle("No Active Downloads");
        builder.setContentText(getNotificationText(downloads));
      }
    }
    return builder.build();
  }

  private Download getActiveDownload(List<Download> downloads) {
    for (Download download : downloads) {
      if (download.getState() == Download.STATE_LOADING) {
        return download;
      }
    }
    return null;
  }

  private int getQueuedDownloads(List<Download> downloads) {
    int count = 0;
    for (Download download : downloads) {
      if (download.getState() == Download.STATE_QUEUED) {
        count++;
      }
    }
    return count;
  }

  private int getErrorDownloads(List<Download> downloads) {
    int count = 0;
    for (Download download : downloads) {
      if (download.getState() == Download.STATE_ERROR) {
        count++;
      }
    }
    return count;
  }

  private String getNotificationText(List<Download> downloads) {
    String content = "";
    int queuedDownloads = getQueuedDownloads(downloads);
    int errorDownloads = getErrorDownloads(downloads);
    if (queuedDownloads + errorDownloads > 0) {
      if (queuedDownloads > 0) {
        content += queuedDownloads + " download" + (queuedDownloads == 1 ? " is" : "s are")
            + " in the queue. ";
      }
      if (errorDownloads > 0) {
        content += errorDownloads + " download" + (errorDownloads == 1 ? "" : "s") + " got error.";
      }
    }
    return content;
  }

  @Override
  public boolean shouldKeepNotification(@NonNull DownloadServiceBinder downloadServiceBinder) {
    // The default implementation returns false here, which means that
    // notification will be removed when the download service has no pending downloads
    // and leaves foreground mode.
    // We return true here to keep our notifications displayed and let the user actively dismiss
    // them. Note that we make sure to call setOngoing accordingly when we build our notifications
    // to prevent the user from removing a notification while downloads are still in
    // progress
    return true;
  }

  @Override
  public boolean onDownloadEvent(@NonNull DownloadServiceBinder downloadServiceBinder,
                                 @NonNull Intent intent) {
    // we get the event action here and cache it. This method is called before the
    // notification is created so we can use that when building the notification above
    lastAction = intent.getAction();
    if (lastAction == null) {
      return false;
    }

    // We can return false from this method if we don't want a notification to be shown.
    // Here we want to make sure that we do not show a notification when
    // we are deleting downloads. In call other cases we want the notification
    // to be shown to the user.
    switch (lastAction) {
      case MessageHandler.ACTION_DOWNLOAD_DELETED:
        return false;
      case MessageHandler.ACTION_DOWNLOAD_STORAGE_LOW:
        storageLow = true;
        break;
      case MessageHandler.ACTION_DOWNLOAD_STORAGE_OK:
        storageLow = false;
        break;
      default:
    }
    return true;
  }

  @RequiresApi(api = Build.VERSION_CODES.O)
  private static void createNotificationChannel(@NonNull Context context, @NonNull String id) {

    CharSequence name = "Downloads";
    String description = "These notifications will inform you about the ongoing downloads";

    NotificationManager notificationManager = (NotificationManager) context.getSystemService(
        Context.NOTIFICATION_SERVICE);
    if (notificationManager != null) {
      int importance = NotificationManager.IMPORTANCE_LOW;
      NotificationChannel channel = new NotificationChannel(id, name, importance);
      channel.setDescription(description);
      channel.enableLights(false);
      channel.enableVibration(false);
      notificationManager.createNotificationChannel(channel);
    }
  }
}
