package com.rnsdk.clvideo.utils;

import android.os.Bundle;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import com.castlabs.analytics.AnalyticsMetaData;
import com.castlabs.android.PlayerSDK;
import com.castlabs.android.SdkConsts;
import com.castlabs.android.drm.Drm;
import com.castlabs.android.drm.DrmConfiguration;
import com.castlabs.android.drm.DrmTodayConfiguration;
import com.castlabs.android.drm.DrmUtils;
import com.castlabs.android.network.NetworkConfiguration;
import com.castlabs.android.network.RetryConfiguration;
import com.castlabs.android.player.BufferConfiguration;
import com.castlabs.android.player.CatchupConfiguration;
import com.castlabs.android.player.CustomUtcTimingElement;
import com.castlabs.android.player.LiveConfiguration;
import com.castlabs.android.player.PlayerConfig;
import com.castlabs.android.player.models.SideloadedTrack;
import com.google.android.exoplayer2.C;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.concurrent.TimeUnit;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 * Utility to create {@link PlayerConfig} from a json string.
 */
public class JsonUtils {

  private static final String CONTENT_TYPE_DASH = "DASH";
  private static final String CONTENT_TYPE_SMOOTH = "SMOOTH";
  private static final String CONTENT_TYPE_HLS = "HLS";
  private static final String CONTENT_TYPE_MP4 = "MP4";

  private static final String SIDELOADED_TRACK_TYPE_THUMBNAIL = "THUMBNAIL";
  private static final String SIDELOADED_TRACK_TYPE_SUBTITLE = "SUBTITLE";

  private static final String DRM_TYPE_NONE = "NONE";
  private static final String DRM_TYPE_DRMTODAY = "DRMTODAY";
  private static final String DRM_TYPE_DRMTODAY_ONBOARD = "DRMTODAY_ONBOARD";
  private static final String DRM_TYPE_VERIMATRIX = "VERIMATRIX";
  private static final String DRM_TYPE_CUSTOM = "CUSTOM";

  private static final String DRM_SYSTEM_AUTO = "AUTO";
  private static final String DRM_SYSTEM_WIDEVINE = "WIDEVINE";
  private static final String DRM_SYSTEM_OMA = "OMA";
  private static final String DRM_SYSTEM_PLAYREADY = "PLAYREADY";
  private static final String DRM_SYSTEM_FAIRPLAY = "FAIRPLAY";
  private static final String DRM_SYSTEM_CLEAR_KEY = "CLEAR_KEY";
  private static final String DRM_SYSTEM_WISE_PLAY = "WISE_PLAY";
  private static final String DRM_SYSTEM_CASTLABS = "CASTLABS";

  /**
   * Converts a json string to {@link PlayerConfig}.
   *
   * @param json The player configuration
   * @return the {@link PlayerConfig}
   * @throws JSONException the exception during conversion
   */
  @NonNull
  public static PlayerConfig buildFromJsonConfig(@NonNull final JSONObject json)
      throws JSONException {
    final JSONObject source = json.getJSONObject("source");

    PlayerConfig.Builder builder = new PlayerConfig.Builder(source.getString("url"));

    switch (source.getString("type")) {
      case CONTENT_TYPE_DASH:
        builder.contentType(SdkConsts.CONTENT_TYPE_DASH);
        break;
      case CONTENT_TYPE_MP4:
        builder.contentType(SdkConsts.CONTENT_TYPE_MP4);
        break;
      case CONTENT_TYPE_HLS:
        builder.contentType(SdkConsts.CONTENT_TYPE_HLS);
        break;
      case CONTENT_TYPE_SMOOTH:
        builder.contentType(SdkConsts.CONTENT_TYPE_SMOOTHSTREAMING);
        break;
      default:
        throw new IllegalArgumentException("Unexpected content type: " + source.getString("type"));
    }

    builder.id(json.optString("id"));

    if (json.has("autoPlay")) {
      builder.autoPlay(json.getBoolean("autoPlay"));
    } else {
      builder.autoPlay(false);
    }
    if (json.has("startTimeMs")) {
      builder.positionUs(json.getLong("startTimeMs") * 1000);
    }
    if (json.has("live")) {
      builder.live(json.getBoolean("live"));
    }
    if (json.has("threesixty")) {
      builder.threesixty(json.getBoolean("threesixty"));
    }
    if (json.has("preferredTextLanguage")) {
      builder.preferredTextLanguage(json.getString("preferredTextLanguage"));
    }
    if (json.has("preferredAudioLanguage")) {
      builder.preferredAudioLanguage(json.getString("preferredAudioLanguage"));
    }
    if (json.has("metaData")) {
      builder.metaData(jsonToBundle(json.getJSONObject("metaData")));
    }
    if (json.has("analyticsMetaData")) {
      builder.analyticsMetaData(jsonToAnalyticsMetaData(
          json.getJSONObject("analyticsMetaData")));
    }
    if (json.has("contentParameters")) {
      builder.contentParameters(jsonToBundle(json.getJSONObject("contentParameters")));
    }
    if (json.has("contentQueryParameters")) {
      builder.contentQueryParameters(jsonToBundle(
          json.getJSONObject("contentQueryParameters")));
    }
    if (json.has("segmentParameters")) {
      builder.segmentParameters(jsonToBundle(json.getJSONObject("segmentParameters")));
    }
    if (json.has("segmentQueryParameters")) {
      builder.segmentQueryParameters(jsonToBundle(
          json.getJSONObject("segmentQueryParameters")));
    }
    if (json.has("configurationUrl")) {
      builder.configurationUrl(json.getString("configurationUrl"));
    }
    if (json.has("userId")) {
      builder.userID(json.getString("userId"));
    }
    if (json.has("forceInStreamDrmInitData")) {
      builder.forceInStreamDrmInitData(json.getBoolean("forceInStreamDrmInitData"));
    }
    if (json.has("sideloadedTracks")) {
      JSONArray tracksArray = json.getJSONArray("sideloadedTracks");
      builder.sideloadedTracks(jsonToSideloadedTracks(tracksArray));
    }

    JSONObject jsonLiveConfiguration = null;
    JSONObject jsonAndroidLiveConfiguration = null;

    if (json.has("liveConfiguration")) {
      jsonLiveConfiguration = json.getJSONObject("liveConfiguration");
    }

    if (json.has("android")) {
      JSONObject jsonAndroid = json.getJSONObject("android");
      if (jsonAndroid.has("bufferConfiguration")) {
        JSONObject jsonBufferConfiguration = jsonAndroid.getJSONObject("bufferConfiguration");
        builder.bufferConfiguration(jsonToBufferConfiguration(jsonBufferConfiguration));
      }
      if (jsonAndroid.has("liveConfiguration")) {
        jsonAndroidLiveConfiguration = jsonAndroid.getJSONObject("liveConfiguration");
      }
    }

    if (jsonLiveConfiguration != null || jsonAndroidLiveConfiguration != null) {
      builder.liveConfiguration(
          jsonToLiveConfiguration(jsonLiveConfiguration, jsonAndroidLiveConfiguration)
      );
    }

    if (json.has("networkConfiguration")) {
      JSONObject jsonObject = json.getJSONObject("networkConfiguration");
      builder.networkConfiguration(jsonToNetworkConfiguration(jsonObject));
    }

    DrmConfiguration drmConfiguration = jsonToDrmConfiguration(json);
    if (drmConfiguration != null) {
      builder.drmConfiguration(drmConfiguration);
    }

    final PlayerConfig playerConfig = builder.get();

    if (playerConfig.analyticsMetaData != null) {
      playerConfig.analyticsMetaData =
          Analytics.createSpecificMetadata(playerConfig.analyticsMetaData);
    }

    return playerConfig;
  }

  /**
   * Takes the first player configuration in the list and
   * maps `android.secondaryDisplayBehaviour` to the secondary display constant.
   * Defaults to `PlayerSDK.SECONDARY_DISPLAY`.
   *
   * @param jsonPlayerConfigurations The list of player configurations
   * @return the secondary display constant.
   * @throws JSONException the exception during JSON parsing
   */
  public static int jsonToSecondaryDisplay(
      @NonNull final JSONArray jsonPlayerConfigurations
  ) throws JSONException {
    final int defaultSecondaryScreen = SdkConsts.SECONDARY_DISPLAY_NEVER;

    if (jsonPlayerConfigurations.length() == 0) {
      return defaultSecondaryScreen;
    }

    JSONObject jsonPlayerConfiguration = jsonPlayerConfigurations.getJSONObject(0);

    if (
        !jsonPlayerConfiguration
            .has("android")
        || !jsonPlayerConfiguration
            .getJSONObject("android")
            .has("secondaryDisplayBehaviour")
    ) {
      if (jsonPlayerConfiguration.has("secondaryDisplayBehaviour")) {
        String secondaryDisplayBehaviour =
            jsonPlayerConfiguration.getString("secondaryDisplayBehaviour");
        switch (secondaryDisplayBehaviour) {
          case "never":
            return SdkConsts.SECONDARY_DISPLAY_NEVER;
          case "allowAlways":
            return SdkConsts.SECONDARY_DISPLAY_ALLOW_ALWAYS;
          default:
            return defaultSecondaryScreen;
        }
      } else {
        return defaultSecondaryScreen;
      }
    }

    JSONObject jsonAndroid = jsonPlayerConfiguration.getJSONObject("android");
    String secondaryDisplayBehaviour = jsonAndroid.getString("secondaryDisplayBehaviour");
    switch (secondaryDisplayBehaviour) {
      case "never":
        return SdkConsts.SECONDARY_DISPLAY_NEVER;
      case "allowUnprotectedContent":
        return SdkConsts.SECONDARY_DISPLAY_ALLOW_UNPROTECTED_CONTENT;
      case "allowSecureDisplay":
        return SdkConsts.SECONDARY_DISPLAY_ALLOW_SECURE_DISPLAY;
      case "allowAlways":
        return SdkConsts.SECONDARY_DISPLAY_ALLOW_ALWAYS;
      default:
        return defaultSecondaryScreen;
    }
  }

  /**
   * Converts json object to {@link DrmConfiguration}.
   *
   * @param json the input
   * @return the instance of {@link DrmConfiguration}
   * @throws JSONException the exception
   */
  @Nullable
  public static DrmConfiguration jsonToDrmConfiguration(JSONObject json) throws JSONException {
    String drmType = json.optString("drmType", DRM_TYPE_NONE);
    JSONObject drmConfiguration;
    if (drmType.equals(DRM_TYPE_NONE)) {
      return null;
    } else {
      drmConfiguration = json.getJSONObject("drmConfiguration");
    }
    String drmSystem = json.optString("drmSystem", DRM_SYSTEM_AUTO);
    Drm drm = stringToDrm(drmSystem);

    switch (drmType) {
      case DRM_TYPE_DRMTODAY: {
        String environment = drmConfiguration.getString("environment");
        String url = environmentToUrl(environment);
        String userId = drmConfiguration.getString("userId");
        String sessionId = drmConfiguration.getString("sessionId");
        String merchant = drmConfiguration.getString("merchant");
        DrmTodayConfiguration.Builder drmBuilder = new DrmTodayConfiguration.Builder(
            url,
            userId,
            sessionId,
            merchant
        );
        if (drmConfiguration.has("assetId")) {
          drmBuilder.assetId(drmConfiguration.getString("assetId"));
        }
        if (drmConfiguration.has("variantId")) {
          drmBuilder.variantId(drmConfiguration.getString("variantId"));
        }
        if (drmConfiguration.has("authToken")) {
          drmBuilder.authToken(drmConfiguration.getString("authToken"));
        }
        if (drmConfiguration.has("offlineId")) {
          drmBuilder.offlineId(drmConfiguration.getString("offlineId"));
        }
        drmBuilder.drm(drm);
        return drmBuilder.get();
      }
      case DRM_TYPE_DRMTODAY_ONBOARD: {
        String url = drmConfiguration.getString("baseUrl");
        String userId = drmConfiguration.optString("userId", "");
        String sessionId = drmConfiguration.optString("sessionId", "");
        String merchant = drmConfiguration.optString("merchant", "");
        DrmTodayConfiguration.Builder drmBuilder = new DrmTodayConfiguration.Builder(
            url,
            userId,
            sessionId,
            merchant
        );
        if (drmConfiguration.has("assetId")) {
          drmBuilder.assetId(drmConfiguration.getString("assetId"));
        }
        if (drmConfiguration.has("variantId")) {
          drmBuilder.variantId(drmConfiguration.getString("variantId"));
        }
        if (drmConfiguration.has("offlineId")) {
          drmBuilder.offlineId(drmConfiguration.getString("offlineId"));
        }
        drmBuilder.drm(drm);
        return drmBuilder.get();
      }
      case DRM_TYPE_VERIMATRIX:
      case DRM_TYPE_CUSTOM: {

        JSONObject widevineJson = null;
        if (drmConfiguration.has("WIDEVINE")) {
          widevineJson = drmConfiguration.getJSONObject("WIDEVINE");
        }

        JSONObject playreadyJson = null;
        if (drmConfiguration.has("PLAYREADY")) {
          playreadyJson = drmConfiguration.getJSONObject("PLAYREADY");
        }

        JSONObject castLabsJson = null;
        if (drmConfiguration.has("CASTLABS")) {
          castLabsJson = drmConfiguration.getJSONObject("CASTLABS");
        }

        // Select best DRM if available
        Drm bestDrm = DrmUtils.selectBestDrm(Drm.BestAvailable);
        if (bestDrm == Drm.Widevine && widevineJson != null) {
          return getCustomDrmConfiguration(widevineJson, Drm.Widevine);
        } else if (bestDrm == Drm.Playready && playreadyJson != null) {
          return getCustomDrmConfiguration(playreadyJson, Drm.Playready);
        }

        // Select non null DRM
        if (castLabsJson != null) {
          return getCustomDrmConfiguration(castLabsJson, drm);
        } else if (widevineJson != null) {
          return getCustomDrmConfiguration(widevineJson, Drm.Widevine);
        } else if (playreadyJson != null) {
          return getCustomDrmConfiguration(playreadyJson, Drm.Playready);
        } else {
          throw new IllegalArgumentException("Unexpected DRM system");
        }
      }
      default: {
        return null;
      }
    }
  }

  private static DrmConfiguration getCustomDrmConfiguration(JSONObject json, Drm drm)
      throws JSONException {
    final String url = json.getString("licensingUrl");

    String offlineId = null;
    if (json.has("offlineId")) {
      offlineId = json.getString("offlineId");
    }

    Drm audioDrm = null;
    if (json.has("audioDrmSystem")) {
      audioDrm = stringToDrm(json.getString("audioDrmSystem"));
    }

    boolean keyRotation = json.optBoolean("keyRotation", false);
    boolean forceWidevineL3 = json.optBoolean("forceWidevineL3", false);
    boolean playClearSamplesWithoutKeys = json.optBoolean(
        "playClearSamplesWithoutKeys", true);
    long renewalThreshold = json.optLong("renewalThreshold", C.TIME_UNSET);

    Bundle requestParameters = new Bundle();
    if (json.has("licensingParameters")) {
      requestParameters = jsonToBundle(json.getJSONObject("licensingParameters"));
    }

    return new DrmConfiguration(
        url,
        playClearSamplesWithoutKeys,
        drm,
        audioDrm,
        offlineId,
        requestParameters,
        keyRotation,
        forceWidevineL3,
        renewalThreshold);
  }

  @NonNull
  private static ArrayList<SideloadedTrack> jsonToSideloadedTracks(JSONArray tracksArray)
      throws JSONException {
    ArrayList<SideloadedTrack> sideloadedTracks = new ArrayList<>(tracksArray.length());
    for (int trackIdx = 0; trackIdx < tracksArray.length(); trackIdx++) {
      JSONObject track = tracksArray.getJSONObject(trackIdx);
      SideloadedTrack.Builder trackBuilder;
      final String trackType = track.getString("trackType");
      switch (trackType) {
        case SIDELOADED_TRACK_TYPE_SUBTITLE:
          SideloadedTrack.SubtitleBuilder subtitleBuilder = new SideloadedTrack.SubtitleBuilder();
          if (track.has("language")) {
            subtitleBuilder.language(track.getString("language"));
          }
          if (track.has("label")) {
            subtitleBuilder.label(track.getString("label"));
          }
          if (track.has("kind")) {
            subtitleBuilder.kind(track.getString("kind"));
          }
          trackBuilder = subtitleBuilder;
          break;
        case SIDELOADED_TRACK_TYPE_THUMBNAIL:
          SideloadedTrack.ThumbnailBuilder thumbBuilder = new SideloadedTrack.ThumbnailBuilder();
          if (track.has("gridWidth")) {
            thumbBuilder.gridWidth(track.getInt("gridWidth"));
          }
          if (track.has("gridHeight")) {
            thumbBuilder.gridHeight(track.getInt("gridHeight"));
          }
          if (track.has("intervalMs")) {
            thumbBuilder.intervalMs(track.getLong("intervalMs"));
          }
          trackBuilder = thumbBuilder;
          break;
        default:
          throw new IllegalArgumentException("Unexpected SideloadedTrack type: " + trackType);
      }
      if (track.has("mimeType")) {
        trackBuilder.mimeType(track.getString("mimeType"));
      }
      if (track.has("url")) {
        trackBuilder.url(track.getString("url"));
      }
      sideloadedTracks.add(trackBuilder.get());
    }
    return sideloadedTracks;
  }

  @NonNull
  private static LiveConfiguration jsonToLiveConfiguration(
      JSONObject jsonLiveConfiguration, JSONObject jsonAndroidLiveConfiguration
  ) throws JSONException {
    LiveConfiguration.Builder builder = new LiveConfiguration.Builder();

    if (jsonLiveConfiguration == null) {
      jsonLiveConfiguration = new JSONObject();
    }
    if (jsonAndroidLiveConfiguration == null) {
      jsonAndroidLiveConfiguration = new JSONObject();
    }

    if (jsonLiveConfiguration.has("liveEdgeLatencyMs")) {
      builder.liveEdgeLatencyMs(jsonLiveConfiguration.getInt("liveEdgeLatencyMs"));
    }

    if (jsonAndroidLiveConfiguration.has("availabilityStartTimeOffsetOverwriteMs")) {
      builder.availabilityStartTimeOffsetOverwriteMs(
          jsonAndroidLiveConfiguration.getLong("availabilityStartTimeOffsetOverwriteMs")
      );
    }

    if (jsonAndroidLiveConfiguration.has("catchupConfiguration")) {
      builder.catchupConfiguration(
          jsonToCatchupConfiguration(
            jsonAndroidLiveConfiguration.getJSONObject("catchupConfiguration")
          )
      );
    }

    if (jsonAndroidLiveConfiguration.has("customUtcTimingElement")) {
      builder.customUtcTimingElement(
          jsonToCustomUtcTimingElement(
              jsonAndroidLiveConfiguration.getJSONObject("customUtcTimingElement")
          )
      );
    }

    if (
        jsonAndroidLiveConfiguration.has("hlsForcePlaylistUpdateTargetDuration")
        && jsonAndroidLiveConfiguration.has("hlsPlaylistUpdateTargetDurationCoefficient")
    ) {
      builder.hlsPlaylistUpdateTargetDurationCoefficient(
          (float) jsonAndroidLiveConfiguration.getDouble(
              "hlsPlaylistUpdateTargetDurationCoefficient"
          ),
          jsonAndroidLiveConfiguration.getBoolean("hlsForcePlaylistUpdateTargetDuration")
      );
    }

    if (jsonAndroidLiveConfiguration.has("hlsLiveTailSegmentIndex")) {
      builder.hlsLiveTailSegmentIndex(
          jsonAndroidLiveConfiguration.getInt("hlsLiveTailSegmentIndex")
      );
    }
    if (jsonAndroidLiveConfiguration.has("minManifestUpdatePeriodMs")) {
      builder.minManifestUpdatePeriodMs(
          jsonAndroidLiveConfiguration.getLong("minManifestUpdatePeriodMs")
      );
    }
    if (jsonAndroidLiveConfiguration.has("notifyManifestIntervalMs")) {
      builder.notifyManifestIntervalMs(
          jsonAndroidLiveConfiguration.getInt("notifyManifestIntervalMs")
      );
    }
    if (jsonAndroidLiveConfiguration.has("snapToSegmentStart")) {
      builder.snapToSegmentStart(jsonAndroidLiveConfiguration.getBoolean("snapToSegmentStart"));
    }
    if (jsonAndroidLiveConfiguration.has("timeSyncSafetyMs")) {
      builder.timesyncSafetyMs(jsonAndroidLiveConfiguration.getLong("timeSyncSafetyMs"));
    }

    return builder.get();
  }

  @NonNull
  private static CustomUtcTimingElement jsonToCustomUtcTimingElement(
      JSONObject jsonCustomUtcTimingElement
  ) throws JSONException {
    // type and timeReference are mandatory in JavaScript

    return new CustomUtcTimingElement(
      jsonCustomUtcTimingElement.getString("schemeIdUri"),
      jsonCustomUtcTimingElement.getString("value"),
      jsonCustomUtcTimingElement.getBoolean("force")
    );
  }

  @NonNull
  private static CatchupConfiguration jsonToCatchupConfiguration(
      JSONObject jsonCatchupConfiguration
  ) throws JSONException {
    if (!jsonCatchupConfiguration.has("type")) {
      throw new IllegalArgumentException(
          "The type reference is required for the catchup configuration"
      );
    }
    if (!jsonCatchupConfiguration.has("timeReference")) {
      throw new IllegalArgumentException(
          "The time reference is required for the catchup configuration"
      );
    }

    CatchupConfiguration.TimeReference timeReference = null;
    switch (jsonCatchupConfiguration.getString("timeReference")) {
      case "mediaEnd":
        timeReference = CatchupConfiguration.TimeReference.MEDIA_END;
        break;
      case "bufferHead":
        timeReference = CatchupConfiguration.TimeReference.BUFFER_AHEAD;
        break;
      default:
        throw new IllegalArgumentException(
            "The time reference is required for the catchup configuration"
        );
    }

    CatchupConfiguration.Builder builder = new CatchupConfiguration.Builder(timeReference);

    switch (jsonCatchupConfiguration.getString("type")) {
      case "none":
        builder.none();
        break;
      case "seek":
        if (!jsonCatchupConfiguration.has("lowerTimeThresholdMs")) {
          throw new IllegalArgumentException(
              "The lower time threshold is required for the seek type"
          );
        }
        if (!jsonCatchupConfiguration.has("upperTimeThresholdMs")) {
          throw new IllegalArgumentException(
              "The upper time threshold is required for the seek type"
          );
        }
        builder.seek(
            jsonCatchupConfiguration.getLong("lowerTimeThresholdMs"),
            jsonCatchupConfiguration.getLong("upperTimeThresholdMs")
        );
        break;
      case "speed":
        if (!jsonCatchupConfiguration.has("lowerTimeThresholdMs")) {
          throw new IllegalArgumentException(
              "The lower time threshold is required for the speed type"
          );
        }
        if (!jsonCatchupConfiguration.has("upperTimeThresholdMs")) {
          throw new IllegalArgumentException(
              "The upper time threshold is required for the speed type"
          );
        }
        if (!jsonCatchupConfiguration.has("fallbackTimeThresholdMs")) {
          throw new IllegalArgumentException(
              "The fallback time threshold is required for the speed type"
          );
        }
        if (!jsonCatchupConfiguration.has("speedCoefficient")) {
          throw new IllegalArgumentException(
              "The speed coefficient is required for the speed type"
          );
        }
        builder.speed(
            jsonCatchupConfiguration.getLong("lowerTimeThresholdMs"),
            jsonCatchupConfiguration.getLong("upperTimeThresholdMs"),
            jsonCatchupConfiguration.getLong("fallbackTimeThresholdMs"),
            (float) jsonCatchupConfiguration.getDouble("speedCoefficient")
        );
        break;
      default:
        throw new IllegalArgumentException("The type is required for the catchup configuration");
    }

    if (jsonCatchupConfiguration.has("seekOnDownloadError")) {
      builder.shouldSeekOnDownloadError(
          jsonCatchupConfiguration.getBoolean("seekOnDownloadError")
      );
    }

    return builder.get();
  }

  @NonNull
  private static NetworkConfiguration jsonToNetworkConfiguration(JSONObject json)
      throws JSONException {
    NetworkConfiguration.Builder builder = new NetworkConfiguration.Builder();

    if (json.has("manifestAttemptParameters")) {
      final JSONObject jsonObject =
          json.getJSONObject("manifestAttemptParameters");

      if (jsonObject.has("connectionTimeout")) {
        builder.manifestConnectionTimeoutMs(
            jsonObject.getInt("connectionTimeout")
        );
      }
      if (jsonObject.has("readTimeout")) {
        builder.manifestReadTimeoutMs(
            jsonObject.getInt("readTimeout")
        );
      }

      builder.manifestRetryConfiguration(jsonToRetryConfiguration(jsonObject));
    }

    if (json.has("segmentAttemptParameters")) {
      final JSONObject jsonObject =
          json.getJSONObject("segmentAttemptParameters");

      if (jsonObject.has("connectionTimeout")) {
        builder.segmentsConnectionTimeoutMs(
            jsonObject.getInt("connectionTimeout")
        );
      }
      if (jsonObject.has("readTimeout")) {
        builder.segmentsReadTimeoutMs(
            jsonObject.getInt("readTimeout")
        );
      }

      builder.segmentsRetryConfiguration(jsonToRetryConfiguration(jsonObject));
    }

    if (json.has("drmAttemptParameters")) {
      final JSONObject jsonObject =
              json.getJSONObject("drmAttemptParameters");

      if (jsonObject.has("connectionTimeout")) {
        builder.drmConnectionTimeoutMs(
                jsonObject.getInt("connectionTimeout")
        );
      }
      if (jsonObject.has("readTimeout")) {
        builder.drmReadTimeoutMs(
                jsonObject.getInt("readTimeout")
        );
      }
      if (jsonObject.has("acquisitionTimeout")) {
        builder.drmAcquisitionTimeoutMs(
                jsonObject.getInt("acquisitionTimeout")
        );
      }

      builder.drmRetryConfiguration(jsonToRetryConfiguration(jsonObject));
    }

    return builder.get();
  }

  @NonNull
  private static RetryConfiguration jsonToRetryConfiguration(JSONObject json)
      throws JSONException {
    final RetryConfiguration.Builder retryConfiguration = new RetryConfiguration.Builder();

    if (json.has("maxAttempts")) {
      retryConfiguration.maxAttempts(json.getInt("maxAttempts"));
    }
    if (json.has("baseDelay")) {
      retryConfiguration.baseDelayMs(json.getInt("baseDelay"));
    }
    if (json.has("retryCodes")) {
      JSONArray jsonArray = json.getJSONArray("retryCodes");
      int[] retryCodes = new int[jsonArray.length()];
      for (int i = 0; i < jsonArray.length(); i++) {
        retryCodes[i] = jsonArray.getInt(i);
      }
      retryConfiguration.retryCodes(retryCodes);
    }

    return retryConfiguration.get();
  }

  @NonNull
  private static BufferConfiguration jsonToBufferConfiguration(JSONObject json)
      throws JSONException {
    BufferConfiguration.Builder builder = new BufferConfiguration.Builder();


    if (json.has("audioFeedYieldEnabled")) {
      builder.audioFeedYield(json.getBoolean("audioFeedYieldEnabled"));
    }
    if (json.has("backBufferDurationMs")) {
      builder.backBufferDuration(json.getLong("backBufferDurationMs"), TimeUnit.MILLISECONDS);
    }
    if (json.has("bufferSegmentSize")) {
      builder.bufferSegmentSize(json.getInt("bufferSegmentSize"));
    }
    if (json.has("bufferSizeBytes")) {
      builder.bufferSizeBytes(json.getInt("bufferSizeBytes"));
    }
    if (json.has("drainWhileCharging")) {
      builder.drainWhileCharging(json.getBoolean("drainWhileCharging"));
    }
    if (json.has("highMediaTimeMs")) {
      builder.highMediaTime(json.getInt("highMediaTimeMs"), TimeUnit.MILLISECONDS);
    }
    if (json.has("lowMediaTimeMs")) {
      builder.lowMediaTime(json.getInt("lowMediaTimeMs"), TimeUnit.MILLISECONDS);
    }
    if (json.has("minPlaybackStartMs")) {
      builder.minPlaybackStart(json.getInt("minPlaybackStartMs"), TimeUnit.MILLISECONDS);
    }
    if (json.has("minRebufferStartMs")) {
      builder.minRebufferStart(json.getInt("minRebufferStartMs"), TimeUnit.MILLISECONDS);
    }
    if (json.has("prioritizeInstreamOverManifestDuration")) {
      builder.prioritizeInstreamOverManifestDuration(
          json.getBoolean("prioritizeInstreamOverManifestDuration"));
    }
    if (json.has("prioritizeTimeOverSizeThresholds")) {
      builder.prioritizeTimeOverSizeThresholds(
          json.getBoolean("prioritizeTimeOverSizeThresholds"));
    }
    if (json.has("videoFeedYieldEnabled")) {
      builder.videoFeedYield(json.getBoolean("videoFeedYieldEnabled"));
    }

    return builder.get();
  }

  @NonNull
  private static AnalyticsMetaData jsonToAnalyticsMetaData(JSONObject json) throws JSONException {

    boolean live = false;
    String assetId = "Unknown";

    if (json.has("live")) {
      live = json.getBoolean("live");
    }
    if (json.has("assetId")) {
      assetId = json.getString("assetId");
    }

    AnalyticsMetaData analyticsMetaData = new AnalyticsMetaData(live, assetId);

    if (json.has("viewerId")) {
      analyticsMetaData.viewerId = json.getString("viewerId");
    }
    if (json.has("duration")) {
      analyticsMetaData.durationSeconds = json.getInt("duration");
    }

    return analyticsMetaData;
  }

  @NonNull
  private static Bundle jsonToBundle(@Nullable final JSONObject json) throws JSONException {
    final Bundle bundle = new Bundle();
    if (json != null) {
      for (Iterator<String> it = json.keys(); it.hasNext();) {
        String key = it.next();
        bundle.putString(key, json.getString(key));
      }
    }
    return bundle;
  }

  @NonNull
  private static JSONObject bundleToJson(@Nullable final Bundle bundle) throws JSONException {
    JSONObject json = new JSONObject();
    if (bundle != null) {
      for (String key : bundle.keySet()) {
        json.put(key, bundle.getString(key));
      }
    }
    return json;
  }

  @NonNull
  private static Drm stringToDrm(@NonNull String strDrm) throws IllegalArgumentException {
    switch (strDrm) {
      case DRM_SYSTEM_AUTO:
      case DRM_SYSTEM_CASTLABS: {
        return Drm.BestAvailable;
      }
      case DRM_SYSTEM_WIDEVINE: {
        return Drm.Widevine;
      }
      case DRM_SYSTEM_OMA: {
        return Drm.Oma;
      }
      case DRM_SYSTEM_PLAYREADY: {
        return Drm.Playready;
      }
      case DRM_SYSTEM_CLEAR_KEY: {
        return Drm.Clearkey;
      }
      case DRM_SYSTEM_WISE_PLAY: {
        return Drm.Wiseplay;
      }
      case DRM_SYSTEM_FAIRPLAY:
      default: {
        throw new IllegalArgumentException("Unexpected DRM system: " + strDrm);
      }
    }
  }

  private static String environmentToUrl(String environment) throws IllegalArgumentException {
    switch (environment) {
      case "TESTING": return DrmTodayConfiguration.DRMTODAY_TEST;
      case "STAGING": return DrmTodayConfiguration.DRMTODAY_STAGING;
      case "PRODUCTION": return DrmTodayConfiguration.DRMTODAY_PRODUCTION;
      default: throw new IllegalArgumentException("Unexpected environment: " + environment);
    }
  }
}
