package com.rnsdk.clvideo.video;

import android.annotation.SuppressLint;
import android.app.Activity;
import androidx.annotation.NonNull;
import com.castlabs.android.PlayerSDK;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.rnsdk.clvideo.utils.Analytics;

/**
 * The video module to implement the methods of CLVideo component.
 */
public class VideoModule extends ReactContextBaseJavaModule {

  public static final String REACT_CLASS = "VideoModule";

  @SuppressLint("StaticFieldLeak")
  public static Activity activity;

  public VideoModule(ReactApplicationContext context) {
    super(context);
  }

  @Override
  @NonNull
  public String getName() {
    return REACT_CLASS;
  }

  /**
   * Initialize the SDK.
   *
   * @param initData The data to initialize the SDK
   * @param companionSdkInfo The companion SDK descriptor
   * @param promise The promise which is used to send the result
   */
  @ReactMethod
  public void initialize(ReadableMap initData, ReadableMap companionSdkInfo, Promise promise) {
    try {
      if (companionSdkInfo.getString("name") == null)  {
        throw new AssertionError("companionSdkInfo.name cannot be null");
      }
      if (companionSdkInfo.getString("version") == null)  {
        throw new AssertionError("companionSdkInfo.version cannot be null");
      }

      // Register Youbora plugin
      Analytics.registerYouboraPlugin(initData);

      // Get license key
      String key = initData.getMap("licenseKey").getString("android");

      PlayerSDK.COMPANION_SDK_NAME = companionSdkInfo.getString("name");
      PlayerSDK.COMPANION_SDK_VERSION = companionSdkInfo.getString("version");

      // Initialize the SDK
      PlayerSDK.init(getReactApplicationContext(), key);

      // Initialize the activity
      activity = getCurrentActivity();

      promise.resolve("successful");
    } catch (Exception e) {
      promise.reject("error", e.getMessage());
    }
  }

  /**
   * Return the player state.
   *
   * @param instanceId The player ID.
   * @param promise The promise.
   */
  @ReactMethod
  public void getPlayerState(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getPlayerState());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Return the duration of the stream.
   *
   * @param instanceId The player ID.
   * @param promise The promise.
   */
  @ReactMethod
  public void getDuration(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getDuration());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Return true if the stream is live.
   *
   * @param instanceId The player ID.
   * @param promise The promise.
   */
  @ReactMethod
  public void isLive(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.isLive());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Return the playback position.
   *
   * @param instanceId The player ID.
   * @param promise The promise.
   */
  @ReactMethod
  public void getPosition(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getPosition());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Return the playback progress as percentage.
   *
   * @param instanceId The player ID.
   * @param promise The promise.
   */
  @ReactMethod
  public void getProgressPercentage(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getProgressPercentage());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Return the percentage of buffered content.
   *
   * @param instanceId The player ID.
   * @param promise The promise.
   */
  @ReactMethod
  public void getBufferedPercentage(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getBufferedPercentage());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Set a new playback position.
   *
   * @param instanceId The player ID.
   * @param position The new playback position.
   */
  @ReactMethod
  public void setPosition(int instanceId, double position) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.setPosition(position);
    }
  }

  /**
   * Return the player volume.
   *
   * @param instanceId The player ID.
   * @param promise The promise.
   */
  @ReactMethod
  public void getVolume(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getVolume());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Set a new player volume.
   *
   * @param instanceId The player ID.
   * @param volume The new player volume.
   */
  @ReactMethod
  public void setVolume(int instanceId, double volume) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.setVolume(volume);
    }
  }

  /**
   * Return the active video quality.
   *
   * @param instanceId The player ID.
   * @param promise The promise.
   */
  @ReactMethod
  public void getVideoQuality(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getVideoQuality());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Return the video quality mode, i.e. ABR or manual.
   *
   * @param instanceId The player ID.
   * @param promise The promise.
   */
  @ReactMethod
  public void getVideoQualityMode(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getVideoQualityMode());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Set the video quality mode, ABR or manual.
   *
   * @param instanceId The player ID
   * @param index The new video quality mode
   */
  @ReactMethod
  public void setVideoQuality(int instanceId, int index) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.setVideoQuality(index);
    }
  }

  /**
   * Return video qualities.
   *
   * @param instanceId The player ID
   * @param promise The promise.
   */
  @ReactMethod
  public void getVideoQualities(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getVideoQualities());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Return the active audio track.
   *
   * @param instanceId The player ID
   * @param promise The promise
   */
  @ReactMethod
  public void getAudioTrack(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getAudioTrack());
    } else {
      promise.resolve(null);
    }

  }

  /**
   * Set an active audio track.
   *
   * @param instanceId The player ID
   * @param index The active audio track index
   */
  @ReactMethod
  public void setAudioTrack(int instanceId, int index) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.setAudioTrack(index);
    }
  }

  /**
   * Return the audio tracks.
   *
   * @param instanceId The player ID
   * @param promise The promise
   */
  @ReactMethod
  public void getAudioTracks(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getAudioTracks());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Return the active text track.
   *
   * @param instanceId The player ID
   * @param promise The promise.
   */
  @ReactMethod
  public void getSubtitleTrack(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getSubtitleTrack());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Set an active text track.
   *
   * @param instanceId The player ID
   * @param index The active text track index
   */
  @ReactMethod
  public void setSubtitleTrack(int instanceId, int index) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.setSubtitleTrack(index);
    }
  }

  /**
   * Return the text tracks.
   *
   * @param instanceId The player ID
   * @param promise The promise
   */
  @ReactMethod
  public void getSubtitleTracks(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getSubtitleTracks());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Set a new text style.
   *
   * @param instanceId The player ID
   * @param style The new text style
   */
  @ReactMethod
  public void setSubtitleStyle(int instanceId, ReadableMap style) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.setSubtitleStyle(style);
    }
  }

  /**
   * Load a thumbnail for the given position.
   *
   * @param instanceId The player ID
   * @param position The thumbnail position.
   */
  @ReactMethod
  public void getThumbnailData(int instanceId, double position) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.getThumbnailData(position);
    }
  }

  /**
   * Set the thumbnail component.
   *
   * @param instanceId The player ID
   * @param thumbnailId The thumbnail ID
   */
  @ReactMethod
  public void setThumbnail(int instanceId, int thumbnailId) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.setThumbnail(thumbnailId);
    }
  }

  /**
   * Return the live start time.
   *
   * @param instanceId The player ID
   * @param promise The promise
   */
  @ReactMethod
  public void getLiveStartTime(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getLiveStartTime());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Return if picture in picture is possible.
   *
   * @param instanceId The plauer ID
   * @param promise The promise
   */
  @ReactMethod
  public void isPictureInPicturePossible(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.isPictureInPicturePossible());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Enable or disable picture in picture.
   *
   * @param instanceId The player ID
   */
  @ReactMethod
  public void togglePictureInPicture(int instanceId) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.togglePictureInPicture();
    }
  }

  /**
   * Return statistics and information about the current state of the player.
   *
   * @param instanceId The player ID
   * @param promise The promise
   */
  @ReactMethod
  public void getStats(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getStats());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Play the video.
   *
   * @param instanceId The player ID
   * @param promise The promise
   */
  @ReactMethod
  public void play(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.play();
    }
    promise.resolve(null);
  }

  /**
   * Pause the video.
   *
   * @param instanceId The player ID
   * @param promise The promise
   */
  @ReactMethod
  public void pause(int instanceId, Promise promise) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.pause();
    }
    promise.resolve(null);
  }

  /**
   * Play the playlist item with the given index.
   *
   * @param instanceId The player ID
   * @param index The playlist item index
   * @param promise The promise
   */
  @ReactMethod
  public void playPlaylistItem(
      final int instanceId,
      final int index,
      final @NonNull Promise promise
  ) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.playPlaylistItem(index);
    }
    promise.resolve(null);
  }

  /**
   * Add a new playlist item.
   *
   * @param instanceId The player ID
   * @param jsPlayerConfiguration The player configuration from the item
   * @param index The playlist item index
   * @param promise The promise
   */
  @ReactMethod
  public void addPlaylistItem(
      final int instanceId,
      final @NonNull String jsPlayerConfiguration,
      final int index,
      final @NonNull Promise promise
  ) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.addPlaylistItem(index, jsPlayerConfiguration);
    }
    promise.resolve(null);
  }

  /**
   * Remove a playlist item.
   *
   * @param instanceId The player ID
   * @param index The playlist item index
   * @param promise The promise
   */
  @ReactMethod
  public void removePlaylistItem(
      final int instanceId,
      final int index,
      final @NonNull Promise promise
  ) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.removePlaylistItem(index);
    }
    promise.resolve(null);
  }

  /**
   * Returns speed of the playback.
   *
   * @param instanceId The player ID
   * @param promise The promise
   */
  @ReactMethod
  public void getPlaybackRate(
      final int instanceId,
      final @NonNull Promise promise
  ) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      promise.resolve(videoView.getPlaybackRate());
    } else {
      promise.resolve(null);
    }
  }

  /**
   * Sets speed of the playback, where 1 means "normal" speed.
   *
   * @param instanceId The player ID
   * @param newPlaybackRate The new playback rate, where 1 means "normal" speed.
   * @param promise The promise
   */
  @ReactMethod
  public void setPlaybackRate(
      final int instanceId,
      final double newPlaybackRate,
      final @NonNull Promise promise
  ) {
    VideoView videoView = VideoInstanceManager.getInstance(instanceId);
    if (videoView != null) {
      videoView.setPlaybackRate(newPlaybackRate);
    }
    promise.resolve(null);
  }
}
