package com.rnsdk.clvideo.video;

import android.content.Context;
import android.graphics.Bitmap;
import android.util.Log;
import androidx.annotation.Nullable;
import com.castlabs.android.player.DashDescriptor;
import com.castlabs.android.player.PlayerController;
import com.castlabs.android.player.models.AudioTrack;
import com.castlabs.android.player.models.SubtitleTrack;
import com.castlabs.android.player.models.Track;
import com.castlabs.android.player.models.VideoTrackQuality;
import com.castlabs.subtitles.presentation.SubtitlesStyle;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import java.util.List;

/**
 * Utility class for video module.
 */
public class VideoUtils {

  private static final String TAG = "VideoUtils";

  /**
   * Converts {@link VideoTrackQuality} to {@link WritableMap}.
   *
   * @param videoQuality The {@link VideoTrackQuality} to convert
   * @return the map
   */
  public static WritableMap videoQualityToMap(@Nullable VideoTrackQuality videoQuality) {

    int bitrate = -1;
    int width = -1;
    int height = -1;
    String codecs = null;
    String mimeType = null;

    if (videoQuality != null) {
      bitrate = videoQuality.getBitrate();
      width = videoQuality.getWidth();
      height = videoQuality.getHeight();
      codecs = videoQuality.getCodecs();
      mimeType = videoQuality.getMimeType();
    }

    WritableMap quality = Arguments.createMap();
    quality.putInt("bitrate", bitrate);
    quality.putInt("width", width);
    quality.putInt("height", height);
    quality.putString("codecs", codecs);
    quality.putString("mimeType", mimeType);
    return quality;
  }

  /**
   * Converts {@link AudioTrack} to {@link WritableMap}.
   *
   * @param audioTrack The {@link AudioTrack} to convert
   * @return the map
   */
  public static WritableMap audioTrackToMap(@Nullable AudioTrack audioTrack) {

    int bitrate = -1;
    String label = null;
    String language = null;
    String codecs = null;
    String mimeType = null;

    if (audioTrack != null) {
      bitrate = audioTrack.getBitrate();
      if (bitrate == -1) {
        bitrate = 0;
      }
      label = audioTrack.getLabel();
      language = audioTrack.getLanguage();
      codecs = audioTrack.getCodecs();
      mimeType = audioTrack.getMimeType();
    }

    WritableMap track = Arguments.createMap();
    track.putInt("bitrate", bitrate);
    track.putString("label", label);
    track.putString("language", language);
    track.putString("codecs", codecs);
    track.putString("mimeType", mimeType);
    track.putMap("descriptors", getDescriptors(audioTrack));
    return track;
  }

  /**
   * Converts {@link SubtitleTrack} to {@link WritableMap}.
   *
   * @param subtitleTrack The {@link SubtitleTrack} to convert
   * @return the map
   */
  public static WritableMap subtitleTrackToMap(@Nullable SubtitleTrack subtitleTrack) {

    String label = null;
    String language = null;
    String mimeType = null;

    if (subtitleTrack != null) {
      label = subtitleTrack.getLabel();
      language = subtitleTrack.getLanguage();
      mimeType = subtitleTrack.getMimeType();
    }

    WritableMap track = Arguments.createMap();
    track.putString("label", label);
    track.putString("language", language);
    track.putString("mimeType", mimeType);
    track.putMap("descriptors", getDescriptors(subtitleTrack));
    return track;
  }

  /**
   * Derives the descriptors form the input track.
   *
   * @param track The input track
   * @return the descriptors as {@link WritableMap}
   */
  public static WritableMap getDescriptors(Track track) {
    WritableMap descriptors = Arguments.createMap();

    if (track == null || track.getDescriptors() == null) {
      return descriptors;
    }

    for (DashDescriptor src : track.getDescriptors()) {
      String type = null;
      switch (src.getType()) {
        case DashDescriptor.ROLE:
          type = "DASH_ROLE";
          break;
        case DashDescriptor.ACCESSIBILITY:
          type = "DASH_ACCESSIBILITY";
          break;
        default:
          break;
      }

      if (type != null) {
        WritableMap descriptor = Arguments.createMap();

        descriptor.putString("schemaIdUri", src.getSchemeIdUri());

        if (src.getId() != null) {
          descriptor.putString("id", src.getId());
        }
        if (src.getValue() != null) {
          descriptor.putString("value", src.getValue());
        }

        descriptors.putMap(type, descriptor);
      }
    }

    return descriptors;
  }

  /**
   * Converts {@link ReadableMap} to {@link SubtitlesStyle}.
   *
   * @param context The context
   * @param newStyle The map of subtitle style which should be converted
   * @param currentStyle The current subtitle style of the player
   * @return the converted subtitle style
   */
  public static SubtitlesStyle mapToSubtitlesStyle(Context context, ReadableMap newStyle,
                                                   @Nullable SubtitlesStyle currentStyle) {

    int defaultFontSize = 50;
    int defaultForegroundColor = SubtitlesStyle.DEFAULT_FOREGROUND_COLOR;
    int defaultBackgroundColor = SubtitlesStyle.DEFAULT_BACKGROUND_COLOR;
    float defaultBackgroundOpacity = 1.0f;
    float defaultForegroundOpacity = 1.0f;
    int defaultLeftMargin = SubtitlesStyle.DEFAULT_LEFT_MARGIN;
    int defaultRightMargin = SubtitlesStyle.DEFAULT_RIGHT_MARGIN;
    int defaultTopMargin = SubtitlesStyle.DEFAULT_TOP_MARGIN;
    int defaultBottomMargin = SubtitlesStyle.DEFAULT_BOTTOM_MARGIN;

    if (currentStyle != null) {
      defaultFontSize = currentStyle.fontSize;
      defaultForegroundColor = currentStyle.foregroundColor;
      defaultBackgroundColor = currentStyle.backgroundColor;
      defaultLeftMargin = currentStyle.leftMargin;
      defaultRightMargin = currentStyle.rightMargin;
      defaultTopMargin = currentStyle.topMargin;
      defaultBottomMargin = currentStyle.bottomMargin;
    }

    int fontSize = getIntFromMap(newStyle, "fontSize", defaultFontSize);
    int foregroundColor = getIntFromMap(newStyle, "foregroundColor", defaultForegroundColor);
    int backgroundColor = getIntFromMap(newStyle, "backgroundColor", defaultBackgroundColor);
    float backgroundOpacity = (float) getDoubleFromMap(
        newStyle, "backgroundOpacity", defaultBackgroundOpacity);
    float foregroundOpacity = (float) getDoubleFromMap(
        newStyle, "foregroundOpacity", defaultForegroundOpacity);
    int leftMargin = getIntFromMap(newStyle, "leftMargin", defaultLeftMargin);
    int rightMargin = getIntFromMap(newStyle, "rightMargin", defaultRightMargin);
    int topMargin = getIntFromMap(newStyle, "topMargin", defaultTopMargin);
    int bottomMargin = getIntFromMap(newStyle, "bottomMargin", defaultBottomMargin);

    return SubtitlesStyle.Builder
        .create(context, true)
        .fontSize(fontSize, true)
        .foregroundColor(foregroundColor, true)
        .backgroundColor(backgroundColor, true)
        .foregroundOpacity(foregroundOpacity)
        .backgroundOpacity(backgroundOpacity)
        .leftMargin(leftMargin)
        .rightMargin(rightMargin)
        .topMargin(topMargin)
        .bottomMargin(bottomMargin)
        .get();
  }

  /**
   * Returns the index of the given {@link VideoTrackQuality} in the
   * {@link PlayerController#getVideoQualities()} array.
   *
   * @param playerController The instance of {@link PlayerController}
   * @param videoTrackQuality The {@link VideoTrackQuality}
   * @return the index
   */
  public static int getVideoQualityIndex(PlayerController playerController,
                                         VideoTrackQuality videoTrackQuality) {
    if (playerController == null || videoTrackQuality == null) {
      return -1;
    }
    List<VideoTrackQuality> videoTrackQualities = playerController.getVideoQualities();
    int index = videoTrackQualities.indexOf(videoTrackQuality);
    if (index != -1) {
      return index;
    }
    for (int i = 0; i < videoTrackQualities.size(); i++) {
      if (videoTrackQuality.getBitrate() == videoTrackQualities.get(i).getBitrate()) {
        return i;
      }
    }
    return -1;
  }

  /**
   * Returns the index of the given {@link AudioTrack} in the
   * {@link PlayerController#getAudioTracks()} array.
   *
   * @param playerController The instance of {@link PlayerController}
   * @param audioTrack The {@link AudioTrack}
   * @return the index
   */
  public static int getAudioTrackIndex(PlayerController playerController, AudioTrack audioTrack) {
    if (playerController == null || audioTrack == null) {
      return -1;
    }
    List<AudioTrack> audioTracks = playerController.getAudioTracks();
    int index = audioTracks.indexOf(audioTrack);
    if (index != -1) {
      return index;
    }
    for (int i = 0; i < audioTracks.size(); i++) {
      if (audioTrack.getLanguage() != null
          && audioTrack.getLanguage().equals(audioTracks.get(i).getLanguage())) {
        return i;
      }
    }
    return -1;
  }

  /**
   * Returns the index of the given {@link SubtitleTrack} in the
   * {@link PlayerController#getSubtitleTracks()} array.
   *
   * @param playerController The instance of {@link PlayerController}
   * @param subtitleTrack The {@link SubtitleTrack}
   * @return the index
   */
  public static int getSubtitleTrackIndex(PlayerController playerController,
                                          SubtitleTrack subtitleTrack) {
    if (playerController == null || subtitleTrack == null) {
      return -1;
    }
    List<SubtitleTrack> subtitleTracks = playerController.getSubtitleTracks();
    int index = subtitleTracks.indexOf(subtitleTrack);
    if (index != -1) {
      return index;
    }
    for (int i = 0; i < subtitleTracks.size(); i++) {
      if (subtitleTrack.getLanguage() != null
          && subtitleTrack.getLanguage().equals(subtitleTracks.get(i).getLanguage())) {
        return i;
      }
    }
    return -1;
  }

  /**
   * Converts a {@link Bitmap} to {@link WritableArray} with the values of the bitmap pixels.
   *
   * @param bitmap The bitmap
   * @return the array of pixels
   */
  public static ReadableArray bitmapToArray(Bitmap bitmap) {
    int w = bitmap.getWidth();
    int h = bitmap.getHeight();
    int[] pixels = new int[w * h];
    bitmap.getPixels(pixels, 0, w, 0, 0, w, h);
    WritableArray out = Arguments.createArray();
    for (int pixel : pixels) {
      out.pushInt(pixel);
    }
    return out;
  }

  private static int getIntFromMap(ReadableMap map, String key, int defaultValue) {
    int out = defaultValue;
    try {
      out = map.getInt(key);
    } catch (Exception e) {
      Log.i(TAG, "No value found for " + key);
    }
    return out;
  }

  private static double getDoubleFromMap(ReadableMap map, String key, double defaultValue) {
    double out = defaultValue;
    try {
      out = map.getDouble(key);
    } catch (Exception e) {
      Log.i(TAG, "No value found for " + key);
    }
    return out;
  }
}
