package com.rnsdk.clvideo.video;

import static com.castlabs.android.SdkConsts.VIDEO_QUALITY_ADAPTIVE;

import android.app.Activity;
import android.app.PictureInPictureParams;
import android.content.Context;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.Rect;
import android.os.Build;
import android.os.Bundle;
import android.util.AttributeSet;
import android.util.Log;
import android.util.Rational;
import android.view.View;
import android.widget.FrameLayout;
import androidx.annotation.ChecksSdkIntAtLeast;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import com.castlabs.android.PlayerSDK;
import com.castlabs.android.player.AbstractPlayerListener;
import com.castlabs.android.player.PlayerConfig;
import com.castlabs.android.player.PlayerController;
import com.castlabs.android.player.PlayerListener;
import com.castlabs.android.player.PlayerView;
import com.castlabs.android.player.SingleControllerPlaylist;
import com.castlabs.android.player.SingleControllerPlaylist.PlaylistListener;
import com.castlabs.android.player.VideoRendererListener;
import com.castlabs.android.player.exceptions.CastlabsPlayerException;
import com.castlabs.android.player.models.AudioTrack;
import com.castlabs.android.player.models.SubtitleTrack;
import com.castlabs.android.player.models.VideoTrackQuality;
import com.castlabs.sdk.mediasession.MediaSessionPlugin;
import com.castlabs.sdk.thumbs.ThumbnailProvider;
import com.castlabs.sdk.thumbs.ThumbsPlugin;
import com.castlabs.subtitles.presentation.SubtitlesStyle;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.uimanager.events.RCTEventEmitter;
import com.rnsdk.clvideo.R;
import com.rnsdk.clvideo.thumbnail.ThumbnailInstanceManager;
import com.rnsdk.clvideo.thumbnail.ThumbnailView;
import com.rnsdk.clvideo.utils.AbstractVideoRendererListener;
import com.rnsdk.clvideo.utils.JsonUtils;
import java.util.ArrayList;
import java.util.List;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;


/**
 * The view component for CLVideo.
 */
public class VideoView extends FrameLayout {
  private static final String TAG = "VideoView";

  private PlayerController playerController;

  private FrameLayout mainView;
  private PlayerView playerView;

  private VideoRendererListener videoRendererListener;
  private PlayerListener playerListener;
  private PlaylistListener playlistListener;

  private ThumbnailView thumbnailView;
  private ThumbnailProvider.Callback thumbnailCallback;
  private Runnable measureAndLayout;

  private boolean createEventSent = false;
  private int droppedFrames = 0;

  /**
   * Creates a new instance of VideoView.
   *
   * @param context The context.
   * @param attrs   The attributes.
   */
  public VideoView(@NonNull Context context, @Nullable AttributeSet attrs) {
    super(context, attrs);

    initMainView();
    initPlayerView();
    initVideoRendererListener();
    initPlayerListener();
    initPlaylistListener();
    initThumbnailCallback();
  }

  private void initMainView() {
    mainView = (FrameLayout) View.inflate(
        getContext(),
        R.layout.view_video,
        null
    );
    addView(mainView);
  }

  private void initPlayerView() {
    playerView = mainView.findViewById(R.id.playerView);
  }

  private void initVideoRendererListener() {
    videoRendererListener = new AbstractVideoRendererListener() {
      @Override
      public void onDroppedFrames(int count, long elapsedMs) {
        droppedFrames = count;
      }
    };
  }

  private void initPlayerListener() {
    measureAndLayout = () -> {
      measure(
          MeasureSpec.makeMeasureSpec(getWidth(), MeasureSpec.EXACTLY),
          MeasureSpec.makeMeasureSpec(getHeight(), MeasureSpec.EXACTLY)
      );
      layout(getLeft(), getTop(), getRight(), getBottom());
    };

    playerListener = new AbstractPlayerListener() {
      @Override
      public void onStateChanged(@NonNull PlayerController.State state) {
        super.onStateChanged(state);
        onStateChangedEvent(state);

        if (state == PlayerController.State.Preparing) {
          post(measureAndLayout);
        }
      }

      @Override
      public void onError(@NonNull CastlabsPlayerException error) {
        super.onError(error);
        onErrorEvent(error);
      }

      @Override
      public void onSeekTo(long newPosition) {
        super.onSeekTo(newPosition);
        onSeekToEvent(newPosition);
      }

      @Override
      public void onSeekCompleted() {
        super.onSeekCompleted();
        onSeekCompletedEvent();
      }

      @Override
      public void onFullyBuffered() {
        super.onFullyBuffered();
        onFullyBufferedEvent();
      }

      @Override
      public void onPlayerModelChanged() {
        super.onPlayerModelChanged();
        onPlayerModelChangedEvent();
      }

      @Override
      public void onVideoSizeChanged(int width, int height, float pixelWidthHeightRatio) {
        super.onVideoSizeChanged(width, height, pixelWidthHeightRatio);
        post(measureAndLayout);
      }
    };
  }

  private void initPlaylistListener() {
    playlistListener = new PlaylistListener() {
      @Override
      public void onItemChange(@NonNull PlayerConfig config) {
        if (isNotPlaylist()) {
          return;
        }

        final SingleControllerPlaylist playlistController = getPlaylistController();

        final int currentIndex = playlistController.getCurrentItemIndex();
        onPlaylistItemChangedEvent(currentIndex);
      }

      @Override
      public void onPlaylistEnded() {
        onPlaylistEndedEvent();
      }
    };
  }

  private void initPlayerController() {
    if (playerController == null) {
      return;
    }

    playerController.addPlayerListener(playerListener);
    playerController.addVideoRendererListener(videoRendererListener);
    MediaSessionPlugin.enableMediaSession(playerController);
  }

  private void destroyPlayerController() {
    if (playerController == null) {
      return;
    }

    MediaSessionPlugin.disableMediaSession(playerController);
    playerController.removePlayerListener(playerListener);
    playerController.removeVideoRendererListener(videoRendererListener);
    playerController.destroy();
    playerController = null;
  }

  private boolean isPlaylist() {
    return playerController instanceof SingleControllerPlaylist;
  }

  private boolean isNotPlaylist() {
    return !isPlaylist();
  }

  private SingleControllerPlaylist getPlaylistController() {
    if (playerController instanceof SingleControllerPlaylist) {
      return (SingleControllerPlaylist) playerController;
    } else {
      throw new IllegalStateException("Playlist not in use.");
    }
  }

  private void open(
      @Nullable final PlayerConfig playerConfig,
      final int secondaryDisplay
  ) {
    if (isPlaylist()) {
      destroyPlayerController();
    }

    if (playerController == null) {
      playerController = new PlayerController(getContext());
      initPlayerController();
    }

    playerController.release();
    playerController.setSecondaryDisplay(secondaryDisplay);

    playerView.setPlayerController(playerController);

    resetStats();

    if (playerConfig == null) {
      return;
    }

    ReactContext reactContext = (ReactContext) getContext();
    Activity activity = reactContext.getCurrentActivity();
    if (activity != null) {
      activity.runOnUiThread(() -> {
        playerController.open(playerConfig);
      });
    }
  }

  private void openPlaylist(
      final int startAt,
      @NonNull final List<PlayerConfig> playerConfigs,
      final int secondaryDisplay
  ) {
    if (isNotPlaylist()) {
      destroyPlayerController();
    }

    if (playerController == null) {
      playerController = new SingleControllerPlaylist(getContext(), playlistListener);
      initPlayerController();
    }

    playerController.release();
    playerController.setSecondaryDisplay(secondaryDisplay);

    playerView.setPlayerController(playerController);

    resetStats();

    ReactContext reactContext = (ReactContext) getContext();
    Activity activity = reactContext.getCurrentActivity();
    if (activity != null) {
      activity.runOnUiThread(() -> {
        ((SingleControllerPlaylist) playerController).open(startAt, playerConfigs);
      });
    }
  }

  private void initThumbnailCallback() {
    thumbnailCallback = (
      requestedPositionUs, actualPositionUs, thumbnail
    ) -> {
      double requestedPosition = requestedPositionUs / 1000000.0;
      double actualPosition = actualPositionUs / 1000000.0;
      onThumbnailLoaded(requestedPosition, actualPosition, thumbnail);
    };
  }

  /**
   * Use for react property: mediaLoadRequest.
   */
  public void setMediaLoadRequest(@NonNull final String jsMediaLoadRequest) {
    try {
      final JSONObject json = new JSONObject(jsMediaLoadRequest);

      final boolean usePlaylist = json.getBoolean("usePlaylist");
      final int startAt = json.getInt("startAt");
      final JSONArray items = json.getJSONArray("items");

      final List<PlayerConfig> playerConfigs = new ArrayList<>();
      for (int i = 0; i < items.length(); i++) {
        final JSONObject item = items.getJSONObject(i);
        playerConfigs.add(JsonUtils.buildFromJsonConfig(item));
      }

      final int secondaryDisplay = JsonUtils.jsonToSecondaryDisplay(items);

      if (usePlaylist) {
        openPlaylist(startAt, playerConfigs, secondaryDisplay);
      } else {
        open(playerConfigs.get(0), secondaryDisplay);
      }
    } catch (JSONException e) {
      Log.e(TAG, e.getMessage());
    }
  }

  /**
   * Play the playlist item with the given index.
   *
   * @param index The index of the item to play.
   */
  public void playPlaylistItem(final int index) {
    if (isNotPlaylist()) {
      return;
    }

    final SingleControllerPlaylist playlistController = getPlaylistController();
    playlistController.playItem(index);
  }

  /**
   * Add a new playlist item.
   *
   * @param index                 The index at which to add the item.
   * @param jsPlayerConfiguration The player configuration for the item.
   */
  public void addPlaylistItem(
      final int index,
      final @NonNull String jsPlayerConfiguration
  ) {
    if (isNotPlaylist()) {
      return;
    }

    try {
      final SingleControllerPlaylist playlistController = getPlaylistController();

      final JSONObject json = new JSONObject(jsPlayerConfiguration);
      final PlayerConfig playerConfig = JsonUtils.buildFromJsonConfig(json);
      playlistController.addItem(index, playerConfig);
    } catch (JSONException e) {
      Log.e(TAG, e.getMessage());
    }
  }

  /**
   * Remove the playlist item with the given index.
   *
   * @param index The index of the item to remove.
   */
  public void removePlaylistItem(final int index) {
    if (isNotPlaylist()) {
      return;
    }

    final SingleControllerPlaylist playlistController = getPlaylistController();
    playlistController.removeItem(playlistController.getPlaylist().get(index));
  }

  /**
   * Use for react method: getPlayerState.
   */
  public int getPlayerState() {
    return playerController.getPlayerState().ordinal();
  }

  /**
   * Use for react method: getDuration.
   */
  public double getDuration() {
    return playerController.getDuration() / 1000000.0;
  }

  /**
   * Use for react method: isLive.
   */
  public boolean isLive() {
    return playerController.isLive();
  }

  /**
   * Use for react method: getPosition.
   */
  public double getPosition() {
    return playerController.getPosition() / 1000000.0;
  }

  /**
   * Use for react method: getProgressPercentage.
   */
  public int getProgressPercentage() {
    long duration = playerController.getDuration();
    long position = playerController.getPosition();
    if (duration == -1) {
      return 0;
    }
    return (int) (position * 100 / duration);
  }

  /**
   * Use for react method: getBufferedPercentage.
   */
  public int getBufferedPercentage() {
    return playerController.getBufferedPercentage();
  }

  /**
   * Use for react method: setPosition.
   */
  public void setPosition(double position) {
    playerController.setPosition((long) (position * 1000000.0));
  }

  /**
   * Use for react method: getVolume.
   */
  public double getVolume() {
    return playerController.getVolume();
  }

  /**
   * Use for react method: setVolume.
   */
  public void setVolume(double volume) {
    playerController.setVolume((float) volume);
  }

  /**
   * Use for react method: getVideoQuality.
   */
  public ReadableMap getVideoQuality() {
    VideoTrackQuality videoQuality = playerController.getVideoQuality();
    return VideoUtils.videoQualityToMap(videoQuality);
  }

  /**
   * Use for react method: getLiveStartTime.
   */
  public double getLiveStartTime() {
    return playerController.getLiveStartTime() / 1000000.0;
  }

  /**
   * Use for react method: getVideoQualityMode.
   */
  public String getVideoQualityMode() {
    int mode = playerController.getVideoQualityMode();
    if (mode == VIDEO_QUALITY_ADAPTIVE) {
      return "adaptive";
    } else {
      return "manual";
    }
  }

  /**
   * Use for react method: setVideoQuality.
   */
  public void setVideoQuality(int index) {
    if (index == -1) {
      playerController.setVideoQuality(null);
    } else {
      VideoTrackQuality videoQuality = playerController.getVideoQualities().get(index);
      playerController.setVideoQuality(videoQuality);
    }
  }

  /**
   * Use for react method: getVideoQualities.
   */
  public ReadableArray getVideoQualities() {
    WritableArray qualities = Arguments.createArray();
    List<VideoTrackQuality> videoQualities = playerController.getVideoQualities();
    for (int i = 0; i < videoQualities.size(); i++) {
      WritableMap quality = VideoUtils.videoQualityToMap(videoQualities.get(i));
      qualities.pushMap(quality);
    }
    return qualities;
  }

  /**
   * Use for react method: getAudioTrack.
   */
  public ReadableMap getAudioTrack() {
    AudioTrack audioTrack = playerController.getAudioTrack();
    return VideoUtils.audioTrackToMap(audioTrack);
  }

  /**
   * Use for react method: setAudioTrack.
   */
  public void setAudioTrack(int index) {
    if (index == -1) {
      playerController.setAudioTrack(null);
    } else {
      AudioTrack audioTrack = playerController.getAudioTracks().get(index);
      playerController.setAudioTrack(audioTrack);
    }
  }

  /**
   * Use for react method: getAudioTracks.
   */
  public ReadableArray getAudioTracks() {
    WritableArray tracks = Arguments.createArray();
    List<AudioTrack> audioTracks = playerController.getAudioTracks();
    for (int i = 0; i < audioTracks.size(); i++) {
      WritableMap track = VideoUtils.audioTrackToMap(audioTracks.get(i));
      tracks.pushMap(track);
    }
    return tracks;
  }

  /**
   * Use for react method: getSubtitleTrack.
   */
  public ReadableMap getSubtitleTrack() {
    SubtitleTrack subtitleTrack = playerController.getSubtitleTrack();
    return VideoUtils.subtitleTrackToMap(subtitleTrack);
  }

  /**
   * Use for react method: setSubtitleTrack.
   */
  public void setSubtitleTrack(int index) {
    if (index == -1) {
      playerController.setSubtitleTrack(null);
    } else {
      SubtitleTrack subtitleTrack = playerController.getSubtitleTracks().get(index);
      playerController.setSubtitleTrack(subtitleTrack);
    }
  }

  /**
   * Use for react method: getSubtitleTracks.
   */
  public ReadableArray getSubtitleTracks() {
    WritableArray tracks = Arguments.createArray();
    List<SubtitleTrack> subtitleTracks = playerController.getSubtitleTracks();
    for (int i = 0; i < subtitleTracks.size(); i++) {
      WritableMap track = VideoUtils.subtitleTrackToMap(subtitleTracks.get(i));
      tracks.pushMap(track);
    }
    return tracks;
  }

  /**
   * Use for react method: setSubtitleStyle.
   */
  public void setSubtitleStyle(ReadableMap style) {
    SubtitlesStyle subtitlesStyle = VideoUtils.mapToSubtitlesStyle(
        getContext(), style, playerController.getSubtitlesStyle());
    playerController.setSubtitlesStyle(subtitlesStyle);
  }

  /**
   * Use for react method: getThumbnailData.
   */
  public void getThumbnailData(double position) {
    ThumbnailProvider provider = playerController.getComponent(ThumbnailProvider.class);
    if (provider != null) {
      long positionUs = (long) (position * 1000000.0);
      int thumbnailIndex = ThumbsPlugin.THUMBNAIL_INDEX_CURRENT;
      provider.getThumbnail(positionUs, thumbnailCallback, thumbnailIndex);
    }
  }

  /**
   * Use for react method: setThumbnail.
   */
  public void setThumbnail(int thumbnailId) {
    thumbnailView = ThumbnailInstanceManager.getInstance(thumbnailId);
  }

  /**
   * Use for react method: isPictureInPicturePossible.
   */
  @ChecksSdkIntAtLeast(api = Build.VERSION_CODES.N)
  public boolean isPictureInPicturePossible() {
    PackageManager packageManager = getContext().getPackageManager();
    return Build.VERSION.SDK_INT >= Build.VERSION_CODES.N
        && packageManager.hasSystemFeature(PackageManager.FEATURE_PICTURE_IN_PICTURE);
  }

  /**
   * Use for react method: togglePictureInPicture.
   */
  public void togglePictureInPicture() {
    Activity activity = ((ReactContext) getContext()).getCurrentActivity();
    if (playerController != null && activity != null
        && Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
      // Set PiP params if supported
      if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
        PictureInPictureParams.Builder builder = new PictureInPictureParams.Builder();
        Rect sourceRectHint = new Rect();
        getGlobalVisibleRect(sourceRectHint);
        if (playerController.getVideoQuality() != null) {
          int w = playerController.getVideoQuality().getWidth();
          int h = playerController.getVideoQuality().getHeight();
          builder.setSourceRectHint(getVideoRect(w, h, sourceRectHint));
          builder.setAspectRatio(new Rational(w, h));
        } else {
          builder.setSourceRectHint(sourceRectHint);
          builder.setAspectRatio(new Rational(16, 9));
        }
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
          builder.setAutoEnterEnabled(false);
          builder.setSeamlessResizeEnabled(true);
        }
        activity.setPictureInPictureParams(builder.build());
      }
      // Start PiP mode. (On Android there is native UI buttons to return
      // back to the maximum size)
      activity.enterPictureInPictureMode();
    }
  }

  /**
   * Use for react method: getStats.
   */
  public ReadableMap getStats() {
    WritableMap stats = Arguments.createMap();

    stats.putDouble("droppedFrames", droppedFrames);

    if (playerController.getVideoQuality() != null) {
      final int newStreamBandwidth = playerController.getVideoQuality().getBitrate();
      if (newStreamBandwidth > 0) {
        stats.putDouble("streamBandwidth", newStreamBandwidth);
      }
    }

    final long newEstimatedBandwidth = playerController.getBitrateEstimate();
    if (newEstimatedBandwidth > 0) {
      stats.putDouble("estimatedBandwidth", newEstimatedBandwidth);
    }

    return stats;
  }

  public double getPlaybackRate() {
    return playerController.getSpeed();
  }

  public void setPlaybackRate(double newPlaybackRate) {
    playerController.setSpeed((float) newPlaybackRate);
  }

  public void play() {
    playerController.play();
  }

  public void pause() {
    playerController.pause();
  }

  private Rect getVideoRect(int videoWidth, int videoHeight, Rect mainRect) {

    float videoRatio = (float) videoWidth / videoHeight;
    float mainRatio = (float) getWidth() / getHeight();

    if (videoRatio > mainRatio) {
      int left = mainRect.left;
      int right = mainRect.right;
      int margin = (int) ((getHeight() - getWidth() / videoRatio) / 2);
      int top = mainRect.top + margin;
      int bottom = mainRect.bottom - margin;
      return new Rect(left, top, right, bottom);
    } else {
      int top = mainRect.top;
      int bottom = mainRect.bottom;
      int margin = (int) ((getWidth() - getHeight() * videoRatio) / 2);
      int left = mainRect.left + margin;
      int right = mainRect.right - margin;
      return new Rect(left, top, right, bottom);
    }
  }

  /**
   * Releases the resources.
   */
  public void release() {
    playerView.setKeepScreenOn(false);
    destroyPlayerController();
    VideoInstanceManager.removeInstance(getId());
  }

  /**
   * Use for react event: onViewCreated.
   */
  private void onViewCreatedEvent() {
    VideoInstanceManager.addInstance(getId(), this);
    WritableMap event = Arguments.createMap();
    event.putInt("instanceId", getId());
    createEvent(event, VideoViewManager.EVENT_VIEW_CREATED);
  }

  /**
   * Use for react event: onStateChanged.
   */
  private void onStateChangedEvent(PlayerController.State state) {
    WritableMap event = Arguments.createMap();
    event.putInt("playbackState", state.ordinal());
    createEvent(event, VideoViewManager.EVENT_STATE_CHANGED);
  }

  /**
   * Use for react event: onError.
   */
  private void onErrorEvent(CastlabsPlayerException error) {
    WritableMap event = Arguments.createMap();
    event.putInt("severity", error.getSeverity());
    event.putInt("type", error.getType());
    event.putString("message", error.getCauseMessage());

    Bundle errorData = error.getErrorData();
    event.putInt("httpCode", errorData.getInt("httpCode", 0));
    event.putString("url", errorData.getString("url", null));
    event.putString("responseBody", errorData.getString("responseBody", null));

    createEvent(event, VideoViewManager.EVENT_ERROR);
  }

  /**
   * Use for react event: onSeekTo.
   */
  private void onSeekToEvent(long newPosition) {
    WritableMap event = Arguments.createMap();
    event.putDouble("newPosition", newPosition / 1000000.0);
    createEvent(event, VideoViewManager.EVENT_SEEK_TO);
  }

  /**
   * Use for react event: onSeekCompleted.
   */
  private void onSeekCompletedEvent() {
    createEvent(null, VideoViewManager.EVENT_SEEK_COMPLETED);
  }

  /**
   * Use for react event: onFullyBuffered.
   */
  private void onFullyBufferedEvent() {
    createEvent(null, VideoViewManager.EVENT_FULLY_BUFFERED);
  }

  /**
   * Use for react event: onPlayerModelChanged.
   */
  private void onPlayerModelChangedEvent() {
    createEvent(null, VideoViewManager.EVENT_PLAYER_MODEL_CHANGED);
  }

  /**
   * Use for react event: onPlaylistItemChanged.
   */
  private void onPlaylistItemChangedEvent(int index) {
    WritableMap event = Arguments.createMap();
    event.putInt("newItemIndex", index);
    createEvent(event, VideoViewManager.EVENT_PLAYLIST_ITEM_CHANGED);
  }

  /**
   * Use for react event: onPlaylistEnded.
   */
  private void onPlaylistEndedEvent() {
    createEvent(null, VideoViewManager.EVENT_PLAYLIST_ENDED);
  }

  /**
   * Use for react event: onThumbnailLoaded.
   */
  private void onThumbnailLoaded(double requestedPosition, double actualPosition,
                                 @Nullable Bitmap thumbnail) {
    if (thumbnail == null) {
      return;
    }
    WritableMap event = Arguments.createMap();
    event.putDouble("requestedPosition", requestedPosition);
    event.putDouble("actualPosition", actualPosition);
    event.putInt("width", thumbnail.getWidth());
    event.putInt("height", thumbnail.getHeight());
    createEvent(event, VideoViewManager.EVENT_THUMBNAIL_LOADED);
    if (thumbnailView != null) {
      thumbnailView.setImageBitmap(thumbnail);
    }
  }

  private void resetStats() {
    droppedFrames = 0;
  }

  private void createEvent(WritableMap event, String name) {
    ReactContext reactContext = (ReactContext) getContext();
    reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(getId(), name, event);
  }

  @Override
  protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
    super.onLayout(changed, left, top, right, bottom);
    if (!createEventSent) {
      playerView.setKeepScreenOn(true);
      onViewCreatedEvent();
    }
    createEventSent = true;
  }
}
