import React, {Component} from 'react';
import {
  StyleSheet,
  View,
  LayoutChangeEvent,
  GestureResponderEvent,
  Platform,
  TouchableOpacity,
} from 'react-native';

import Colors from '../utils/Colors';

const UNSET = -1;

type Props = {
  isVisible?: boolean;
  progress?: number;
  buffered?: number;
  max?: number;
  style?: any;
  onTouchStart?: () => void;
  onTouchMove?: (progress: number) => void;
  onTouchRelease?: (progress: number) => void;
};

type States = {
  isVisible: boolean;
  isFocused: boolean;
  isLive: boolean;
};

class CLSeekBar extends Component<Props, States> {
  public x: number = UNSET;
  public y: number = UNSET;
  public width: number = UNSET;
  public height: number = UNSET;
  public hasTouch: boolean = false;
  public progress: number;
  public buffered: number;
  public max: number;
  public thumbPosition: number = UNSET;
  private backgroundBar: Bar | null = null;
  private bufferedBar: Bar | null = null;
  private progressBar: Bar | null = null;
  private thumb: Thumb | null = null;

  public constructor(props: Props) {
    super(props);

    this.state = {
      isVisible: props.isVisible !== undefined ? props.isVisible : true,
      isFocused: false,
      isLive: false,
    };

    this.progress = props.progress !== undefined ? props.progress : 0;
    this.buffered = props.buffered !== undefined ? props.buffered : 0;
    this.max = props.max !== undefined ? props.max : 100;
  }

  render() {
    if (this.state.isLive || !this.state.isVisible) {
      return null;
    }

    return (
      <View
        style={[styles.container, this.props.style]}
        onLayout={event => {
          this.onLayout(event);
        }}>
        {Platform.isTV && (
          <TouchableOpacity
            style={this.state.isFocused ? styles.onFocus : styles.onBlur}
            hasTVPreferredFocus={true}
            tvParallaxProperties={{enabled: false}}
            activeOpacity={1.0}
            onFocus={() => {
              this.setState({isFocused: true});
            }}
            onBlur={() => {
              this.setState({isFocused: false});
            }}>
            <View
              style={styles.contents}
              onStartShouldSetResponder={() => true}
              onTouchStart={() => {
                this.onTouchStart();
              }}
              onResponderMove={event => {
                this.onTouchMove(event);
              }}
              onResponderRelease={event => {
                this.onTouchRelease(event);
              }}>
              <Bar
                ref={bar => {
                  this.backgroundBar = bar;
                }}
                style={styles.backgroundBar}
                width={0}
              />
              <Bar
                ref={bar => {
                  this.bufferedBar = bar;
                }}
                style={styles.bufferedBar}
                width={0}
              />
              <Bar
                ref={bar => {
                  this.progressBar = bar;
                }}
                style={styles.progressBar}
                width={0}
              />
              <Thumb
                ref={thumb => {
                  this.thumb = thumb;
                }}
                size={10}
                left={0}
              />
            </View>
          </TouchableOpacity>
        )}

        {!Platform.isTV && (
          <View
            style={styles.contents}
            onStartShouldSetResponder={() => true}
            onTouchStart={() => {
              this.onTouchStart();
            }}
            onResponderMove={event => {
              this.onTouchMove(event);
            }}
            onResponderRelease={event => {
              this.onTouchRelease(event);
            }}>
            <Bar
              ref={bar => {
                this.backgroundBar = bar;
              }}
              style={styles.backgroundBar}
              width={0}
            />
            <Bar
              ref={bar => {
                this.bufferedBar = bar;
              }}
              style={styles.bufferedBar}
              width={0}
            />
            <Bar
              ref={bar => {
                this.progressBar = bar;
              }}
              style={styles.progressBar}
              width={0}
            />
            <Thumb
              ref={thumb => {
                this.thumb = thumb;
              }}
              size={10}
              left={0}
            />
          </View>
        )}
      </View>
    );
  }

  private onLayout(event: LayoutChangeEvent) {
    let {x, y, width, height} = event.nativeEvent.layout;
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = height;
    this.refresh();
  }

  private onTouchStart() {
    this.hasTouch = true;
    if (this.props.onTouchStart) {
      this.props.onTouchStart();
    }
  }

  private onTouchMove(event: GestureResponderEvent) {
    if (this.props.onTouchMove) {
      let posX = event.nativeEvent.pageX - this.x;
      let progress = Math.floor((posX * this.max) / this.width);
      if (progress > this.max) {
        progress = this.max;
      }
      if (progress < 0) {
        progress = 0;
      }
      this.props.onTouchMove(progress);
    }
  }

  private onTouchRelease(event: GestureResponderEvent) {
    this.hasTouch = false;
    if (this.props.onTouchRelease) {
      let posX = event.nativeEvent.pageX - this.x;
      let progress = Math.floor((posX * this.max) / this.width);
      if (progress > this.max) {
        progress = this.max;
      }
      if (progress < 0) {
        progress = 0;
      }
      this.props.onTouchRelease(progress);
    }
  }

  public setProgress(progress: number) {
    this.progress = progress;
    this.refresh();
  }

  public setBuffered(buffered: number) {
    this.buffered = buffered;
    this.refresh();
  }

  private refresh() {
    if (!this.state.isVisible || this.width === undefined) {
      return;
    }
    let thumbSize: number;
    if (this.hasTouch) {
      thumbSize = 20;
    } else {
      thumbSize = 10;
    }
    let bufferedWidth = (this.width * this.buffered) / this.max;
    let progressWidth = (this.width * this.progress) / this.max;
    this.thumbPosition = progressWidth + this.x;
    this.backgroundBar?.setState({width: this.width});
    this.bufferedBar?.setState({width: bufferedWidth});
    this.progressBar?.setState({width: progressWidth});
    this.thumb?.setState({size: thumbSize, left: progressWidth});
  }

  public isFocused(): boolean {
    return this.state.isFocused;
  }
}

type ThumbProps = {
  isVisible?: boolean;
  size: number;
  left: number;
};

type ThumbStates = {
  isVisible: boolean;
  size: number;
  left: number;
};

class Thumb extends Component<ThumbProps, ThumbStates> {
  public constructor(props: ThumbProps) {
    super(props);

    this.state = {
      isVisible: props.isVisible !== undefined ? props.isVisible : true,
      size: props.size,
      left: props.left,
    };
  }

  render() {
    let width = this.state.size;
    let height = this.state.size;
    let borderRadius = width / 2;
    let left = this.state.left - width / 2;

    if (this.state.isVisible) {
      return (
        <View
          style={[
            styles.thumb,
            {
              width: width,
              height: height,
              left: left,
              borderRadius: borderRadius,
            },
          ]}
        />
      );
    } else {
      return null;
    }
  }
}

type BarProps = {
  isVisible?: boolean;
  width: number;
  style?: any;
};

type BarStates = {
  isVisible: boolean;
  width: number;
};

class Bar extends Component<BarProps, BarStates> {
  public constructor(props: BarProps) {
    super(props);

    this.state = {
      isVisible: props.isVisible !== undefined ? props.isVisible : true,
      width: props.width,
    };
  }

  render() {
    if (this.state.isVisible) {
      return (
        <View
          style={[this.props.style, styles.bar, {width: this.state.width}]}
        />
      );
    } else {
      return null;
    }
  }
}

const styles = StyleSheet.create({
  container: {
    justifyContent: 'center',
  },
  contents: {
    width: '100%',
    height: '100%',
    justifyContent: 'center',
    backgroundColor: '#FFFFFF00',
  },
  backgroundBar: {
    backgroundColor: '#FFFFFF33',
  },
  progressBar: {
    backgroundColor: Colors.theme1,
  },
  bufferedBar: {
    backgroundColor: '#FFFFFF77',
  },
  bar: {
    height: 2,
    position: 'absolute',
  },
  thumb: {
    backgroundColor: Colors.theme1,
    position: 'absolute',
  },
  onFocus: {
    width: '100%',
    height: '100%',
    backgroundColor: Colors.buttonFocus,
  },
  onBlur: {
    width: '100%',
    height: '100%',
    backgroundColor: 'transparent',
  },
});

export default CLSeekBar;
