import React, {Component} from 'react';
import {StyleSheet, View, NativeModules, Platform, EventSubscription} from 'react-native';
// @ts-ignore (For TV remote control)
import {TVEventHandler, HWEvent} from 'react-native';
import CLPlayButton from './CLPlayButton';
import CLSeekBar from './CLSeekBar';
import CLTime from './CLTime';
import CLIndicator from './CLIndicator';
import CLPipButton from './CLPipButton';
import CLAirplayButton from './CLAirplayButton';
import CLAudioButton from './CLAudioButton';
import NextItemButton from './NextItemButton';
import CLImage from './CLImage';
import CLSelection, {Item, MenuItem} from './CLSelection';
import {
  CLThumbnail,
  CLVideo,
  AudioTrack,
  PlayerState,
  SubtitleTrack,
  VideoQuality,
} from '../../../src/index';
import HideableView from './HideableView';
import {CastButton} from 'react-native-google-cast';
import MoreButton from './MoreButton';
import equal from 'fast-deep-equal';

type Props = {
  isVisible?: boolean;
  style?: any;
  canCast: boolean;
  hasNextItem: boolean;
  onPlayNextItem?: () => void;
};

type States = {
  isVisible: boolean;
  active: boolean;
};

const SKIP_SECONDS = 10.0;

export default class PlayerControls extends Component<Props, States> {
  private UIShown: boolean;
  private background: CLImage | null = null;
  private playButton: CLPlayButton | null = null;
  private pipButton: CLPipButton | null = null;
  private airplayButton: CLAirplayButton | null = null;
  private castButton: HideableView | null = null;
  private audioButton: CLAudioButton | null = null;
  public seekBar: CLSeekBar | null = null;
  private time: CLTime | null = null;
  public thumbnail: CLThumbnail | null = null;
  private indicator: CLIndicator | null = null;
  private selection: CLSelection | null = null;
  private video: CLVideo | null = null;
  public loop: boolean = false;
  private hideTimer?: NodeJS.Timeout;
  private stepTimer?: NodeJS.Timeout;
  private isPipPossible: boolean = false;
  private isAirplayPossible: boolean = false;
  private tvEventSubscription: EventSubscription | undefined;

  public constructor(props: Props) {
    super(props);

    console.log('PlayerControls constructor()');

    this.state = {
      isVisible: props.isVisible !== undefined ? props.isVisible : true,
      active: true,
    };

    this.UIShown = true;
    this.hideUIWithTimer();

    this.toggleUI = this.toggleUI.bind(this);
    this.playButtonClick = this.playButtonClick.bind(this);
    this.nextButtonClick = this.nextButtonClick.bind(this);
    this.pipButtonClick = this.pipButtonClick.bind(this);
    this.airplayButtonClick = this.airplayButtonClick.bind(this);
    this.audioButtonClick = this.audioButtonClick.bind(this);
    this.moreButtonClick = this.moreButtonClick.bind(this);
    this.onSeekBarTouchStart = this.onSeekBarTouchStart.bind(this);
    this.onSeekBarTouchMove = this.onSeekBarTouchMove.bind(this);
    this.onSeekBarTouchRelease = this.onSeekBarTouchRelease.bind(this);
    this.selectionItemClick = this.selectionItemClick.bind(this);
    this.onSelectionClose = this.onSelectionClose.bind(this);
  }

  render() {
    console.log('PlayerControls render()');

    if (!this.state.isVisible) {
      return null;
    }

    return (
      <View style={[styles.main, this.props.style]}>
        <CLImage
          ref={background => {
            this.background = background;
          }}
          style={styles.background}
          isVisible={!Platform.isTV}
          onPress={this.toggleUI}
        />

        <CLPlayButton
          ref={playButton => {
            this.playButton = playButton;
          }}
          style={styles.playButton}
          onPress={this.playButtonClick}
        />

        {this.state.active && this.props.hasNextItem && (
          <NextItemButton
            style={styles.nextButton}
            onPress={this.nextButtonClick}
          />
        )}

        <CLPipButton
          ref={pipButton => {
            this.pipButton = pipButton;
          }}
          style={styles.pipButton}
          onPress={this.pipButtonClick}
        />

        <CLAirplayButton
          ref={airplayButton => {
            this.airplayButton = airplayButton;
          }}
          style={styles.airplayButton}
          onPress={this.airplayButtonClick}
        />

        {!Platform.isTV && this.props.canCast && (
          <HideableView
            ref={castButton => {
              this.castButton = castButton;
            }}>
            <CastButton style={styles.castButton} />
          </HideableView>
        )}

        <CLAudioButton
          ref={audioButton => {
            this.audioButton = audioButton;
          }}
          style={styles.audioButton}
          onPress={this.audioButtonClick}
        />

        {this.state.active && (
          <MoreButton
            style={styles.moreButton}
            onPress={this.moreButtonClick}
          />
        )}

        <CLSeekBar
          ref={seekBar => {
            this.seekBar = seekBar;
          }}
          style={styles.seekBar}
          onTouchStart={this.onSeekBarTouchStart}
          onTouchMove={this.onSeekBarTouchMove}
          onTouchRelease={this.onSeekBarTouchRelease}
        />

        <CLTime
          ref={time => {
            this.time = time;
          }}
          style={styles.time}
        />

        <CLThumbnail
          ref={thumbnail => {
            this.thumbnail = thumbnail;
          }}
          style={styles.thumbnail}
        />

        <CLIndicator
          ref={indicator => {
            this.indicator = indicator;
          }}
          style={styles.indicator}
        />

        <CLSelection
          ref={selection => {
            this.selection = selection;
          }}
          style={styles.selection}
          isVisible={false}
          onPress={this.selectionItemClick}
          onClose={this.onSelectionClose}
        />
      </View>
    );
  }

  private async playButtonClick() {
    if (this.video == null) {
      return;
    }
    if (this.video.isPlayWhenReady()) {
      await this.video.pause();
    } else {
      await this.video.play();
    }
    this.hideUIWithTimer();
  }

  private nextButtonClick() {
    if (this.props.onPlayNextItem == null) {
      return;
    }
    this.props.onPlayNextItem();
  }

  private pipButtonClick() {
    this.video?.togglePictureInPicture();
    if (Platform.OS === 'android') {
      this.hideUI();
      this.indicator?.setState({isVisible: false});
    } else {
      this.hideUIWithTimer();
    }
  }

  private airplayButtonClick() {
    if (this.isAirplayPossible) {
      NativeModules.AirplayRoutePicker.showRoutePicker({
        prioritizesVideoDevices: true,
      });
    }
    this.hideUIWithTimer();
  }

  private audioButtonClick() {
    if (this.audioButton?.isEnabled) {
      this.audioButton.disable();
      this.video?.setVolume(0);
    } else {
      this.audioButton?.enable();
      this.video?.setVolume(1);
    }
    this.hideUIWithTimer();
  }

  private async moreButtonClick() {
    if (this.selection?.state.isVisible) {
      this.selection.setState({isVisible: false});
    } else {
      let data = await this.getSelectionData();
      this.selection?.setState({isVisible: true, data: data});
    }
    this.hideUIWithTimer();
  }

  private onSeekBarTouchStart() {
    if (this.thumbnail == null) {
      return;
    }
    this.video?.setThumbnail(this.thumbnail);
  }

  private onSeekBarTouchMove(progress: number) {
    if (this.time == null || this.seekBar == null) {
      return;
    }
    this.seekBar?.setProgress(progress);
    let position = (progress * this.time.state.duration) / this.seekBar.max;
    this.time.setPosition(position);
    this.video?.getThumbnailData(position);
    this.refreshThumbnail();
    this.hideUIWithTimer();
  }

  private onSeekBarTouchRelease(progress: number) {
    if (this.time == null || this.seekBar == null) {
      return;
    }
    let position = (progress * this.time.state.duration) / this.seekBar.max;
    this.video?.setPosition(position);
    this.thumbnail?.setState({isVisible: false});
  }

  private refreshThumbnail() {
    if (this.seekBar == null || this.thumbnail == null) {
      return;
    }
    let left = this.seekBar.thumbPosition - this.thumbnail.width / 2;
    if (left < this.seekBar.x) {
      left = this.seekBar.x;
    }
    if (left > this.seekBar.x + this.seekBar.width - this.thumbnail.width) {
      left = this.seekBar.x + this.seekBar.width - this.thumbnail.width;
    }
    this.thumbnail.setState({left: left});
  }

  private selectionItemClick(item: Item) {
    switch (item.type) {
      case 'video':
        this.video?.setVideoQuality(item.index);
        break;
      case 'audio':
        this.video?.setAudioTrack(item.index);
        break;
      case 'subtitle':
        this.video?.setSubtitleTrack(item.index);
        break;
      case 'playbackRate':
        switch (item.index) {
          case 0: {
            this.video?.setPlaybackRate(0.5);
            break;
          }
          case 1: {
            this.video?.setPlaybackRate(1);
            break;
          }
          case 2: {
            this.video?.setPlaybackRate(2);
            break;
          }
        }
        break;
    }
    this.selection?.setState({isVisible: false});
    this.hideUIWithTimer();
  }

  private onSelectionClose(): void {
    if (this.selection == null) {
      return;
    }
    this.selection.setState({isVisible: false});
  }

  public setVideo(video: CLVideo) {
    this.video = video;
    if (!this.loop) {
      this.loop = true;
      this.step();
    }
  }

  private toggleUI() {
    if (this.UIShown) {
      this.hideUI();
    } else {
      this.showUI();
    }
  }

  private showUI() {
    this.UIShown = true;
    this.setState({active: true});
    this.background?.setState({opacity: 1});
    this.playButton?.setState({isVisible: true});
    this.pipButton?.setState({isVisible: this.isPipPossible});
    this.airplayButton?.setState({isVisible: this.isAirplayPossible});
    this.castButton?.setState({isVisible: true});
    this.audioButton?.setState({isVisible: true});
    this.seekBar?.setState({isVisible: true});
    this.time?.setState({isVisible: true});
    this.hideUIWithTimer();
  }

  private hideUI() {
    if (this.selection?.state.isVisible) {
      return;
    }
    this.UIShown = false;
    this.setState({active: false});
    this.background?.setState({opacity: 0});
    this.playButton?.setState({isVisible: false});
    this.pipButton?.setState({isVisible: false});
    this.airplayButton?.setState({isVisible: false});
    this.castButton?.setState({isVisible: false});
    this.audioButton?.setState({isVisible: false});
    this.seekBar?.setState({isVisible: false});
    this.time?.setState({isVisible: false});

    this.playButton?.setState({isFocus: false});
    this.pipButton?.setState({isFocus: false});
    this.airplayButton?.setState({isFocus: false});
    this.audioButton?.setState({isFocus: false});
  }

  private hideUIWithTimer() {
    if (this.hideTimer) {
      clearTimeout(this.hideTimer);
      this.hideTimer = undefined;
    }
    let time = Platform.isTV ? 10000 : 5000;
    this.hideTimer = setTimeout(() => this.hideUI(), time);
  }

  private async getSelectionData(): Promise<MenuItem[]> {
    // Get All tracks
    let videoQualities = await this.video?.getVideoQualities();
    let audioTracks = await this.video?.getAudioTracks();
    let subtitleTracks = await this.video?.getSubtitleTracks();
    if (audioTracks) {
      audioTracks.forEach(track => {
        console.log('audioTrack', track);
      });
    }
    if (subtitleTracks) {
      subtitleTracks.forEach(track => {
        console.log('subtitleTrack', track);
      });
    }

    // Get selected tracks
    let videoQuality = await this.video?.getVideoQuality();
    let audioTrack = await this.video?.getAudioTrack();
    let subtitleTrack = await this.video?.getSubtitleTrack();

    // Check values
    if (
      videoQualities == null ||
      audioTracks == null ||
      subtitleTracks == null ||
      videoQuality == null ||
      audioTrack == null ||
      subtitleTrack == null
    ) {
      return Promise.reject();
    }

    // Prepare video menu
    let videoQualityMode = await this.video?.getVideoQualityMode();
    let adaptiveMode = videoQualityMode === 'adaptive';
    let videoData = [
      {index: -1, title: 'Auto', selected: adaptiveMode, type: 'video'},
    ];
    for (let i = 0; i < videoQualities.length; i++) {
      if (videoQualities[i].bitrate === -1) {
        continue;
      }
      let title = this.getVideoTitle(videoQualities[i]);
      let selected = this.hasSameVideoQuality(videoQualities[i], videoQuality);
      if (adaptiveMode) {
        selected = false;
      }
      videoData.push({
        index: i,
        title: title,
        selected: selected,
        type: 'video',
      });
    }
    let videoItem = {title: 'Video', data: videoData};

    // Prepare audio menu
    let audioDisabled = audioTrack.bitrate === -1;
    let audioData = [
      {index: -1, title: 'Disable', selected: audioDisabled, type: 'audio'},
    ];
    for (let i = 0; i < audioTracks.length; i++) {
      if (audioTracks[i].bitrate === -1) {
        continue;
      }
      let title = this.getAudioTitle(audioTracks[i]);
      let selected = this.hasSameAudioTrack(audioTracks[i], audioTrack);
      audioData.push({
        index: i,
        title: title,
        selected: selected,
        type: 'audio',
      });
    }
    let audioItem = {title: 'Audio', data: audioData};

    // Get current playback rate
    const playbackRate = await this.video?.getPlaybackRate();

    // Prepare subtitle menu
    let subtitleDisabled = subtitleTrack.label == null;
    let subtitleData = [
      {
        index: -1,
        title: 'Disable',
        selected: subtitleDisabled,
        type: 'subtitle',
      },
    ];
    for (let i = 0; i < subtitleTracks.length; i++) {
      let title = this.getSubtitleTitle(subtitleTracks[i]);
      let selected = this.hasSameSubtitleTrack(
        subtitleTracks[i],
        subtitleTrack,
      );
      subtitleData.push({
        index: i,
        title: title,
        selected: selected,
        type: 'subtitle',
      });
    }
    let subtitleItem = {title: 'Subtitle', data: subtitleData};

    // Return selection data
    let selectionData = [videoItem];
    if (audioItem.data.length > 1) {
      selectionData.push(audioItem);
    }
    if (subtitleItem.data.length > 1) {
      selectionData.push(subtitleItem);
    }

    selectionData.push({
      title: 'Playback Rate',
      data: [
        {
          index: 0,
          title: 'Slower',
          selected: playbackRate === 0.5,
          type: 'playbackRate',
        },
        {
          index: 1,
          title: 'Normal',
          selected: playbackRate === 1,
          type: 'playbackRate',
        },
        {
          index: 2,
          title: 'Faster',
          selected: playbackRate === 2,
          type: 'playbackRate',
        },
      ],
    });

    return selectionData;
  }

  private getVideoTitle(videoQuality: VideoQuality): string {
    let bitrate = videoQuality.bitrate;
    let width = videoQuality.width;
    let height = videoQuality.height;
    let codecs = videoQuality.codecs;
    let title = width + 'x' + height + ' @ ' + this.formatBitrate(bitrate);
    if (width === -1 || height === -1) {
      title = codecs + ' @ ' + this.formatBitrate(bitrate);
    }
    return title;
  }

  private getAudioTitle(audioTrack: AudioTrack): string {
    let bitrate = audioTrack.bitrate;
    let label = audioTrack.label;
    let title: string;
    if (bitrate > 0) {
      title = label + ' @ ' + this.formatBitrate(bitrate);
    } else {
      title = label;
    }
    return title;
  }

  private getSubtitleTitle(subtitleTrack: SubtitleTrack): string {
    return `${subtitleTrack.label} - ${subtitleTrack.mimeType}`;
  }

  private formatBitrate(bitrate: number): string {
    if (bitrate > 1000000) {
      return (bitrate / 1000000).toPrecision(2) + ' Mbps';
    } else {
      return (bitrate / 1000).toPrecision(3) + ' Kbps';
    }
  }

  private hasSameVideoQuality(
    quality1: VideoQuality,
    quality2: VideoQuality,
  ): boolean {
    if (quality1.bitrate !== quality2.bitrate) {
      return false;
    }
    if (quality1.width !== quality2.width) {
      return false;
    }
    if (quality1.height !== quality2.height) {
      return false;
    }
    if (quality1.codecs !== quality2.codecs) {
      return false;
    }
    if (quality1.mimeType !== quality2.mimeType) {
      return false;
    }
    return true;
  }

  private hasSameAudioTrack(track1: AudioTrack, track2: AudioTrack): boolean {
    return equal(track1, track2);
  }

  private hasSameSubtitleTrack(
    track1: SubtitleTrack,
    track2: SubtitleTrack,
  ): boolean {
    return equal(track1, track2);
  }

  private async step() {
    if (
      this.video == null ||
      this.indicator == null ||
      this.seekBar == null ||
      this.time == null ||
      !this.loop
    ) {
      return;
    }

    // Get the player state
    let playerState: PlayerState;
    try {
      playerState = await this.video.getPlayerState();
    } catch (error) {
      playerState = PlayerState.Idle;
    }

    if (this.video.isPlayWhenReady()) {
      this.playButton?.play();
    } else {
      this.playButton?.pause();
    }

    // Show the indicator if buffering
    if (playerState === PlayerState.Buffering) {
      this.indicator.setState({isVisible: true});
    } else {
      this.indicator.setState({isVisible: false});
    }

    // Get the player position
    let position: number;
    try {
      position = await this.video.getPosition();
    } catch (error) {
      position = 0;
    }

    // Update the position
    if (position < 0) {
      position = 0;
    }
    if (this.seekBar && !this.seekBar.hasTouch) {
      this.time.setPosition(position);
    }

    // Get the duration
    let duration: number;
    try {
      duration = await this.video.getDuration();
    } catch (error) {
      duration = 0;
    }

    const isLive = await this.video.isLive();

    // Update the duration
    if (duration < 0) {
      duration = 0;
    }
    if (this.time) {
      this.time.setState({isLive});
      this.time.setDuration(duration);
    }

    // Get the progress percentage
    let progressPercentage: number;
    try {
      progressPercentage = await this.video.getProgressPercentage();
    } catch (error) {
      progressPercentage = 0;
    }

    // Update seekBar
    if (this.seekBar) {
      this.seekBar.setState({isLive});
    }
    if (this.seekBar && !this.seekBar.hasTouch) {
      this.seekBar.setProgress(progressPercentage);
    }

    // Get the buffered percentage
    let bufferedPercentage: number;
    try {
      bufferedPercentage = await this.video.getBufferedPercentage();
    } catch (error) {
      bufferedPercentage = 0;
    }

    // Update seekBar
    if (this.seekBar) {
      this.seekBar.setBuffered(bufferedPercentage);
    }

    try {
      this.isPipPossible = await this.video.isPictureInPicturePossible();
    } catch (error) {
      this.isPipPossible = false;
    }

    if (this.UIShown) {
      this.pipButton?.setState({isVisible: this.isPipPossible});
    }

    // Airplay is available only on iOS using AVPlayer
    if (Platform.OS === 'ios') {
      try {
        this.isAirplayPossible = await this.video.isAirplayPossible();
      } catch (error) {
        this.isAirplayPossible = false;
      }
    } else {
      this.isAirplayPossible = false;
    }

    if (this.UIShown) {
      this.airplayButton?.setState({isVisible: this.isAirplayPossible});
    }

    if (this.loop) {
      this.stepTimer = setTimeout(() => this.step(), 100);
    }

    // const stats = await this.video?.getStats();
    // if (stats) {
    //   console.log('==> stats:', stats);
    // }
  }

  componentDidMount() {
    if (Platform.isTV) {
      this.enableTVEventHandler(this);
    }
  }

  componentWillUnmount() {
    if (Platform.isTV) {
      this.disableTVEventHandler();
    }
    if (this.hideTimer) {
      clearTimeout(this.hideTimer);
      this.hideTimer = undefined;
    }
    if (this.stepTimer) {
      clearTimeout(this.stepTimer);
      this.stepTimer = undefined;
    }
  }

  private enableTVEventHandler(owner: PlayerControls) {
    const tvEventHandler = (evt: HWEvent) => {
      if (!evt) {
        return;
      }
      if (
        ![
          'playPause',
          'select',
          'right',
          'left',
          'up',
          'down',
          'swipeRight',
          'swipeLeft',
          'swipeUp',
          'swipeDown',
        ].includes(evt.eventType)
      ) {
        return;
      }

      if (!owner.UIShown) {
        owner.showUI();
      }

      switch (evt.eventType) {
        case 'playPause':
          owner.playButtonClick();
          break;
        case 'right':
        case 'swipeRight':
          if (owner.seekBar != null && owner.seekBar.isFocused()) {
            owner.skipForward();
          }
          break;
        case 'left':
        case 'swipeLeft':
          if (owner.seekBar != null && owner.seekBar.isFocused()) {
            owner.skipBackward();
          }
          break;
      }
    };
    this.tvEventSubscription = TVEventHandler.addListener(tvEventHandler);
  }

  private skipForward() {
    if (this.time == null) {
      return;
    }

    let position = Number(this.time.state.position) + SKIP_SECONDS;
    if (Number(this.time.state.duration) < Number(position)) {
      position = Number(this.time.state.duration);
    }
    this.video?.setPosition(position);
  }

  private skipBackward() {
    if (this.time == null) {
      return;
    }

    let position = Number(this.time.state.position) - SKIP_SECONDS;
    if (Number(position) < 0) {
      position = 0.0;
    }
    this.video?.setPosition(position);
  }

  private disableTVEventHandler() {
    if (this.tvEventSubscription !== undefined) {
      this.tvEventSubscription.remove();
    }
  }
}

const styles = StyleSheet.create({
  main: {
    width: '100%',
    height: '100%',
  },
  background: {
    position: 'absolute',
    width: '100%',
    height: '100%',
  },
  playButton: {
    position: 'absolute',
    width: 40,
    height: 40,
    left: 15,
    bottom: 15,
  },
  nextButton: {
    position: 'absolute',
    width: 40,
    height: 40,
    left: 50,
    bottom: 15,
  },
  airplayButton: {
    position: 'absolute',
    width: 40,
    height: 40,
    right: 95,
    bottom: 15,
  },
  pipButton: {
    position: 'absolute',
    width: 40,
    height: 40,
    right: 55,
    bottom: 15,
  },
  audioButton: {
    position: 'absolute',
    width: 40,
    height: 40,
    right: 15,
    bottom: 15,
  },
  castButton: {
    position: 'absolute',
    tintColor: '#FFFFFF',
    width: 40,
    height: 40,
    right: 60,
    top: 24,
  },
  moreButton: {
    position: 'absolute',
    width: 40,
    height: 40,
    right: 10,
    top: 24,
  },
  time: {
    position: 'absolute',
    left: 95,
    height: 30,
    bottom: 15,
  },
  thumbnail: {
    position: 'absolute',
    width: 120,
    height: 80,
    bottom: 100,
    left: 30,
    borderColor: '#FFFFFF88',
    borderWidth: 1,
    borderStyle: 'solid',
  },
  selection: {
    position: 'absolute',
  },
  seekBar: {
    height: 25,
    position: 'absolute',
    left: 30,
    right: 30,
    bottom: 65,
  },
  indicator: {
    flex: 1,
    alignSelf: 'center',
  },
});
