import React from 'react';
import {
  Alert,
  EmitterSubscription,
  FlatList,
  Image,
  StyleSheet,
  Text,
  TouchableOpacity,
  View,
} from 'react-native';
import {
  Action,
  ActionUiProperty,
  CLAppleTVPlayer,
  PlayerOptions,
  PopupMenu,
  SubMenu,
  SubMenuItem,
  TVStream,
} from '../../../src/index';
import Colors from '../utils/Colors';
import Screen, {ScreenProps} from '../components/Screen';
import {DrmTodayEnvironment, DrmType} from '../../../src';

const nextLogo = require('../assets/images/nextChannelLogo.png');
const nextLogoUri = Image.resolveAssetSource(nextLogo).uri;
const previousLogo = require('../assets/images/previousChannelLogo.png');
const previousLogoUri = Image.resolveAssetSource(previousLogo).uri;

type TVDemo = {
  id: string;
  title: string;
  description: string;
  tvStreams: TVStream[];
};

const ActionId = {
  Record: 'record',
  Loop: 'loop',
  Speed: 'speed',
  Skip00: 'skip00',
  Skip01: 'skip01',
  WatchLater: 'watch-later',
};

const skipDemo: TVStream[] = [
  {
    playerConfig: {
      source:
        'https://d24rwxnt7vw9qb.cloudfront.net/out/v1/74781b1e229b49cb8a57fa64f76f9e88/index.m3u8',
    },
    channelMetadata: {
      mainTitle: 'AWS Elemental clear live HLS',
      subTitle: 'Clear live hls with thumbnails',
      artworkImage: 'artwork',
      description: 'AWS elemental clear live hls stream with thumbnails',
    },
    nextChannelInfo: {
      previousChannelInterstitialView: {
        xibName: 'ChannelInterstitialView',
        labels: [{1000: '2'}, {1001: 'Clear live TOS HLS'}],
        images: [{2000: previousLogoUri}, {2001: 'image00'}],
      },
      nextChannelInterstitialView: {
        xibName: 'ChannelInterstitialView',
        labels: [{1000: '1'}, {1001: 'Channel 1 HLS'}],
        images: [{2000: nextLogoUri}, {2001: 'image01'}],
      },
    },
  },
  {
    playerConfig: {
      source:
        'https://usp-demo.castlabs.com/live/channel1/channel1.isml/hls.m3u8',
    },
    channelMetadata: {
      mainTitle: 'Channel 1 HLS',
      subTitle: 'Clear HLS live with 10s latency',
      artworkImage: 'artwork',
      description: 'In house clear live hls stream',
    },
    nextChannelInfo: {
      previousChannelInterstitialView: {
        xibName: 'ChannelInterstitialView',
        labels: [{1000: '3'}, {1001: 'AWS Elemental clear live HLS'}],
        images: [{2000: previousLogoUri}, {2001: 'image00'}],
      },
      nextChannelInterstitialView: {
        xibName: 'ChannelInterstitialView',
        labels: [{1000: '2'}, {1001: 'Clear live TOS HLS'}],
        images: [{2000: nextLogoUri}, {2001: 'image01'}],
      },
    },
  },
];

class AppleTVDemosScreen extends Screen<{}> {
  public demoData: TVDemo[] = this.getDemos();

  private skipDemoCurrentIndex = 0;

  private actionSubscription: EmitterSubscription | null = null;

  public constructor(props: ScreenProps) {
    super(props);
    this.setPlayerEventHandlers();
    this.addActionListener();
  }

  render() {
    return (
      <View style={styles.main}>
        <View style={styles.listContainer}>
          <FlatList
            style={styles.flatList}
            data={this.demoData}
            numColumns={1}
            renderItem={({item, index}) => (
              <TouchableOpacity
                style={[
                  styles.itemContainer,
                  {marginTop: index === 0 ? 30 : 20},
                  {marginBottom: index === this.demoData.length - 1 ? 30 : 0},
                ]}
                onPress={() => {
                  this.openPlayer(item.tvStreams);
                }}>
                <View style={styles.itemText}>
                  <Text style={styles.itemTitle}> {item.title} </Text>
                  <Text style={styles.itemDescription}>
                    {' '}
                    {item.description}{' '}
                  </Text>
                </View>
                <Image
                  style={styles.itemImage}
                  source={require('../assets/images/play.png')}
                />
              </TouchableOpacity>
            )}
            keyExtractor={item => item.id}
          />
        </View>
      </View>
    );
  }

  private setPlayerEventHandlers() {
    CLAppleTVPlayer.didPlayToEnd = (event: any) => {
      console.log('CLAppleTVPlayer.didPlayToEnd event:', event);
      CLAppleTVPlayer.closePlayer();
    };

    CLAppleTVPlayer.playerIsClosed = (event: any) => {
      console.log('CLAppleTVPlayer.playerIsClosed event:', event);
    };

    CLAppleTVPlayer.willSeek = (event: any) => {
      // On live stream, `beyondSeekableRange` means seeking the LIVE position
      console.log('CLAppleTVPlayer.willSeek event:', event);
      this.getPosition();
      this.getLiveStartTime();
    };

    CLAppleTVPlayer.didSeek = (event: any) => {
      console.log('CLAppleTVPlayer.didSeek event:', event);
      this.getPosition();
    };

    CLAppleTVPlayer.timeControlStatus = (event: any) => {
      console.log('CLAppleTVPlayer.timeControlStatus event:', event);
      this.getCurrentStreamIndex();
    };

    CLAppleTVPlayer.skipToNextChannel = (event: any) => {
      console.log('CLAppleTVPlayer.skipToNextChannel event:', event);

      this.skipDemoCurrentIndex = this.skipDemoCurrentIndex + 1;
      if (this.skipDemoCurrentIndex >= skipDemo.length) {
        this.skipDemoCurrentIndex = 0;
      }

      this.openPlayer([skipDemo[this.skipDemoCurrentIndex]]);
    };

    CLAppleTVPlayer.skipToPreviousChannel = (event: any) => {
      console.log('CLAppleTVPlayer.skipToPreviousChannel event:', event);

      this.skipDemoCurrentIndex = this.skipDemoCurrentIndex - 1;
      if (this.skipDemoCurrentIndex < 0) {
        this.skipDemoCurrentIndex = skipDemo.length - 1;
      }

      this.openPlayer([skipDemo[this.skipDemoCurrentIndex]]);
    };

    CLAppleTVPlayer.onError = (event: any) => {
      console.log('CLAppleTVPlayer.onError event:', event);
      Alert.alert('Error', JSON.stringify(event.error), []);
      CLAppleTVPlayer.closePlayer();
    };

    CLAppleTVPlayer.audioTrackChanged = (event: any) => {
      console.log('CLAppleTVPlayer.audioTrackChanged event:', event);
    };

    CLAppleTVPlayer.subtitleTrackChanged = (event: any) => {
      console.log('CLAppleTVPlayer.subtitleTrackChanged event:', event);
    };

    CLAppleTVPlayer.streamChanged = (event: any) => {
      console.log('CLAppleTVPlayer.streamChanged event:', event);
    };
  }

  private async getPosition() {
    let pos = await CLAppleTVPlayer.getPosition();
    console.log('Current position:', pos);
  }

  private async getLiveStartTime() {
    let pos = await CLAppleTVPlayer.getLiveStartTime();
    console.log('Live Start Time:', pos);
  }

  private async getAudioTrack() {
    let track = await CLAppleTVPlayer.getAudioTrack();
    console.log('Current audio track:', track);
  }

  private async getSubtitleTrack() {
    let track = await CLAppleTVPlayer.getSubtitleTrack();
    console.log('Current subitle track:', track);
  }

  private async getCurrentStreamIndex() {
    let index = await CLAppleTVPlayer.getCurrentStreamIndex();
    console.log('Current stream index:', index);
  }

  private addActionListener() {
    const onActionEvent = (event: any) => {
      console.log('onActionEvent: ', event);
      switch (event.id) {
        case ActionId.Record:
          if (event.data === 'false') {
            let uiProp: ActionUiProperty = {imageName: 'record.circle.fill'};
            CLAppleTVPlayer.setActionStateAndUi(event.id, true, uiProp);
          } else {
            let uiProp: ActionUiProperty = {imageName: 'record.circle'};
            CLAppleTVPlayer.setActionStateAndUi(event.id, false, uiProp);
          }
          break;
        case ActionId.Loop:
          if (event.data === 'false') {
            let uiProp: ActionUiProperty = {
              title: 'Loop on',
              imageName: 'infinity.circle.fill',
            };
            CLAppleTVPlayer.setActionStateAndUi(event.id, true, uiProp);
          } else {
            let uiProp: ActionUiProperty = {
              title: 'Loop off',
              imageName: 'infinity.circle',
            };
            CLAppleTVPlayer.setActionStateAndUi(event.id, false, uiProp);
          }
          break;
        case ActionId.Speed:
          CLAppleTVPlayer.setRate(Number(event.data));
          break;
        case ActionId.Skip00:
          CLAppleTVPlayer.seekToSecond(15);
          break;
        case ActionId.Skip01:
          CLAppleTVPlayer.seekToSecond(25);
          break;
        case ActionId.WatchLater:
          CLAppleTVPlayer.closePlayer();
          break;
      }
    };
    this.actionSubscription = CLAppleTVPlayer.addActionListener(onActionEvent);
  }

  private removeActionListener() {
    if (this.actionSubscription) {
      CLAppleTVPlayer.removeActionListener(this.actionSubscription);
      this.actionSubscription = null;
    }
  }

  private openPlayer(tvStreams: TVStream[]) {
    let options: PlayerOptions = {
      allowsPictureInPicturePlayback: true,
      closeWhenPlaybackEnd: false,
      requiresLinearPlayback: false,
      disableChannelSkipInLiveStream: false,
    };
    let transportBarActions = this.createTransportBarActions();
    let infoTabActions = this.createInfoTabActions();
    let contextualActions = this.createContextualActions();
    let loopStreams = true;

    CLAppleTVPlayer.openPlayer(
      tvStreams,
      options,
      transportBarActions,
      infoTabActions,
      contextualActions,
      loopStreams,
      (error: any) => {
        console.log('CLAppleTVPlayer.openPlayer > onError callback');
        if (error) {
          console.error(error);
          Alert.alert('Error', error, []);
        } else {
          console.log('Player is opened');
        }
      },
      () => {
        console.log('CLAppleTVPlayer.openPlayer > onSuccess callback');
      },
    );
  }

  public changeOptions_sample() {
    let newMetadata = {
      mainTitle: 'New Main Title',
      subTitle: 'New Subtitle',
      artworkImage:
        'https://demo.cf.castlabs.com/media/prestodash/prestoplay_small.jpg',
      description: 'New description of the channel.',
    };
    CLAppleTVPlayer.setMetadata(newMetadata);

    let newOptions: PlayerOptions = {
      allowsPictureInPicturePlayback: true,
      closeWhenPlaybackEnd: true,
      requiresLinearPlayback: false,
      disableChannelSkipInLiveStream: false,
    };
    CLAppleTVPlayer.setOptions(newOptions);

    let newTransportBarActions = this.createNewTransportBarActions();
    CLAppleTVPlayer.setTransportBarActions(newTransportBarActions);

    let newInfoTabActions = this.createNewInfoTabActions();
    CLAppleTVPlayer.setInfoTabActions(newInfoTabActions);

    let newContextualActions = this.createNewContextualActions();
    CLAppleTVPlayer.setContextualActions(newContextualActions);
  }

  private createTransportBarActions(): any[] {
    let recordAction = this.createRecordAction();
    let popupMenu = this.createPopupMenu();
    return [recordAction, popupMenu];
  }

  private createRecordAction(): Action {
    return {
      id: ActionId.Record,
      title: 'Record',
      imageName: 'record.circle',
    };
  }

  private createPopupMenu(): PopupMenu {
    let loopAction: Action = {
      id: ActionId.Loop,
      title: 'Loop off',
      imageName: 'infinity.circle',
    };
    let subMenuItem00: SubMenuItem = {title: 'Half', value: '0.5'};
    let subMenuItem01: SubMenuItem = {title: 'Default', value: '1.0'};
    let subMenuItem02: SubMenuItem = {title: 'Double', value: '2.0'};
    let subMenu: SubMenu = {
      defaultValue: '1.0',
      menuItems: [subMenuItem00, subMenuItem01, subMenuItem02],
    };
    let speedAction: Action = {
      id: ActionId.Speed,
      title: 'Speed',
      subMenu: subMenu,
    };
    return {
      title: 'Preferences',
      imageName: 'gearshape',
      actions: [loopAction, speedAction],
    };
  }

  private createContextualActions(): Action[] {
    let contextualAction00: Action = {
      id: ActionId.Skip00,
      title: 'Skip to 15 seconds',
      startSecond: 5,
      endSecond: 15,
    };
    let contextualAction01: Action = {
      id: ActionId.Skip01,
      title: 'Skip to 25 seconds',
      startSecond: 15,
      endSecond: 25,
    };
    return [contextualAction00, contextualAction01];
  }

  private createInfoTabActions(): Action[] {
    let watchLaterAction: Action = {
      id: ActionId.WatchLater,
      title: 'Watch Later',
      imageName: 'eyeglasses',
    };
    return [watchLaterAction];
  }

  private createNewTransportBarActions(): Action[] {
    let action00 = {
      id: 'new-transportbar-action00',
      title: 'New Action 00',
      imageName: 'infinity.circle',
    };
    let action01 = {
      id: 'new-transportbar-action01',
      title: 'New Action 01',
      imageName: 'infinity.circle',
    };
    return [action00, action01];
  }

  private createNewInfoTabActions(): Action[] {
    let watchLaterAction = {
      id: ActionId.WatchLater,
      title: 'Watch Later(new)',
      imageName: 'eyeglasses',
    };
    return [watchLaterAction];
  }

  private createNewContextualActions(): Action[] {
    let contextualAction00 = {
      id: 'new-contextual-action00',
      title: 'Skip to 20 seconds(new)',
      startSecond: 10,
      endSecond: 20,
    };
    let contextualAction01 = {
      id: 'new-contextual-action01',
      title: 'Skip to 30 seconds(new)',
      startSecond: 20,
      endSecond: 30,
    };
    return [contextualAction00, contextualAction01];
  }

  private getDemos(): TVDemo[] {
    let demos: TVDemo[] = [];

    let demo1: TVDemo = {
      id: '1',
      title: 'HLS (iOS) - 10sec offset',
      description: 'Play an unencrypted HLS stream.',
      tvStreams: [
        {
          playerConfig: {
            source: 'https://demo.cf.castlabs.com/media/prestohls/master.m3u8',
            startTimeMs: 10000,
          },
          channelMetadata: {
            mainTitle: 'Demo1, Main Title',
            subTitle: 'Demo1, Subtitle',
            artworkImage:
              'https://demo.cf.castlabs.com/media/prestodash/prestoplay_small.jpg',
            description: 'Demo1, Description of the channel.',
          },
        },
      ],
    };
    demos.push(demo1);

    let demo2: TVDemo = {
      id: '2',
      title: 'HLS live stream playlist',
      description: 'Test live playlist',
      tvStreams: [
        {
          playerConfig: {
            source:
              'https://usp-demo.castlabs.com/live/channel1/channel1.isml/hls.m3u8',
            liveConfiguration: {
              liveEdgeLatencyMs: 60000,
            },
          },
          channelMetadata: {
            mainTitle: 'Channel 1 title',
            subTitle: 'Channel 1 subtitle',
            artworkImage: 'artwork',
            description: 'Channel 1 description',
          },
          nextChannelInfo: {
            previousChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '3'}, {1001: 'Channel 3 title'}],
              images: [{2000: previousLogoUri}, {2001: 'image00'}],
            },
            nextChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '2'}, {1001: 'Channel 2 title'}],
              images: [{2000: nextLogoUri}, {2001: 'image01'}],
            },
          },
        },
        {
          playerConfig: {
            source:
              'https://usp-demo.castlabs.com/live/channel1/channel1.isml/hls.m3u8',
            liveConfiguration: {
              liveEdgeLatencyMs: 40000,
            },
          },
          channelMetadata: {
            mainTitle: 'Channel 2 title',
            subTitle: 'Channel 2 subtitle',
            artworkImage: 'artwork',
            description: 'Channel 2 description',
          },
          nextChannelInfo: {
            previousChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '1'}, {1001: 'Channel 1 title'}],
              images: [{2000: previousLogoUri}, {2001: 'image00'}],
            },
            nextChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '3'}, {1001: 'Channel 3 title'}],
              images: [{2000: nextLogoUri}, {2001: 'image01'}],
            },
          },
        },
        {
          playerConfig: {
            source:
              'https://usp-demo.castlabs.com/live/channel1/channel1.isml/hls.m3u8',
            liveConfiguration: {
              liveEdgeLatencyMs: 20000,
            },
          },
          channelMetadata: {
            mainTitle: 'Channel 3 title',
            subTitle: 'Channel 3 subtitle',
            artworkImage: 'artwork',
            description: 'Channel 3 description',
          },
          nextChannelInfo: {
            previousChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '2'}, {1001: 'Channel 2 title'}],
              images: [{2000: previousLogoUri}, {2001: 'image00'}],
            },
            nextChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '1'}, {1001: 'Channel 1 title'}],
              images: [{2000: nextLogoUri}, {2001: 'image01'}],
            },
          },
        },
      ],
    };
    demos.push(demo2);

    let demo3: TVDemo = {
      id: '3',
      title: 'Encrypted HLS VOD',
      description: 'Play HLS VOD encrypted with Fairplay',
      tvStreams: [
        {
          playerConfig: {
            source: 'https://demo.cf.castlabs.com/media/fps/prog_index.m3u8',
            drmType: DrmType.DRMTODAY,
            drmConfiguration: {
              environment: DrmTodayEnvironment.STAGING,
              userId: 'rental1',
              sessionId: 'p0',
              merchant: 'six',
              assetId: 'twelve',
            },
          },
          channelMetadata: {
            mainTitle: 'Demo3, Main Title',
            subTitle: 'Demo3, Subtitle',
            artworkImage: 'artwork',
            description: 'Demo3, Description of the channel.',
          },
        },
      ],
    };
    demos.push(demo3);

    let demo4: TVDemo = {
      id: '4',
      title: 'HLS live stream playlist',
      description:
        'Play Live HLS (might be broken since we are not using in house streams but from customers)',
      tvStreams: [
        {
          playerConfig: {
            source:
              'https://usp-demo.castlabs.com/live/channel1/channel1.isml/hls.m3u8',
            liveConfiguration: {
              liveEdgeLatencyMs: 10000,
            },
          },
          channelMetadata: {
            mainTitle: 'Channel 1 HLS',
            subTitle: 'Clear HLS live with 10s latency',
            artworkImage: 'artwork',
            description: 'In house clear live hls stream',
          },
          nextChannelInfo: {
            previousChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '3'}, {1001: 'AWS Elemental clear live HLS'}],
              images: [{2000: previousLogoUri}, {2001: 'image00'}],
            },
            nextChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '2'}, {1001: 'Clear live TOS HLS'}],
              images: [{2000: nextLogoUri}, {2001: 'image01'}],
            },
          },
        },
        {
          playerConfig: {
            source:
              'https://cph-p2p-msl.akamaized.net/hls/live/2000341/test/master.m3u8',
            // liveConfiguration: {
            //   liveEdgeLatencyMs: 60000,
            // },
          },
          channelMetadata: {
            mainTitle: 'Clear live TOS HLS',
            subTitle: 'Clear live HLS stream without latency',
            artworkImage: 'artwork',
            description: 'live TOS stream from akamai ',
          },
          nextChannelInfo: {
            previousChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '1'}, {1001: 'Channel 1 HLS'}],
              images: [{2000: previousLogoUri}, {2001: 'image00'}],
            },
            nextChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '3'}, {1001: 'AWS Elemental clear live HLS'}],
              images: [{2000: nextLogoUri}, {2001: 'image01'}],
            },
          },
        },
        {
          playerConfig: {
            source:
              'https://d24rwxnt7vw9qb.cloudfront.net/out/v1/74781b1e229b49cb8a57fa64f76f9e88/index.m3u8',
            // liveConfiguration: {
            //   liveEdgeLatencyMs: 60000,
            // },
          },
          channelMetadata: {
            mainTitle: 'AWS Elemental clear live HLS',
            subTitle: 'Clear live hls with thumbnails',
            artworkImage: 'artwork',
            description: 'AWS elemental clear live hls stream with thumbnails',
          },
          nextChannelInfo: {
            previousChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '2'}, {1001: 'Clear live TOS HLS'}],
              images: [{2000: previousLogoUri}, {2001: 'image00'}],
            },
            nextChannelInterstitialView: {
              xibName: 'ChannelInterstitialView',
              labels: [{1000: '1'}, {1001: 'Channel 1 HLS'}],
              images: [{2000: nextLogoUri}, {2001: 'image01'}],
            },
          },
        },
      ],
    };
    demos.push(demo4);

    let demo5: TVDemo = {
      id: '5',
      title: 'HLS live stream (single item)',
      description: 'Test skipToNextChannel and skipToPreviousChannel',
      tvStreams: [skipDemo[0]],
    };
    demos.push(demo5);

    return demos;
  }
}

const styles = StyleSheet.create({
  main: {
    flex: 1,
    flexDirection: 'column',
    justifyContent: 'center',
    alignItems: 'center',
  },
  itemContainer: {
    width: 300,
    height: 90,
    backgroundColor: Colors.theme2,
    borderRadius: 10,
    borderWidth: 1,
    borderColor: Colors.theme1,
    justifyContent: 'center',
    alignSelf: 'center',
  },
  itemText: {
    position: 'absolute',
    left: 10,
    marginRight: 40,
  },
  itemTitle: {
    color: Colors.theme1,
    fontSize: 14,
    fontWeight: '600',
  },
  itemDescription: {
    color: Colors.theme3,
    fontSize: 12,
    marginTop: 5,
  },
  itemImage: {
    width: 30,
    height: 30,
    tintColor: Colors.theme1,
    position: 'absolute',
    right: 10,
  },
  listContainer: {
    height: 600,
  },
  flatList: {
    width: '100%',
    height: '100%',
  },
});

export default AppleTVDemosScreen;
