import React, {Component} from 'react';
import {
  PermissionsAndroid,
  Platform,
  ScrollView,
  StyleSheet,
  Text,
  View,
} from 'react-native';
import Colors from '../utils/Colors';
import CLScreen, {ScreenProps} from '../components/Screen';
import CLOption from '../components/CLOption';
import CLProgressbar from '../components/CLProgressbar';
import CLButton from '../components/CLButton';
import MsgBundle from '../utils/MsgBundle';
import CLDownloader from '../../../src/utils/CLDownloader';
import ScreenTags from './ScreenTags';
import {
  DownloadConfig,
  DownloadError,
  DownloadProgress,
  DownloadState,
  DrmSystem,
  DrmTodayEnvironment,
  DrmType,
  TrackSelection,
  TrackSet,
} from '../../../src';
import Demo from '../models/Demo';

type DownloadItem = {
  title: string;
  navigation: any;
  config: DownloadConfig;
};

class DownloadScreen extends CLScreen<{}> {
  public constructor(props: ScreenProps) {
    super(props);
  }

  public componentDidMount() {
    this.requestNotificationPermission();
  }

  render() {
    if (Platform.OS === 'android') {
      return (
        <ScrollView style={styles.main}>
          <DownloadComponent
            title={'Simple Playback (Clear)'}
            navigation={this.props.navigation}
            config={{
              id: '1',
              url: 'http://demo.castlabs.com/media/TOS/abr/Manifest_clean_sizes.mpd',
            }}
          />

          <DownloadComponent
            title={'Offline Key Storage (Widevine)'}
            navigation={this.props.navigation}
            config={{
              id: '2',
              url: 'http://demo.castlabs.com/media/TOS/DASH/DEMO.mpd',
              drm: {
                drmType: DrmType.DRMTODAY,
                drmSystem: DrmSystem.WIDEVINE,
                drmConfiguration: {
                  environment: DrmTodayEnvironment.STAGING,
                  userId: 'purchase',
                  sessionId: 'sessionId',
                  merchant: 'six',
                  assetId: 'dasheverywhere_demo',
                  offlineId: 'dasheverywhere_demo',
                },
              },
            }}
          />

          <DownloadComponent
            title={'404 Error'}
            navigation={this.props.navigation}
            config={{
              id: '3',
              url: 'http://demo.cf.castlabs.com/media/bbb_abr/Manifest_404.mpd',
              drm: {
                drmType: DrmType.DRMTODAY,
                drmSystem: DrmSystem.WIDEVINE,
                drmConfiguration: {
                  environment: DrmTodayEnvironment.STAGING,
                  userId: 'purchase',
                  sessionId: 'sessionId',
                  merchant: 'client_dev',
                  assetId: 'BBB_TEST',
                  offlineId: 'BBB_TEST',
                },
              },
            }}
          />
        </ScrollView>
      );
    } else {
      return (
        <ScrollView style={styles.main}>
          <DownloadComponent
            title={'Simple Playback (Clear)'}
            navigation={this.props.navigation}
            config={{
              id: '1',
              url: 'https://demo.cf.castlabs.com/media/prestohls/master.m3u8',
            }}
          />

          <DownloadComponent
            title={'Offline Key Storage (Fairplay)'}
            navigation={this.props.navigation}
            config={{
              id: '2',
              url: 'http://demo.cf.castlabs.com/public/Client/hls/ioss291-encrypted720/master.m3u8',
              drm: {
                drmType: DrmType.DRMTODAY,
                drmSystem: DrmSystem.FAIRPLAY,
                drmConfiguration: {
                  environment: DrmTodayEnvironment.STAGING,
                  userId: 'purchase',
                  sessionId: 'p1',
                  merchant: 'six',
                  assetId: 'FPSOffline',
                  offlineId: 'FPSOffline',
                },
              },
            }}
          />
        </ScrollView>
      );
    }
  }

  private async requestNotificationPermission() {
    // Only for Android
    if (Platform.OS !== 'android') {
      return;
    }

    // Check if notification permission already granted
    let isGranted = await PermissionsAndroid.check(
      PermissionsAndroid.PERMISSIONS.POST_NOTIFICATIONS,
    );
    if (isGranted) {
      return;
    }

    // Request the permission
    let result = await PermissionsAndroid.request(
      PermissionsAndroid.PERMISSIONS.POST_NOTIFICATIONS,
      {
        title: 'RN Demos Notification Permission',
        message: 'RN Demos needs to show notification',
        buttonNeutral: 'Ask Me Later',
        buttonNegative: 'Cancel',
        buttonPositive: 'OK',
      },
    );

    if (result === PermissionsAndroid.RESULTS.GRANTED) {
      console.log('Notification allowed');
    } else {
      console.log('Notification not allowed');
    }
  }
}

class DownloadComponent extends Component<DownloadItem, DownloadItem> {
  private downloader: CLDownloader | null = null;
  private option: CLOption | null = null;
  private progressbar: CLProgressbar | null = null;
  private downloadButton: CLButton | null = null;
  private deleteButton: CLButton | null = null;
  private playButton: CLButton | null = null;
  private prepared: boolean = false;

  public constructor(props: DownloadItem) {
    super(props);

    this.state = {
      title: props.title,
      navigation: props.navigation,
      config: props.config,
    };
  }

  public componentDidMount() {
    this.downloader = new CLDownloader();
    this.downloader
      .prepareById(this.state.config.id)
      .then((event: DownloadProgress) => {
        this.prepared = true;
        this.addListeners();
        this.progressbar?.setProgress(event.progress);
        this.deleteButton?.setState({enabled: true});
        this.downloader?.getState().then((state: DownloadState) => {
          if (state === DownloadState.Done) {
            this.progressbar?.setProgress(100);
            this.playButton?.setState({enabled: true});
            this.downloadButton?.setState({text: 'Download', enabled: false});
          } else if (state === DownloadState.Loading) {
            this.playButton?.setState({enabled: false});
            this.downloadButton?.setState({text: 'Pause', enabled: true});
          } else {
            this.playButton?.setState({enabled: false});
            this.downloadButton?.setState({text: 'Resume', enabled: true});
          }
        });
      })
      .catch(() => {
        this.prepared = false;
        this.addListeners();
      });
  }

  render() {
    return (
      <View style={styles.itemContainer}>
        <Text style={styles.itemTitle}> {this.state.title} </Text>

        <CLOption
          ref={option => {
            this.option = option;
          }}
          style={styles.option}
          title={'Quality'}
          value={0}
          labels={['HD', 'SD', 'Highest', 'Lowest']}
        />

        <CLProgressbar
          ref={progressbar => {
            this.progressbar = progressbar;
          }}
          style={styles.progressbar}
          progress={0}
        />

        <View style={styles.buttonContainer}>
          <CLButton
            ref={button => {
              this.downloadButton = button;
            }}
            text={'Download'}
            onPress={() => {
              this.download();
            }}
            enabled={true}
          />
          <CLButton
            ref={button => {
              this.deleteButton = button;
            }}
            text={'Delete'}
            onPress={() => {
              this.delete();
            }}
            enabled={false}
          />
          <CLButton
            ref={button => {
              this.playButton = button;
            }}
            text={'Play'}
            onPress={() => {
              this.play();
            }}
            enabled={false}
          />
        </View>
      </View>
    );
  }

  private async download() {
    if (this.downloader == null) {
      return;
    }
    if (!this.prepared) {
      this.downloadButton?.setState({text: 'Preparing', enabled: false});
      let trackSet = await this.downloader.prepare(this.state.config);
      this.prepared = true;
      this.downloadButton?.setState({text: 'Pause', enabled: true});
      let selection = this.getSelection(trackSet);
      this.downloader.start(selection);
    } else {
      let downloadState = await this.downloader.getState();
      if (downloadState === DownloadState.Loading) {
        this.downloadButton?.setState({text: 'Resume', enabled: true});
        this.deleteButton?.setState({enabled: true});
        this.downloader.pause();
      } else {
        this.downloadButton?.setState({text: 'Pause', enabled: true});
        this.deleteButton?.setState({enabled: false});
        this.downloader.resume();
      }
    }
  }

  private delete() {
    this.prepared = false;
    this.downloader?.delete();
    this.downloadButton?.setState({text: 'Download', enabled: true});
    this.deleteButton?.setState({enabled: false});
    this.playButton?.setState({enabled: false});
    this.progressbar?.setProgress(0);
  }

  private async play() {
    // Check downloader
    if (this.downloader == null) {
      return;
    }

    // Create player config
    let media: any = {autoPlay: true};

    // Get content url
    media.source = await this.downloader.getLocalManifestUrl();

    // Check drm
    let drm = this.state.config.drm;
    if (drm != null) {
      media.drmType = drm.drmType;
      media.drmConfiguration = drm.drmConfiguration;
    }

    // Create the demo
    let demo: Demo = {
      id: this.downloader.instanceId!,
      title: this.state.title,
      description: 'Play downloaded content',
      media: media,
      showOnApple: true,
      showOnAndroid: true,
      canCast: false,
    };
    this.gotoPlayer(demo);
  }

  private gotoPlayer(demo: Demo) {
    MsgBundle.set('title', demo.title);
    MsgBundle.set('media', demo.media);
    this.props.navigation.navigate(ScreenTags.Player);
  }

  private getSelection(trackSet: TrackSet): TrackSelection {
    let videoQualityIndex = 0;

    switch (this.option?.state.value) {
      // HD
      case 0: {
        let selected = false;
        for (let i = 0; i < trackSet.videoQualities.length; i++) {
          let quality = trackSet.videoQualities[i];
          if (quality.height >= 720) {
            selected = true;
            videoQualityIndex = i;
            break;
          }
        }
        if (!selected) {
          videoQualityIndex = 0;
        }
        break;
      }
      // SD
      case 1: {
        let selected = false;
        for (let i = 0; i < trackSet.videoQualities.length; i++) {
          let quality = trackSet.videoQualities[i];
          if (quality.height < 720) {
            selected = true;
            videoQualityIndex = i;
            break;
          }
        }
        if (!selected) {
          videoQualityIndex = trackSet.videoQualities.length - 1;
        }
        break;
      }
      // Highest
      case 2: {
        videoQualityIndex = 0;
        break;
      }
      // lowest
      case 3: {
        videoQualityIndex = trackSet.videoQualities.length - 1;
        break;
      }
    }

    return {
      videoQuality: videoQualityIndex,
      audioTracks: [0],
      subtitleTracks: [0],
    };
  }

  private addListeners() {
    this.downloader?.onProgress((event: DownloadProgress) => {
      this.progressbar?.setProgress(event.progress);
    });

    this.downloader?.onComplete(() => {
      this.progressbar?.setProgress(100);
      this.downloadButton?.setState({text: 'Download', enabled: false});
      this.deleteButton?.setState({enabled: true});
      this.playButton?.setState({enabled: true});
    });

    this.downloader?.onError((event: DownloadError) => {
      console.log('Error: ' + event.error);
      this.downloadButton?.setState({text: 'Error', enabled: false});
    });
  }
}

const styles = StyleSheet.create({
  main: {
    flex: 1,
    paddingLeft: 20,
    paddingRight: 20,
  },
  flatList: {
    width: '100%',
    height: '100%',
    paddingLeft: 20,
    paddingRight: 20,
  },
  itemContainer: {
    width: '100%',
    height: 160,
    marginTop: 20,
    backgroundColor: Colors.theme2,
    borderRadius: 15,
    borderWidth: 1,
    borderColor: Colors.theme1,
    padding: 15,
  },
  itemTitle: {
    color: Colors.theme1,
    fontSize: 16,
    fontWeight: '600',
  },
  option: {
    marginTop: 10,
  },
  progressbar: {
    marginTop: 20,
    marginLeft: 5,
    marginRight: 5,
  },
  buttonContainer: {
    alignSelf: 'baseline',
    flexDirection: 'row',
    marginTop: 20,
  },
});

export default DownloadScreen;
