import React, {ReactNode} from 'react';
import {
  View,
  StyleSheet,
  NativeEventEmitter,
  NativeModules,
  Platform,
  EventSubscription,
} from 'react-native';
import {CLVideo, PlayerConfiguration} from '../../../src/index';
import PlayerControls from '../components/PlayerControls';
import MsgBundle from '../utils/MsgBundle';
import ScreenTags from './ScreenTags';
import Screen, {ScreenProps} from '../components/Screen';
import Colors from '../utils/Colors';
import {Media, Playlist} from '../../../src';
import CastService from '../services/CastService';
import GoogleCast from 'react-native-google-cast';
import ErrorData from '../../../src/types/ErrorData';

// TODO remove global variables
let thisScreen: PlayerScreen;

// Picture in Picture events on iOS and tvOS.
// Restore this PlayerScreen when Picture in Picture stops.
if (Platform.OS === 'ios') {
  const {PictureInPictureEventModule} = NativeModules;
  const nativeEventEmitter = new NativeEventEmitter(
    PictureInPictureEventModule,
  );
  const onPipEvent = (event: any) => {
    console.log('onPipEvent: ', event);
    if (event.id === 'restore-user-interface-for-picture-in-picture-stop') {
      thisScreen.props.navigation.navigate(ScreenTags.Player);
    }
  };
  nativeEventEmitter.addListener('onPictureInPictureEvent', onPipEvent);
}

interface PlayerScreenState {
  videoCreated: boolean;
  hasNextItem: boolean;
}

class PlayerScreen extends Screen<PlayerScreenState> {
  private readonly media: Media;
  private readonly canCast: boolean;
  private readonly castSessionListener: EventSubscription;

  private video: CLVideo | null = null;
  private controls: PlayerControls | null = null;

  public constructor(props: ScreenProps) {
    super(props);

    console.log('PlayerScreen constructor()');

    // eslint-disable-next-line consistent-this
    thisScreen = this;

    this.castSessionListener = GoogleCast.getSessionManager().onSessionStarted(
      () => {
        if (this.video) {
          CastService.instance?.castFromVideo(this.video);
        }
      },
    );

    this.onVideoCreated = this.onVideoCreated.bind(this);
    this.onStateChanged = this.onStateChanged.bind(this);
    this.onError = this.onError.bind(this);
    this.onSeekTo = this.onSeekTo.bind(this);
    this.onSeekCompleted = this.onSeekCompleted.bind(this);
    this.onFullyBuffered = this.onFullyBuffered.bind(this);
    this.onThumbnailLoaded = this.onThumbnailLoaded.bind(this);
    this.onPlayerModelChanged = this.onPlayerModelChanged.bind(this);
    this.onPlaylistItemChanged = this.onPlaylistItemChanged.bind(this);
    this.onPlayNextItemClicked = this.onPlayNextItemClicked.bind(this);

    const media = MsgBundle.get('media') as
      | PlayerConfiguration
      | PlayerConfiguration[];

    this.media = Array.isArray(media)
      ? new Playlist(
          media,
          this.onPlaylistItemChanged,
          this.onPlaylistItemChanged,
        )
      : media;

    this.canCast = MsgBundle.get('canCast') as boolean;

    this.state = {
      videoCreated: false,
      hasNextItem: this.playlist?.hasNext() || false,
    };
  }

  public onBeforeRemove(e: any): void {
    super.onBeforeRemove(e);

    if (this.controls) {
      this.controls.loop = false;
    }

    this.video?.pause();
  }

  public componentWillUnmount(): void {
    this.castSessionListener.remove();
  }

  private onVideoCreated(): void {
    this.setState({videoCreated: true});
  }

  private onStateChanged(playbackState: number): void {
    console.log('state: ' + playbackState);
    if (playbackState === 4) {
      // playing
      this.video?.getLiveStartTime().then(liveStartTime => {
        console.log('live start time [s]: ' + liveStartTime);
      });
    }
  }

  private onError(error: ErrorData): void {
    console.log('error severity: ' + error.severity);
    console.log('error type: ' + error.type);
    console.log('error message: ' + error.message);
    console.log('error http code: ' + error.httpCode);
    console.log('error url: ' + error.url);
    console.log('response body: ' + error.responseBody);
  }

  private onSeekTo(newPosition: number): void {
    console.log('position: ' + newPosition);
  }

  private onSeekCompleted(): void {
    console.log('seek completed');
  }

  private onFullyBuffered(): void {
    console.log('fully buffered');
  }

  private onThumbnailLoaded(): void {
    if (
      this.controls != null &&
      this.controls.thumbnail != null &&
      this.controls.seekBar != null &&
      this.controls.seekBar.hasTouch
    ) {
      this.controls.thumbnail.setState({isVisible: true});
    }
  }

  private onPlayerModelChanged(): void {
    console.log('player model changed');
  }

  private onPlaylistItemChanged(): void {
    console.log('playlist item changed');
    this.setState({
      hasNextItem: this.playlist?.hasNext() || false,
    });
  }

  public render(): ReactNode {
    console.log('PlayerScreen render()');

    return (
      <View style={styles.main}>
        <CLVideo
          ref={video => {
            this.video = video;
          }}
          style={styles.video}
          media={this.media}
          onViewCreated={this.onVideoCreated}
          onStateChanged={this.onStateChanged}
          onError={this.onError}
          onSeekTo={this.onSeekTo}
          onSeekCompleted={this.onSeekCompleted}
          onFullyBuffered={this.onFullyBuffered}
          onThumbnailLoaded={this.onThumbnailLoaded}
          onPlayerModelChanged={this.onPlayerModelChanged}
        />

        {this.state.videoCreated && (
          <PlayerControls
            ref={controls => {
              this.controls = controls;
              if (this.video != null) {
                controls?.setVideo(this.video);
              }
            }}
            style={styles.controls}
            canCast={this.canCast}
            hasNextItem={this.state.hasNextItem}
            onPlayNextItem={this.onPlayNextItemClicked}
          />
        )}
      </View>
    );
  }

  private async onPlayNextItemClicked(): Promise<void> {
    if (this.playlist?.hasNext()) {
      await this.playlist?.playNext();
    }
  }

  private get playlist(): Playlist | undefined {
    return this.media instanceof Playlist ? this.media : undefined;
  }
}

const styles = StyleSheet.create({
  main: {
    flex: 1,
    flexDirection: 'column',
    justifyContent: 'center',
    alignItems: 'center',
    backgroundColor: Colors.theme5,
  },
  video: {
    width: '100%',
    height: '100%',
  },
  controls: {
    width: '100%',
    height: '100%',
    position: 'absolute',
  },
});

export default PlayerScreen;
