import {
  CLVideo,
  MediaLoadRequestUtils,
  MediaStatusUtils,
  PlayerConfiguration,
} from '../../../src';
import Demo from '../models/Demo';
import GoogleCast, {CastSession, MediaStatus} from 'react-native-google-cast';
import {EventSubscription, Platform} from 'react-native';
import MsgBundle from '../utils/MsgBundle';
import NavigationService from './NavigationService';
import ScreenTags from '../screens/ScreenTags';

export default class CastService {
  public static instance = Platform.isTV ? undefined : new CastService();

  private session: CastSession | null = null;
  private mediaStatus: MediaStatus | null = null;

  private sessionStartedListener: EventSubscription;
  private sessionResumedListener: EventSubscription;
  private sessionEndedListener: EventSubscription;

  private mediaStatusUpdatedListener: EventSubscription | null = null;
  private mediaProgressUpdatedListener: {remove: () => void} | null = null;

  constructor() {
    const sessionManager = GoogleCast.getSessionManager();

    this.sessionStartedListener = sessionManager.onSessionStarted(
      this.onSessionStartedOrResumed.bind(this),
    );
    this.sessionResumedListener = sessionManager.onSessionResumed(
      this.onSessionStartedOrResumed.bind(this),
    );

    this.sessionEndedListener = sessionManager.onSessionEnded(
      this.onSessionEnded.bind(this),
    );

    // The sessionResumed event might not be triggered
    // if the session is restored before registering the listeners
    sessionManager.getCurrentCastSession().then(session => {
      if (session) {
        this.onSessionStartedOrResumed(session);
      }
    });
  }

  private onMediaStatusUpdated(status: MediaStatus | null): void {
    this.mediaStatus = status;
  }
  private onMediaProgressUpdated(progress: number): void {
    if (this.mediaStatus) {
      this.mediaStatus.streamPosition = progress;
    }
  }

  private async onSessionStartedOrResumed(session: CastSession): Promise<void> {
    console.log('The cast session started or resumed');

    this.session = session;
    this.mediaStatus = await session.getClient().getMediaStatus();

    // Skip registering the listener if it's already registered
    if (!this.mediaStatusUpdatedListener) {
      this.mediaStatusUpdatedListener = session
        .getClient()
        .onMediaStatusUpdated(this.onMediaStatusUpdated.bind(this));
    }

    // Skip registering the listener if it's already registered
    if (!this.mediaProgressUpdatedListener) {
      this.mediaProgressUpdatedListener = session
        .getClient()
        .onMediaProgressUpdated(this.onMediaProgressUpdated.bind(this));
    }
  }

  private onSessionEnded(): void {
    console.log('The cast session ended');

    if (this.mediaStatus) {
      // NOTE: The mediaStatus contains the latest streamPosition
      // thanks to the onMediaProgressUpdated listener. By default,
      // the streamPosition is not up-to-date.

      // HINT: Deserialize your custom data
      // from the `mediaStatus.mediaInfo.customData` object.

      const playerConfiguration = MediaStatusUtils.toPlayerConfiguration(
        this.mediaStatus,
      );

      console.log('playerConfiguration', JSON.stringify(playerConfiguration));

      if (playerConfiguration) {
        MsgBundle.set('title', this.mediaStatus.mediaInfo?.metadata?.title);
        MsgBundle.set('media', playerConfiguration);
        NavigationService.navigate(ScreenTags.Player);
      }
    }

    this.session = null;
    this.mediaStatus = null;

    this.mediaStatusUpdatedListener?.remove();
    this.mediaStatusUpdatedListener = null;

    // The listener is automatically removed on session ended.
    // this.mediaProgressUpdatedListener?.remove();
    this.mediaProgressUpdatedListener = null;
  }

  public async castFromDemo(demo: Demo): Promise<void> {
    if (!this.session) {
      return;
    }

    // TODO handle it better
    const mediaLoadRequest = MediaLoadRequestUtils.fromPlayerConfiguration(
      demo.media as PlayerConfiguration,
    );

    // HINT: Serialize your custom data
    // into the `mediaLoadRequest.mediaInfo.customData` object.

    mediaLoadRequest.mediaInfo!.metadata!.title = demo.title;

    await this.session?.getClient().loadMedia(mediaLoadRequest);
  }

  public async castFromVideo(video: CLVideo): Promise<void> {
    if (!this.session) {
      return;
    }

    const mediaLoadRequest = await MediaLoadRequestUtils.fromVideoComponent(
      video,
    );

    await video.pause();

    // HINT: Serialize your custom data
    // into the `mediaLoadRequest.mediaInfo.customData` object.

    mediaLoadRequest.mediaInfo!.metadata!.title = MsgBundle.get(
      'title',
    ) as string;

    await this.session.getClient().loadMedia(mediaLoadRequest);

    // We assume that we are in the 'Player' screen,
    // and we want to go back to the 'DemosScreen'.
    NavigationService.goBack();
  }

  public dispose(): void {
    this.sessionStartedListener.remove();
    this.sessionResumedListener.remove();
    this.sessionEndedListener.remove();

    this.mediaStatusUpdatedListener?.remove();
    this.mediaStatusUpdatedListener = null;

    this.mediaProgressUpdatedListener?.remove();
    this.mediaProgressUpdatedListener = null;

    this.session = null;
    this.mediaStatus = null;
  }
}
