import CastlabsYoubora
import NpawPlugin
import PRESTOplay

class BridgeDeserializer {

    // MARK: Public

    public static func toMediaLoadRequest(json: String) -> MediaLoadRequest? {
        guard
            let jsonObject = toJsonObject(json: json),
            let usePlaylist = jsonObject["usePlaylist"] as? Bool,
            let jsonItems = jsonObject["items"] as? [[String: Any]] else
        {
            return nil
        }

        var items: [MediaLoadRequestItem] = []

        for jsonItem in jsonItems {
            if let mediaLoadRequestItem = toMediaLoadRequestItem(jsonObject: jsonItem) {
                items.append(mediaLoadRequestItem)
            }
        }

        return MediaLoadRequest(
            usePlaylist: usePlaylist,
            items: items
        )
    }

    public static func toMediaLoadRequestItem(json: String) -> MediaLoadRequestItem? {
        guard let jsonObject = toJsonObject(json: json) else {
            return nil
        }

        return toMediaLoadRequestItem(jsonObject: jsonObject)
    }

    public static func toMediaLoadRequestItem(jsonObject: [String: Any]) -> MediaLoadRequestItem? {
        guard let playerConfiguration = toPlayerConfiguration(jsonObject: jsonObject) else {
            return nil
        }

        let youboraMetadata = toYouboraMetadata(jsonObject: jsonObject)
        let ignoreSilentSwitch = jsonObject["ignoreSilentSwitch"] as? String

        return MediaLoadRequestItem(
            playerConfiguration: playerConfiguration,
            youboraMetadata: youboraMetadata,
            ignoreSilentSwitch: ignoreSilentSwitch
        )
    }

    public static func toYouboraMetadata(jsonObject: [String: Any]) -> YouboraMetadata? {
        guard let analyticsMetaData = jsonObject["analyticsMetaData"] as? [String: Any] else {
            return nil
        }

        let live = analyticsMetaData["live"] as? Bool
        let assetId = analyticsMetaData["assetId"] as? String

        // TODO: verify if the "assetId" value is an expected fallback value
        let youboraMetadata = YouboraMetadata(
            live: live ?? false,
            assetId: assetId ?? "assetId"
        )

        youboraMetadata.analyticsOptions = NpawPlugin.AnalyticsOptions()
        youboraMetadata.analyticsOptions?.contentTitle = assetId

        return youboraMetadata
    }

    public static func toPlayerConfiguration(json: String) -> PlayerConfiguration? {
        guard let jsonObject = toJsonObject(json: json) else { return nil }
        return toPlayerConfiguration(jsonObject: jsonObject)
    }

    public static func toPlayerConfiguration(jsonObject: [String: Any]) -> PlayerConfiguration? {
        var jsonObject = jsonObject

        // remove unsupported metadata
        jsonObject.removeValue(forKey: "metaData")

        if
            let source = jsonObject["source"] as? [String: Any],
            let url = source["url"] as? String
        {
            jsonObject["contentUrl"] = url
            jsonObject["contentType"] = "HLS"
            jsonObject.removeValue(forKey: "source")
        }

        // RN-225
        if let preferredTextLanguage = jsonObject["preferredTextLanguage"] as? String {
            jsonObject["autoSelectTextTrackMode"] =
                AutoSelectTextTrackMode.anyMatchingTrack.rawValue
        }

        var preventSecondScreenPlayback = true;
        if let secondaryDisplayBehaviour = jsonObject["secondaryDisplayBehaviour"] as? String {
            if "never" == secondaryDisplayBehaviour {
                preventSecondScreenPlayback = true;
            }
            if "allowAlways" == secondaryDisplayBehaviour {
                preventSecondScreenPlayback = false;
            }
        }

        if let drmConfiguration = jsonObject["drmConfiguration"] as? [String: Any] {
            var newDrmConfiguration = [String: Any]()

            newDrmConfiguration["environment"] = drmConfiguration["environment"]
            newDrmConfiguration["userId"] = drmConfiguration["userId"]
            newDrmConfiguration["sessionId"] = drmConfiguration["sessionId"]
            newDrmConfiguration["merchant"] = drmConfiguration["merchant"]
            newDrmConfiguration["assetId"] = drmConfiguration["assetId"]
            newDrmConfiguration["variantId"] = drmConfiguration["variantId"]
            newDrmConfiguration["authToken"] = drmConfiguration["authToken"]

            newDrmConfiguration["preventSecondScreenPlayback"] = preventSecondScreenPlayback;

            if let fairplayConfiguration = drmConfiguration["FAIRPLAY"] as? [String: Any] {
                newDrmConfiguration["licensingUrl"] =
                    fairplayConfiguration["licensingUrl"]
                newDrmConfiguration["licensingParameters"] =
                    fairplayConfiguration["licensingParameters"]
                newDrmConfiguration["certificateUrl"] =
                    fairplayConfiguration["certificateUrl"]
                newDrmConfiguration["certificateParameters"] =
                    fairplayConfiguration["certificateParameters"]
            }

            jsonObject["drmConfiguration"] = newDrmConfiguration
        }

        if let sideloadTracks = jsonObject["sideloadedTracks"] as? [[String: Any]] {
            var newSideloadedTracks = [[String: Any]]()

            for track in sideloadTracks {
                var newTrack = track
                let trackType = track["trackType"] as? String

                if let trackType, trackType == "SUBTITLE" {
                    newTrack["trackType"] = "text"
                } else if trackType == "THUMBNAIL" {
                    newTrack["trackType"] = "thumbnail"
                }

                newSideloadedTracks.append(newTrack)
            }

            jsonObject["sideloadedTracks"] = newSideloadedTracks
        }

        guard let json = toJson(jsonObject: jsonObject) else {
            return nil
        }

        return PlayerConfiguration.create(from: json)
    }

    // MARK: Private

    private static func toJsonObject(json: String) -> [String: Any]? {
        guard
            let jsonData = json.data(using: .utf8),
            let jsonObject = try? JSONSerialization.jsonObject(
                with: jsonData,
                options: .allowFragments
            ) as? [String: Any] else
        {
            return nil
        }

        return jsonObject
    }

    private static func toJson(jsonObject: [String: Any]) -> String? {
        guard
            let jsonData = try? JSONSerialization.data(withJSONObject: jsonObject, options: []),
            let json = String(data: jsonData, encoding: .utf8) else
        {
            return nil
        }

        return json
    }
}
