//
//  DownloadModule.swift
//  react-native-prestoplay-module
//
//  Created by Mackode - Bartlomiej Makowski on 14/04/2022.
//

import CoreMedia
import Foundation
import PRESTOplay

@objc(DownloadModule)
class DownloadModule: RCTEventEmitter, DownloadDelegate {

    // MARK: Lifecycle

    override init() {
        super.init()
        DownloadModule.shared = self
    }

    // MARK: Internal

    static var shared: DownloadModule?

    // MARK: - Events

    let EVENT_PREFIX = "CLDownloader";

    var hasListeners = false

    @objc
    override static func requiresMainQueueSetup() -> Bool {
        true
    }


    @objc
    func startDownload(
        _ instanceId: NSString!,
        selection: NSDictionary!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let download = findDownload(instanceId as String) else {
            sendErrorEvent(instanceId as String, nil)
            reject("error", "Download not found!", nil)
            return
        }

        if let videoSelection = selection["videoQuality"] as? Int {
            let renditions: [VideoRendition] = download.videoTracks.flatMap { $0.renditions }
            let _ = download.selectVideoRendition(renditions[videoSelection])
        }

        if let _ = selection["audioTracks"] as? [Int] {
            /// NOT IMPLEMENTED
        }

        if let subtitleSelections = selection["subtitleTracks"] as? [Int] {
            subtitleSelections.forEach { let _ = download.selectTextTrack(download.textTracks[$0]) }
        }

        let downloader = PRESTOplaySDK.shared.downloader()
        let error = downloader.startDownload(download.uuid, delegate: self)
        if error != nil {
            sendErrorEvent(instanceId as String, error)
            reject("error", "unable to start download", error)
            return
        }
        resolve(nil)
    }

    @objc
    func resumeDownload(
        _ instanceId: NSString!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let download = findDownload(instanceId as String) else {
            sendErrorEvent(instanceId as String, nil)
            reject("error", "Download not found!", nil)
            return
        }

        let downloader = PRESTOplaySDK.shared.downloader()
        let error = downloader.resumeDownload(download.uuid)
        if error != nil {
            sendErrorEvent(instanceId as String, error)
            reject("error", "unable to start download", error)
            return
        }
        resolve(nil)
    }

    @objc
    func pauseDownload(
        _ instanceId: NSString!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let download = findDownload(instanceId as String) else {
            sendErrorEvent(instanceId as String, nil)
            reject("error", "Download not found!", nil)
            return
        }

        let downloader = PRESTOplaySDK.shared.downloader()
        let error = downloader.pauseDownload(download.uuid)
        if error != nil {
            sendErrorEvent(instanceId as String, error)
            reject("error", "unable to start download", error)
            return
        }
        resolve(nil)
    }

    @objc
    func deleteDownload(
        _ instanceId: NSString!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let download = findDownload(instanceId as String) else {
            sendErrorEvent(instanceId as String, nil)
            reject("error", "Download not found!", nil)
            return
        }

        let downloader = PRESTOplaySDK.shared.downloader()
        let error = downloader.removeDownload(download.uuid)
        if error != nil {
            sendErrorEvent(instanceId as String, error)
            reject("error", "unable to start download", error)
            return
        }
        resolve(nil)
    }

    @objc
    func prepareById(
        _ downloadId: NSString!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let download = findDownload(downloadId as String) else {
            reject("error", "Download not found!", nil)
            return
        }

        DownloadInstanceManager.shared.setInstance(
            instanceId: downloadId as String,
            download: download
        )
        resolve(getProgressMap(download))
    }

    @objc
    func prepareDownload(
        _ config: NSDictionary!,
        resolver resolve: @escaping RCTPromiseResolveBlock,
        rejecter reject: @escaping RCTPromiseRejectBlock
    ) {
        let instanceId: String = config["id"] as? String ?? ""
        let urlString: String = config["url"] as? String ?? ""

        let downloader = PRESTOplaySDK.shared.downloader()
        guard let url = URL(string: urlString) else {
            sendErrorEvent(instanceId as String, nil)
            reject("error", "Wrong download url", nil)
            return
        }
        let playerConfig = PlayerConfiguration(with: url)
        let download = downloader.createDownload(playerConfig, headers: [:])
        download.title = instanceId

        // TODO: change this to a constructor / playerConfiguration
        if let drm = config["drm"] as? String {
            var drmDict: [String: Any] = [:]
            if let data = drm.data(using: .utf8) {
                let responseDict = try? JSONSerialization.jsonObject(
                    with: data,
                    options: .allowFragments
                ) as? [String: Any]
                if let dict = responseDict {
                    drmDict = dict
                }
            }

            if drmDict["drmSystem"] as! String == "FAIRPLAY" {
                playerConfig.drmSystem = .fairplay
            }

            if drmDict["drmType"] as! String == "DRMTODAY" {
                playerConfig.drmType = .drmToday
            }

            playerConfig.drmConfiguration = DrmConfiguration()
            if let drmConfig = drmDict["drmConfiguration"] as? [String: String] {
                switch drmConfig["environment"] {
                case "PRODUCTION":
                    playerConfig.drmConfiguration?.environment = .production
                case "STAGING":
                    playerConfig.drmConfiguration?.environment = .staging
                case "TESTING":
                    playerConfig.drmConfiguration?.environment = .testing
                case .none: break
                case .some: break
                }

                playerConfig.drmConfiguration?.userId = drmConfig["userId"]
                playerConfig.drmConfiguration?.sessionId = drmConfig["sessionId"]
                playerConfig.drmConfiguration?.merchant = drmConfig["merchant"]
                playerConfig.drmConfiguration?.assetId = drmConfig["assetId"]
                playerConfig.drmConfiguration?.persistLicense = true

                guard let prefetcher = PRESTOplaySDK.shared.prefetcher(for: playerConfig) else {
                    sendErrorEvent(instanceId as String, nil)
                    reject("error", "Download failed", nil)
                    return
                }

                prefetcher.prefetchKeys { error in
                    if let error {
                        self.sendErrorEvent(instanceId as String, error)
                        reject("error", error.message, error)
                        return
                    }
                }
            }
        }

        downloader.prepareDownload(download.uuid) { download, error in
            if error != nil {
                self.sendErrorEvent(instanceId as String, error)
                reject("error", error?.message, error)
                return
            }
            guard let download else {
                self.sendErrorEvent(instanceId as String, error)
                reject("error", "Unable to create a download", nil)
                return
            }
            DownloadInstanceManager.shared.setInstance(instanceId: instanceId, download: download)
            resolve(self.getDownloadableTracks(download))
        }
    }

    @objc
    func getState(
        _ instanceId: NSString!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let download = findDownload(instanceId as String) else {
            sendErrorEvent(instanceId as String, nil)
            reject("error", "Download not found!", nil)
            return
        }

        resolve(PRESTOUtils.mapDownloadState(download.state))
    }

    @objc
    func getLocalManifestUrl(
        _ instanceId: NSString!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let download = findDownload(instanceId as String) else {
            sendErrorEvent(instanceId as String, nil)
            reject("error", "Download not found!", nil)
            return
        }

        resolve(download.localUrl?.absoluteString ?? "")
    }

    @objc
    func getTracks(
        _ instanceId: NSString!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let download = findDownload(instanceId as String) else {
            sendErrorEvent(instanceId as String, nil)
            reject("error", "Download not found!", nil)
            return
        }

        resolve(getDownloadableTracks(download))
    }

    @objc
    func getProgress(
        _ instanceId: NSString!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let download = findDownload(instanceId as String) else {
            sendErrorEvent(instanceId as String, nil)
            reject("error", "Download not found!", nil)
            return
        }

        resolve(getProgressMap(download))
    }

    func didStateChange(_ download: Download) {
        if download.state == .success {
            sendCompleteEvent(download.title)
        } else if download.state == .failed {
            sendErrorEvent(download.title, nil)
        } else if download.state == .paused {
            sendStopEvent(download.title)
        } else if download.state == .running {
            sendStartEvent(download.title)
        }
    }

    func didProgressChange(_ download: Download) {
        sendProgressEvent(download.title)
    }

    /// Will be called when this module's first listener is added.
    override func startObserving() {
        hasListeners = true
    }

    /// Will be called when this module's last listener is removed, or on dealloc.
    override func stopObserving() {
        hasListeners = false
    }

    override func supportedEvents() -> [String]! {
        [
            EVENT_PREFIX + "-Start",
            EVENT_PREFIX + "-Stop",
            EVENT_PREFIX + "-Progress",
            EVENT_PREFIX + "-Error",
            EVENT_PREFIX + "-Complete",
        ]
    }

    // MARK: Private

    private func getDownloadableTracks(_ download: Download) -> [String: Any?] {
        let renditions: [VideoRendition] = download.videoTracks.flatMap { $0.renditions }
        return [
            "videoQualities": renditions.map { PRESTOUtils.mapVideoQuality($0) },
            "audioTracks": download.audioTracks.map { PRESTOUtils.mapAudioTrack($0) },
            "subtitleTracks": download.textTracks.map { PRESTOUtils.mapSubtitleTrack($0) },
        ]
    }

    private func findDownload(_ downloadId: String) -> Download? {
        let downloader = PRESTOplaySDK.shared.downloader()
        guard
            let download = downloader.getDownloads().first(where: { download in
                download.title == downloadId
            }) else
        {
            return nil
        }
        return download
    }

    private func getProgressMap(_ download: Download) -> [String: Any?] {
        var estimatedSize: Float? = download.progress.expectedContentSize
        var downloadedSize: Float? = download.progress.downloadedContentSize
        if estimatedSize == nil {
            estimatedSize = Float(download.progress.expectedContentDuration ?? 0.0)
            downloadedSize = Float(download.progress.downloadedContentDuration ?? 0.0)
        }
        let progress = ((downloadedSize ?? 0.0) / (estimatedSize ?? 1.0) * 100.0)
        return [
            "downloadId": download.title,
            "downloadedSize": downloadedSize,
            "estimatedSize": estimatedSize,
            "progress": progress,
        ]
    }

    private func sendStartEvent(_ downloadId: String) {
        let eventName = EVENT_PREFIX + "-Start"
        emitEvent([
            "downloadId": downloadId,
        ], eventName)
    }

    private func sendStopEvent(_ downloadId: String) {
        let eventName = EVENT_PREFIX + "-Stop"
        emitEvent([
            "downloadId": downloadId,
        ], eventName)
    }

    private func sendProgressEvent(_ downloadId: String) {
        guard let download = findDownload(downloadId) else {
            print("error", "Download not found!")
            return
        }

        if download.state == .running {
            let eventName = EVENT_PREFIX + "-Progress"
            emitEvent(getProgressMap(download), eventName);
        }
    }

    private func sendErrorEvent(_ downloadId: String, _ error: PRESTOplay.PRESTOError?) {
        let eventName = EVENT_PREFIX + "-Error"
        emitEvent([
            "downloadId": downloadId,
            "error": error,
        ], eventName);
    }

    private func sendCompleteEvent(_ downloadId: String) {
        let eventName = EVENT_PREFIX + "-Complete"
        emitEvent([
            "downloadId": downloadId,
        ], eventName)
    }

    private func emitEvent(_ event: [String: Any?]?, _ name: String) {
        guard hasListeners else { return }
        sendEvent(withName: name, body: event)
    }

}
