//
//  VideoUtils.swift
//  example
//
//  Created by Mackode - Bartlomiej Makowski on 13/04/2022.
//

import Foundation

import CastlabsYoubora
import NpawPlugin
import PRESTOplay

class PRESTOUtils: NSObject {
    static func mapState(_ state: PlaybackState) -> Int! {
        //   Idle = 0
        //   Preparing = 1
        //   Buffering = 2
        //   Pausing = 3
        //   Playing = 4
        //   Finished = 5

        switch state {
        case .idle:
            return 0
        case .opening, .ready:
            return 1
        case .buffering:
            return 2
        case .pausing, .paused, .stopping:
            return 3
        case .playing:
            return 4
        case .ended:
            return 5
        default:
            return 2
        }
    }

    static func mapDownloadState(_ state: DownloadState) -> Int! {
        //    Queued = 0
        //    Loading = 1
        //    Error = 2
        //    Done = 3
        //    Idle = 4

        switch state {
        case .idle:
            return 4
        case .success:
            return 3
        case .failed:
            return 2
        case .running:
            return 1
        case .paused:
            return 0
        default:
            return 4
        }
    }

    static func mapSubtitlesStyle(_ style: [String: Any]) -> SubtitlesStyle {
        let subtitlesStyle = SubtitlesStyle(fromSystemSettings: true)
        if let fontSize = style["fontSize"] as? Float {
            subtitlesStyle.fontSizeScale = fontSize
        }

        if let foregroundColor = style["foregroundColor"] as? Int {
            // unsupported
//            subtitlesStyle.foregroundColor = foregroundColor
        }

        if let backgroundColor = style["backgroundColor"] as? Int {
            // unsupported
//            subtitlesStyle.backgroundColor = backgroundColor
        }

        if let backgroundOpacity = style["backgroundOpacity"] as? Float {
            // unsupported
//            subtitlesStyle.backgroundOpacity = backgroundOpacity
        }

        if let foregroundOpacity = style["foregroundOpacity"] as? Float {
            // unsupported
//            subtitlesStyle.foregroundOpacity = foregroundOpacity
        }

        if let leftMargin = style["leftMargin"] as? Int {
            // unsupported
//            subtitlesStyle.leftMargin = leftMargin
        }

        if let rightMargin = style["rightMargin"] as? Int {
            // unsupported
//            subtitlesStyle.rightMargin = rightMargin
        }

        if let topMargin = style["topMargin"] as? Int {
            // unsupported
//            subtitlesStyle.topMargin = topMargin
        }

        if let bottomMargin = style["bottomMargin"] as? Int {
            // unsupported
//            subtitlesStyle.bottomMargin = bottomMargin
        }

        return subtitlesStyle
    }

    static func mapStats(_ stats: Stats) -> [String: Any] {
        var map: [String: Any] = [:]

        // basic metrics unified across Android and iOS

        map["droppedFrames"] = stats.numberOfDroppedVideoFrames > 0
            ? stats.numberOfDroppedVideoFrames
            : 0

        if stats.indicatedBitrate > 0 {
            map["streamBandwidth"] = stats.indicatedBitrate
        }

        if stats.observedBitrate > 0 {
            map["estimatedBandwidth"] = stats.observedBitrate
        } else if stats.numberOfBytesTransferred > 0, stats.transferDuration > 0 {
            let bits = Double(stats.numberOfBytesTransferred) * 8

            // Often, transferDuration is below 1 second and remains constant.
            // Once quality stabilizes, updates stop. This appears to be platform behavior.
            let seconds = stats.transferDuration
            map["estimatedBandwidth"] = bits / seconds
        }

        // extra metrics specific to iOS

        if stats.transferDuration >= 0 {
            map["transferDurationSec"] = stats.transferDuration
        }
        if stats.numberOfBytesTransferred >= 0 {
            map["numberOfBytesTransferred"] = stats.numberOfBytesTransferred
        }
        if stats.numberOfMediaRequests >= 0 {
            map["numberOfMediaRequests"] = stats.numberOfMediaRequests
        }
        if let timeIntervalSince1970 = stats.playbackStartDate?.timeIntervalSince1970 {
            map["playbackStartTimestamp"] =
                Int64(timeIntervalSince1970 * 1000) // miliseconds since epoch
        }
        if stats.startupTime >= 0 {
            map["startupTimeSec"] = stats.startupTime
        }
        if stats.durationWatched >= 0 {
            map["durationWatchedSec"] = stats.durationWatched
        }
        if stats.numberOfStalls >= 0 {
            map["numberOfStalls"] = stats.numberOfStalls
        }
        if stats.segmentsDownloadedDuration >= 0 {
            map["segmentsDownloadedDurationSec"] = stats.segmentsDownloadedDuration
        }
        if stats.observedBitrateStandardDeviation >= 0 {
            map["observedBitrateStandardDeviation"] = stats.observedBitrateStandardDeviation
        }
        if stats.observedBitrate >= 0 {
            map["observedBitrate"] = stats.observedBitrate
        }
        if stats.averageAudioBitrate >= 0 {
            map["averageAudioBitrate"] = stats.averageAudioBitrate
        }
        if stats.averageVideoBitrate >= 0 {
            map["averageVideoBitrate"] = stats.averageVideoBitrate
        }
        if stats.indicatedAverageBitrate >= 0 {
            map["indicatedAverageBitrate"] = stats.indicatedAverageBitrate
        }
        if stats.indicatedBitrate >= 0 {
            map["indicatedBitrate"] = stats.indicatedBitrate
        }

        return map
    }

    static func mapVideoQuality(_ videoRendition: VideoRendition) -> [String: Any] {
        var map: [String: Any] = [:]

        let bitrate = Int(videoRendition.bitrate)
        let width = Int(videoRendition.width)
        let height = Int(videoRendition.height)
        let codecs: String? = videoRendition.codec
        let mimeType: String? = nil // unsupported

        map["bitrate"] = bitrate
        map["width"] = width
        map["height"] = height
        map["codecs"] = codecs
        map["mimeType"] = mimeType

        return map
    }

    static func mapAudioTrack(_ audioTrack: AudioTrack) -> [String: Any] {
        var map: [String: Any] = [:]

        let bitrate = 0 // unsupported
        let label: String? = audioTrack.label
        let language: String? = audioTrack.language
        let codecs: String? = nil // unsupported
        let mimeType: String? = nil // unsupported

        map["bitrate"] = bitrate
        map["label"] = label
        map["language"] = language
        map["codecs"] = codecs
        map["mimeType"] = mimeType
        map["descriptors"] = mapProperties(audioTrack)

        return map
    }

    static func mapSubtitleTrack(_ textTrack: TextTrack) -> [String: Any] {
        var map: [String: Any] = [:]

        let label: String? = textTrack.label
        let language: String? = textTrack.language
        let mimeType: String? = nil // unsupported

        map["label"] = label
        map["language"] = language
        map["mimeType"] = mimeType
        map["descriptors"] = mapProperties(textTrack)

        return map
    }

    static func mapProperties(_ track: Track) -> [String: Any] {
        if track.properties["characteristics"] != nil {
            var descriptor: [String: Any] = [:]
            descriptor["value"] = track.properties["characteristics"]

            var descriptors: [String: Any] = [:]
            descriptors["HLS_CHARACTERISTICS"] = descriptor
            return descriptors
        } else {
            return [String: Any]()
        }
    }

    static func setupAnalytics(
        _ player: PlayerAPI,
        _ playerConfiguration: PlayerConfiguration,
        _ configJson: String
    ) {
        guard let data = configJson.data(using: .utf8) else { return }

        let responseDict = try? JSONSerialization.jsonObject(
            with: data,
            options: .allowFragments
        ) as? [String: Any]
        guard let dict = responseDict else { return }
        guard let analyticsMetaData = dict["analyticsMetaData"] as? NSDictionary else { return }
        setupYoubora(player, playerConfiguration, analyticsMetaData)
    }

    static func setupYoubora(
        _ player: PlayerAPI,
        _ playerConfiguration: PlayerConfiguration,
        _ analyticsConfig: NSDictionary
    ) {
        var youbora = PRESTOplaySDK.shared.analytics(for: player, .youbora)
        let youboraMetadata = YouboraMetadata(
            live: playerConfiguration.live ?? analyticsConfig["live"] as? Bool ?? false,
            assetId: analyticsConfig["assetId"] as? String ?? "assetId"
        )

        youboraMetadata.analyticsOptions = NpawPlugin.AnalyticsOptions()
        youboraMetadata.analyticsOptions?.contentTitle = analyticsConfig["assetId"] as? String

        youbora?.metadata = youboraMetadata
    }

    static func fixPlayerConfiguration(json: String) -> String {
        if "" == json {
            return json
        }

        var config = json
        guard let data = config.data(using: .utf8) else { return config }

        let responseDict = try? JSONSerialization.jsonObject(
            with: data,
            options: .allowFragments
        ) as? [String: Any]
        guard var dict = responseDict else { return config }

        // remove unsupported metadata
        dict.removeValue(forKey: "metaData")

        if
            let source = dict["source"] as? [String: Any],
            let url = source["url"] as? String
        {
            dict["contentUrl"] = url
            dict["contentType"] = "HLS"
            dict.removeValue(forKey: "source")
        }

        // RN-225
        if let preferredTextLanguage = dict["preferredTextLanguage"] as? String {
            dict["autoSelectTextTrackMode"] = AutoSelectTextTrackMode.anyMatchingTrack.rawValue
        }

        if let drmConfiguration = dict["drmConfiguration"] as? [String: Any] {
            var newDrmConfiguration = [String: Any]()

            newDrmConfiguration["environment"] = drmConfiguration["environment"]
            newDrmConfiguration["userId"] = drmConfiguration["userId"]
            newDrmConfiguration["sessionId"] = drmConfiguration["sessionId"]
            newDrmConfiguration["merchant"] = drmConfiguration["merchant"]
            newDrmConfiguration["assetId"] = drmConfiguration["assetId"]
            newDrmConfiguration["variantId"] = drmConfiguration["variantId"]
            newDrmConfiguration["authToken"] = drmConfiguration["authToken"]

            if let fairplayConfiguration = drmConfiguration["FAIRPLAY"] as? [String: Any] {
                newDrmConfiguration["licensingUrl"] = fairplayConfiguration["licensingUrl"]
                newDrmConfiguration["licensingParameters"] =
                    fairplayConfiguration["licensingParameters"]
                newDrmConfiguration["certificateUrl"] = fairplayConfiguration["certificateUrl"]
                newDrmConfiguration["certificateParameters"] =
                    fairplayConfiguration["certificateParameters"]
            }

            dict["drmConfiguration"] = newDrmConfiguration
        }

        if let sideloadTracks = dict["sideloadedTracks"] as? [[String: Any]] {
            var newSideloadedTracks = [[String: Any]]()

            for track in sideloadTracks {
                var newTrack = track
                let trackType = track["trackType"] as? String
                if let trackType, trackType == "SUBTITLE" {
                    newTrack["trackType"] = "text"
                } else if trackType == "THUMBNAIL" {
                    newTrack["trackType"] = "thumbnail"
                }
                newSideloadedTracks.append(newTrack)
            }
            dict["sideloadedTracks"] = newSideloadedTracks
        }

        // serialize the configuration again
        if let jsonData = try? JSONSerialization.data(withJSONObject: dict, options: []) {
            config = String(data: jsonData, encoding: .utf8)!
        }

        return config
    }
}
