//
//  RNError.swift
//  react-native-prestoplay-module
//
//  Created by Taewoo Kim on 23/06/2023.
//

import Foundation
import PRESTOplay

struct RNError {

    // MARK: Lifecycle

    init(
        severity: RNErrorSeverity,
        type: Int,
        message: String,
        httpCode: Int? = 0,
        url: String? = nil,
        responseBody: String? = nil
    ) {
        self.severity = severity
        self.type = .unknown
        self.message = message
        self.httpCode = httpCode
        self.url = url
        self.responseBody = responseBody

        if let errType = ErrorType(rawValue: type) {
            self.type = .getErrorMap(from: errType)
        }
    }

    init(clError: PRESTOplay.PRESTOError?) {
        guard let clError else {
            severity = .warning
            type = .unknown
            message = "Unknown error"
            return
        }

        // Align iOS SDK severity values with their Android SDK counterparts.
        // Android SDK severity values are 0 (info), 1 (warning), 2 (error).
        // iOS SDK severity values are 1 (warning), 2 (recoverable), 3 (fatal).
        // https://players.castlabs.com/android/4.2.81/docs/javadoc/player_sdk/constant-values.html

        switch clError.severity {
        case .warning:
            severity = .warning
            break;
        case .recoverable:
            severity = .recoverable
            break;
        default:
            severity = .fatal
            break;
        }

        type = .getErrorMap(from: clError.type)

        if let message = clError.message, false == message.isEmpty {
            self.message = message
        } else {
            message = clError.parentError?.localizedDescription ?? ""
        }

        if let code = clError.data["httpCode"] {
            httpCode = Int(code)
        }

        url = clError.data["url"]
        responseBody = clError.data["responseBody"]
    }

    // MARK: Internal

    enum ErrorMap: Int {
        // Android error codes
        case unknown = 0
        case audioTrackInitialization = 1
        case audioDecoderInitialization = 2
        case audioWriteError = 3
        case audioDecryptionError = 4
        case videoDecoderInitialization = 5
        case videoDecryptionError = 6
        case manifestLoadingFailed = 7
        case sdkInitError = 8
        case noRendererFound = 9
        case noPlayableContent = 10
        case playbackError = 11
        case apiMismatch = 12
        case unsupportedDRM = 13
        case sdkNotInitialized = 14
        case drmTodayException = 15
        case downloadError = 16
        case secondaryDisplay = 17
        case keyExpired = 18
        case drmKeyDownloadError = 19
        case connectivityLostError = 20
        case connectivityGainedInfo = 21
        case behindLiveWindow = 22
        case invalidPlayerLicence = 23
        case dataDownloadError = 24
        case drmException = 25
        case audioUnsupported = 26
        case videoUnsupported = 27
        case textUnsupported = 28
        case userIdNotProvided = 29
        case manifestParsingFailed = 30
        case adRequestUnsupported = 31
        case drmProvisioningError = 32
        case cslLimitReached = 33
        case cslNetworkError = 34
        case hdcpConnectionWarning = 35
        case adError = 36
        case dnsServerError = 37
        case timeoutError = 38
        case manifestInvalid = 39


        // iOS specific error codes (starting from 1000)

        // Storage errors
        case storageRemoveError = 1000
        case storageReadError = 1001
        case storageWriteError = 1002

        // Generic iOS error/ can include FPS errors also
        case system = 1003

        // Player failed to open session
        case playerOpenError = 1004
        // Generic AVPlayer error
        case playerAvPlayerError = 1005

        // DRM errors
        case drmFairplayError = 1006
        case drmMalformedRequestError = 1007

        case networkRequestCancelled = 1008

        // (sideloaded) Subtitles error
        case textContentInvalid = 1009 // or textUnsupported
        case textTrackAlreadyLoaded = 1010

        // Thumbnails error
        case thumbnailsContentInvalid = 1011
        case thumbnailsImageMissing = 1012


        // MARK: Internal

        /// Get the associated error code
        static func getErrorMap(from type: PRESTOplay.ErrorType) -> ErrorMap {
            var result = unknown
            switch type {
            case .downloader_id_already_in_use,
                 .downloader_id_not_found,
                 .downloader_no_content_to_be_removed,
                 .downloader_not_available_for_this_format,
                 .downloader_request_creation_failed,
                 .downloader_asset_tracks_cannot_be_loaded,
                 .downloader_media_selection_cannot_be_loaded,
                 .downloader_offline_key_not_retrieved:
                result = .downloadError
            case .storage_error_while_removing_file:
                result = .storageRemoveError
            case .storage_error_reading:
                result = .storageReadError
            case .storage_error_writing:
                result = .storageWriteError
            case .network_error:
                result = .downloadError
            case .network_http_bad_status_code:
                result = .downloadError // includes AVPlayer network errors
            case .player_resource_unavailable:
                result = .downloadError
            case .method_not_implemented:
                result = .unknown
            case .hls_playlist_invalid:
                result = .manifestParsingFailed
            case .url_not_set:
                result = .unknown // TODO:
            case .player_open_error:
                result = .playerOpenError
            case .player_avplayer_error:
                result = .playerAvPlayerError
            case .drm_fairplay_key_error:
                result = .drmFairplayError
            case .drm_malformed_request,
                 .drm_missing_licensingUrl,
                 .drm_missing_keyUri:
                result = .drmMalformedRequestError
            case .drm_local_internal_error,
                 .drm_server_internal_error,
                 .drm_server_licensing_error:
                result = .drmException
            case .drm_license_expired:
                result = .keyExpired
            case .drm_second_screen_detected:
                result = .secondaryDisplay
            case .drm_screen_recording_detected:
                result = .secondaryDisplay
            case .drm_unsupported_system:
                result = .apiMismatch
            case .system:
                result = .system
            case .sdk_license_invalid:
                result = .sdkInitError
            case .ads_error:
                result = .adError
            case .network_request_cancelled:
                result = networkRequestCancelled
            case .text_content_invalid:
                result = textContentInvalid
            case .text_track_already_loaded:
                result = textTrackAlreadyLoaded
            case .thumbnails_content_invalid:
                result = thumbnailsContentInvalid
            case .thumbnails_image_missing:
                result = thumbnailsImageMissing
            case .player_avplayer_stall:
                result = timeoutError
            case .system_internet_connection_available:
                result = connectivityGainedInfo
            case .system_internet_connection_unavailable:
                result = connectivityLostError
            case .cast_error:
                result = playbackError
            @unknown default:
                result = .unknown
            }

            return result
        }
    }

    var severity: RNErrorSeverity
    var type: ErrorMap
    var message: String
    var httpCode: Int?
    var url: String?
    var responseBody: String?

    var dictionary: [String:Any] {
        [
            "severity": severity.rawValue,
            "type": type.rawValue,
            "message": message,
            "httpCode": httpCode ?? 0,
            "url": url ?? NSNull(),
            "responseBody": responseBody ?? NSNull(),
        ]
    }
}
