//
//  VideoModule.swift
//  react-native-prestoplay-module
//
//  Created by Taewoo Kim on 23/06/2023.
//

import AVFoundation
import CoreMedia
import Foundation

import CastlabsApple
import CastlabsThumbnails
import PRESTOplay
#if false
import CastlabsYoubora
import YouboraLib
#endif
#if !os(tvOS)
import CastlabsDownloader
#endif

private let PLAYER_EVENT = "onPlayerEvent"
private let ACTION_EVENT = "onActionEvent"

// MARK: - TVPlayerModule

@available(tvOS 15.0, *)
@objc(TVPlayerModule)
class TVPlayerModule: RCTEventEmitter {

    // MARK: Lifecycle

    override init() {
        super.init()
        EventEmitter.sharedInstance.registerEventEmitter(eventEmitter: self)
    }

    // MARK: Open

    @objc
    open override func supportedEvents() -> [String] {
        [PLAYER_EVENT, ACTION_EVENT]
    }

    @objc
    open override func startObserving() {
        hasListeners = true
    }

    @objc
    open override func stopObserving() {
        hasListeners = false
    }

    // MARK: Internal

    var hasListeners = false
    var playerViewController: PlayerViewControllerAPI?
    var playerConfig: PlayerConfiguration?
    var semaphore: DispatchSemaphore?

    @objc
    override static func requiresMainQueueSetup() -> Bool {
        true
    }

    @objc
    func openPlayer(
        _ streamInfos: [NSDictionary],
        options: NSDictionary? = nil,
        transportBarActions: [NSDictionary]? = nil,
        infoTabActions: [NSDictionary]? = nil,
        contextualActions: [NSDictionary]? = nil,
        loopStreams: Bool,
        callback: @escaping RCTResponseSenderBlock
    ) {
        DispatchQueue.main.async { [weak self] in
            guard let self else { return }
            self.open(
                streamInfos,
                options: options,
                transportBarActions: transportBarActions,
                infoTabActions: infoTabActions,
                contextualActions: contextualActions,
                loopStreams: loopStreams,
                callback: callback
            )
        }
    }

    @objc
    func play() {
        DispatchQueue.main.async { [weak self] in
            guard let self else { return }
            self.playerViewController?.play()
        }
    }

    @objc
    func pause() {
        DispatchQueue.main.async { [weak self] in
            guard let self else { return }
            self.playerViewController?.pause()
        }
    }

    @objc
    func closePlayer() {
        DispatchQueue.main.async { [weak self] in
            guard let self else { return }
            self.playerViewController?.close()
        }
    }

    @objc
    func setTransportBarActions(_ actionsInfo: [NSDictionary]?) {
        #if os(tvOS)
        guard let actionsInfo else { return }

        DispatchQueue.main.async { [weak self] in
            guard let playerViewController = self?.playerViewController else { return }

            let actions: [AnyObject]? = actionsInfo.compactMap {
                if let popupMenu = ActionPopupMenuInfo(from: $0) {
                    return popupMenu
                } else if let action = ActionButtonInfo(from: $0) {
                    return action
                } else {
                    return nil
                }
            }

            if let actions {
                playerViewController.setTransportBarActions(actions, eventName: ACTION_EVENT)
            }
        }
        #endif
    }

    @objc
    func setInfoTabActions(_ actionsInfo: [NSDictionary]?) {
        #if os(tvOS)
        guard let actionsInfo else { return }

        DispatchQueue.main.async { [weak self] in
            guard let playerViewController = self?.playerViewController else { return }

            let actions: [ActionButtonInfo]? = actionsInfo.compactMap {
                ActionButtonInfo(from: $0)
            }

            if let actions {
                playerViewController.setInfoTabActions(actions, eventName: ACTION_EVENT)
            }
        }
        #endif
    }

    @objc
    func setContextualActions(_ actionsInfo: [NSDictionary]?) {
        #if os(tvOS)
        guard let actionsInfo else { return }

        DispatchQueue.main.async { [weak self] in
            guard let playerViewController = self?.playerViewController else { return }

            let actions: [ContextualActionButtonInfo]? = actionsInfo.compactMap {
                ContextualActionButtonInfo(from: $0)
            }

            if let actions {
                playerViewController.setContextualActions(actions, eventName: ACTION_EVENT)
            }
        }
        #endif
    }

    @objc
    func setActionState(_ actionId: String, state: Bool) {
        #if os(tvOS)
        DispatchQueue.main.async { [weak self] in
            guard let playerViewController = self?.playerViewController else { return }
            playerViewController.setActionState(
                ofId: actionId,
                state: state,
                title: nil,
                imageName: nil
            )
        }
        #endif
    }

    @objc
    func setActionStateAndUi(
        _ actionId: String,
        state: Bool,
        uiInfo: NSDictionary
    ) {
        #if os(tvOS)
        DispatchQueue.main.async { [weak self] in
            guard let playerViewController = self?.playerViewController else { return }
            playerViewController.setActionState(
                ofId: actionId,
                state: state,
                title: uiInfo["title"] as? String,
                imageName: uiInfo["imageName"] as? String
            )
        }
        #endif
    }

    @objc
    func setMetadata(_ metadataInfo: String) {
        guard let metadata = ChannelMetadata(from: metadataInfo) else { return }

        DispatchQueue.main.async { [weak self] in
            guard let self else { return }
            self.playerViewController?.setChannelMetadata(metadata)
        }
    }

    @objc
    func setOptions(_ options: NSDictionary?) {
        guard let options else { return }

        DispatchQueue.main.async { [weak self] in
            guard let playerViewController = self?.playerViewController else { return }
            if let value = options["allowsPictureInPicturePlayback"] as? Bool {
                playerViewController.allowsPictureInPicturePlayback = value
            }
            if let value = options["closeWhenPlaybackEnd"] as? Bool {
                playerViewController.closeWhenPlaybackEnd = value
            }
            if let value = options["requiresLinearPlayback"] as? Bool {
                playerViewController.requiresLinearPlayback = value
            }
            #if os(tvOS)
            if let value = options["disableChannelSkipInLiveStream"] as? Bool {
                playerViewController.disableChannelSkipInLiveStream = value
            }
            #endif
        }
    }

    @objc
    func seekToSecond(_ second: Double) {
        DispatchQueue.main.async { [weak self] in
            guard let playerViewController = self?.playerViewController else { return }
            let time = CMTimeMake(value: Int64(second), timescale: 1)
            playerViewController.seek(time)
        }
    }

    @objc
    func setRate(_ rate: Double) {
        DispatchQueue.main.async { [weak self] in
            guard let playerViewController = self?.playerViewController else { return }
            playerViewController.setRate(rate)
        }
    }

    @objc
    func getPosition(_ resolve: RCTPromiseResolveBlock, rejecter reject: RCTPromiseRejectBlock) {
        guard let playerViewController else {
            reject("error", "Player is not initialized", nil)
            return
        }

        let cmtime = playerViewController.getCurrentTime()
        var currentPosition = CMTimeGetSeconds(cmtime)

        guard !currentPosition.isNaN else {
            reject("error", "Unable to get position", nil)
            return
        }

        if let isLive = playerViewController.isLive, isLive == true {
            guard
                playerViewController.getLiveStartTime() != nil,
                let lastSeekWindow = playerViewController.seekableTimeRanges?.last else
            {
                reject("error", "Unable to get live position", nil)
                return
            }
            currentPosition = lastSeekWindow.duration.seconds
        }
        resolve(NSNumber(value: currentPosition))
    }

    @objc
    func getLiveStartTime(
        _ resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let playerViewController else {
            reject("Player is not initialized.", nil, nil)
            return
        }

        guard let liveStartTime = playerViewController.getLiveStartTime() else {
            resolve("")
            return
        }
        resolve(liveStartTime)
    }

    @objc
    func getAudioTrack(_ resolve: RCTPromiseResolveBlock, rejecter reject: RCTPromiseRejectBlock) {
        guard let playerViewController else {
            reject("error", "Player is not initialized", nil)
            return
        }

        if let audioTrack = playerViewController.getAudioTrack() {
            resolve(PRESTOUtils.mapAudioTrack(audioTrack))
            return
        }
        resolve("")
    }

    @objc
    func getSubtitleTrack(
        _ resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let playerViewController else {
            reject("error", "Player is not initialized", nil)
            return
        }

        if let textTrack = playerViewController.getTextTrack() {
            resolve(PRESTOUtils.mapSubtitleTrack(textTrack))
            return
        }
        resolve("")
    }

    @objc
    func getCurrentStreamIndex(
        _ resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard let playerViewController else {
            reject("error", "Player is not initialized", nil)
            return
        }

        let index = playerViewController.getCurrentChannelIndex()
        resolve(index)
    }
}

// MARK: PlayerViewControllerProtocol

@available(tvOS 15.0, *)
extension TVPlayerModule: PlayerViewControllerProtocol {
    func action(_: PlayerViewControllerAPI, ofId id: String, eventName: String, data: String) {
        if true == hasListeners {
            let body: [String: Any] = [
                "id": id,
                "data": data,
            ]
            sendEvent(withName: eventName, body: body)
        }
    }

    func restoreUserInterfaceForPictureInPictureStop(
        _ playerViewController: PlayerViewControllerAPI,
        completion: @escaping (Bool) -> Void
    ) {
        // Even when the playback is ended in PiP mode, this event is called.
        // In that case, it does not need to present the PlayerViewController.
        if
            playerViewController.playbackState == nil ||
            playerViewController.playbackState == .ended ||
            playerViewController.playbackState == .error
        {
            completion(true)
            return
        }

        // Close the current full screen PlayerViewController.
        if
            let selfPvc = self.playerViewController,
            selfPvc != playerViewController as UIViewController
        {
            selfPvc.dismiss(animated: false)
        }

        // Present the PiP PlayerViewController.
        RCTPresentedViewController()?
            .present(playerViewController, animated: false) { [weak self] in
                self?.playerViewController = playerViewController
                completion(true)
            }
    }

    func didPlayToEnd(_: PlayerViewControllerAPI) {
        let body: [String: Any] = [
            "id": "did-play-to-end",
        ]
        sendEvent(withName: PLAYER_EVENT, body: body)
    }

    func playerIsClosed(_: PlayerViewControllerAPI) {
        playerViewController = nil
        let body: [String: Any] = [
            "id": "player-is-closed",
        ]
        sendEvent(withName: PLAYER_EVENT, body: body)
    }

    func willSeekFrom(
        _: PlayerViewControllerAPI,
        old oldTime: CMTime,
        to targetTime: CMTime,
        isBeyondSeekableRange beyondSeekableRange: Bool
    ) {
        let body: [String: Any] = [
            "id": "will-seek",
            "oldSeconds": CMTimeGetSeconds(oldTime),
            "targetSeconds": CMTimeGetSeconds(targetTime),
            "beyondSeekableRange": beyondSeekableRange,
        ]
        sendEvent(withName: PLAYER_EVENT, body: body)
    }

    func didSeekFrom(
        _: PlayerViewControllerAPI,
        old oldTime: CMTime,
        to targetTime: CMTime
    ) {
        let body: [String: Any] = [
            "id": "did-seek",
            "oldSeconds": CMTimeGetSeconds(oldTime),
            "targetSeconds": CMTimeGetSeconds(targetTime),
        ]
        sendEvent(withName: PLAYER_EVENT, body: body)
    }

    func timeControlStatus(
        _: PlayerViewControllerAPI,
        status: AVPlayer.TimeControlStatus,
        description: String
    ) {
        let body: [String: Any] = [
            "id": "time-control-status",
            "status": status.rawValue,
            "description": description,
        ]
        sendEvent(withName: PLAYER_EVENT, body: body)
    }

    func setupAnalytics(
        _: PlayerViewControllerAPI,
        for player: PlayerAPI
    ) {
        if let playerConfig {
            PRESTOUtils.setupAnalytics(player, playerConfig, "")
        }
    }

    func playbackStateError(
        _: PlayerViewControllerAPI,
        error: PRESTOplay.PRESTOError?
    ) {
        let rnError = RNError(clError: error)
        let body: [String: Any] = [
            "id": "playback-state-error",
            "error": rnError.dictionary,
        ]
        sendEvent(withName: PLAYER_EVENT, body: body)
    }

    func didAudioTrackChange(
        _: PRESTOplay.PlayerViewControllerAPI,
        audioTrack: AudioTrack?
    ) {
        if let audioTrack {
            let body: [String: Any] = [
                "id": "audio-track-changed",
                "track": PRESTOUtils.mapAudioTrack(audioTrack),
            ]
            sendEvent(withName: PLAYER_EVENT, body: body)
        } else {
            sendEvent(withName: PLAYER_EVENT, body: NSNull())
        }
    }

    func didTextTrackChange(
        _: PRESTOplay.PlayerViewControllerAPI,
        textTrack: TextTrack?
    ) {
        if let textTrack {
            let body: [String: Any] = [
                "id": "subtitle-track-changed",
                "track": PRESTOUtils.mapSubtitleTrack(textTrack),
            ]
            sendEvent(withName: PLAYER_EVENT, body: body)
        } else {
            sendEvent(withName: PLAYER_EVENT, body: NSNull())
        }
    }

    func didChannelChange(
        _: PRESTOplay.PlayerViewControllerAPI,
        index: Int
    ) {
        let body: [String: Any] = [
            "id": "stream-changed",
            "streamIndex": index,
        ]
        sendEvent(withName: PLAYER_EVENT, body: body)
    }

}

// MARK: - TVPlayerModule

@available(tvOS 15.0, *)
extension TVPlayerModule {
    func open(
        _ streamInfos: [NSDictionary],
        options: NSDictionary? = nil,
        transportBarActions: [NSDictionary]? = nil,
        infoTabActions: [NSDictionary]? = nil,
        contextualActions: [NSDictionary]? = nil,
        loopStreams: Bool = false,
        callback: @escaping RCTResponseSenderBlock
    ) {
        let channelItems: [ChannelItem] = streamInfos.compactMap { info in
            let configJson = info["playerConfig"] as? String ?? ""
            let fixedConfig = PRESTOUtils.fixPlayerConfiguration(json: configJson)

            guard let playerConfig = PlayerConfiguration.create(from: fixedConfig) else {
                print("Error: failed to convert the JSON to PlayerConfiguration: \(configJson))")
                return nil
            }

            let metadata = ChannelMetadata(from: info["channelMetadata"] as? String ?? "")
            let nextChannelInfo = NextChannelInfo(from: info["nextChannelInfo"] as? String ?? "")

            return ChannelItem(
                playerConfig: playerConfig,
                metadata: metadata,
                nextChannelInfo: nextChannelInfo
            )
        }

        if true == channelItems.isEmpty {
            let rctError = RCTMakeError(
                "There are no valid player items in \(streamInfos).",
                nil,
                nil
            )
            callback([rctError])
        }

        // Check if a single item is passsed otherwise recreate the player
        var itemReplacementStatus = false
        if channelItems.count == 1, let playerViewController {
            if let semaphore {
                semaphore.signal()
            }
            itemReplacementStatus = playerViewController
                .replaceCurrentChannel(with: channelItems.first!)
        }

        // Item replacement was unsuccesful, recreate the player view controller
        if !itemReplacementStatus {
            playerViewController = PRESTOplaySDK.shared.playerViewController(
                channels: channelItems,
                analytics: nil,
                loopChannels: loopStreams
            )
        }

        guard let playerViewController else {
            let rctError = RCTMakeError("Failed to create a player.", nil, nil)
            callback([rctError])
            return
        }

        playerViewController.playerDelegate = self
        setOptions(options)
        setTransportBarActions(transportBarActions)
        setInfoTabActions(infoTabActions)
        setContextualActions(contextualActions)
        playerViewController.play()

        // Present the view only if the player veiw was recreated
        if !itemReplacementStatus {
            RCTPresentedViewController()?
                .present(playerViewController, animated: false) {
                    callback([NSNull()])
                }
        } else {
            // The item replacement was successful and we need to resolve
            callback([NSNull()])
        }
    }

    func waitChannelChanged(_ completion: @escaping (Bool) -> Void) {
        semaphore = DispatchSemaphore(value: 0)
        let timeout = DispatchTime.now() + DispatchTimeInterval.seconds(3)
        if semaphore?.wait(timeout: timeout) == .timedOut {
            DispatchQueue.main.async {
                completion(false)
            }
        } else {
            DispatchQueue.main.async {
                completion(true)
            }
        }
        semaphore = nil
    }

    func skipToNextChannel(
        _: PlayerViewControllerAPI,
        completion: @escaping (Bool) -> Void
    ) {
        let body: [String: Any] = ["id": "skip-to-next-channel"]
        sendEvent(withName: PLAYER_EVENT, body: body)

        DispatchQueue.global().async { [weak self] in
            guard let self else { return }
            self.waitChannelChanged(completion)
        }
    }

    func skipToPreviousChannel(
        _: PlayerViewControllerAPI,
        completion: @escaping (Bool) -> Void
    ) {
        let body: [String: Any] = ["id": "skip-to-previous-channel"]
        sendEvent(withName: PLAYER_EVENT, body: body)

        DispatchQueue.global().async { [weak self] in
            guard let self else { return }
            self.waitChannelChanged(completion)
        }
    }
}
