//
//  ThumbnailView.swift
//  react-native-prestoplay-module
//
//  Created by Mackode - Bartlomiej Makowski on 13/04/2022.
//

import CastlabsThumbnails
import CoreMedia
import Foundation
import PRESTOplay
import UIKit

@objc
public class ThumbnailView: UIImageView {

    // MARK: Lifecycle

    override init(frame: CGRect) {
        super.init(frame: frame)
        superview?.bringSubviewToFront(self)
    }

    required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    // MARK: Public

    public override func layoutSubviews() {
        onViewCreatedEvent()
    }

    override public func willMove(toSuperview newSuperview: UIView?) {
        guard nil == newSuperview
        else { return } // Remove from superview - unmount in react-native screen
        thumbnails = nil
    }

    // MARK: Internal

    var thumbnails: ThumbnailsAPI?

    @objc var onLayout: RCTDirectEventBlock?
    @objc var onViewCreated: RCTDirectEventBlock?

    func createThumbnails(_ videoView: VideoView, _ instance: Int, _ thumnailId: Int) {
        guard thumbnails == nil else { return }
        guard let player = videoView.getPlayer() else { return }
        guard
            let sideloadedTrack = videoView.getPlayerConfiguration()?.sideloadedTracks?.filter({
                $0.trackType == .thumbnail
            }).first else
        {
            return
        }

        thumbnails = PRESTOplaySDK.shared.thumbnails(for: player)

        if
            let gridWidth = sideloadedTrack.gridWidth,
            let gridHeight = sideloadedTrack.gridHeight,
            let intervalMs = sideloadedTrack.intervalMs
        {
            let gridThumbnail = GridThumbnail(
                baseUrl: sideloadedTrack.url,
                gridHeight: gridWidth,
                gridWidth: gridHeight,
                durationMs: Int64(intervalMs)
            )
            thumbnails?.loadThumbnailsFrom(gridThumbnail: gridThumbnail, completionHandler: nil)
        } else {
            thumbnails?.loadThumbnailsFrom(webVTTtrack: sideloadedTrack.url, completionHandler: nil)
        }

        ThumbnailInstanceManager.shared.addInstance(instanceId: instance, thumbnailView: self)
        ThumbnailInstanceManager.shared.removeInstance(instanceId: thumnailId)

        if let videoView = VideoInstanceManager.shared.getInstance(instanceId: instance) {
            videoView.onThumbnailLoadedEvent(0, 0, 0)
        }
    }

    func updateThumbnails(_ position: Float64, _ videoView: VideoView) {
        if
            let thumb = thumbnails?
                .getThumbnail(atTimestamp: CMTime(seconds: position, preferredTimescale: 1))
        {
            let image = UIImage(data: thumb.imageData)
            DispatchQueue.main.async {
                self.image = image
                self.onLayoutEvent(
                    Int(self.frame.origin.x),
                    Int(self.frame.origin.y),
                    Int(self.frame.width),
                    Int(self.frame.height)
                )
                self.superview?.bringSubviewToFront(self)
            }

            videoView.onThumbnailLoadedEvent(
                thumb.timeStampEnd.seconds,
                Int(image?.size.width ?? 0),
                Int(image?.size.height ?? 0)
            )
        }
    }

    @objc
    func onLayoutEvent(_ x: Int, _ y: Int, _ width: Int, _ height: Int) {
        onLayout?([
            "layout": [
                "x": x,
                "y": y,
                "width": width,
                "heigth": height,
            ],
        ])
    }

    @objc
    func onViewCreatedEvent() {
        ThumbnailInstanceManager.shared.addInstance(instanceId: hashValue, thumbnailView: self)
        onViewCreated?([
            "instanceId": hashValue,
        ])
    }

}
