//
//  VideoModule.swift
//  react-native-prestoplay-module
//
//  Created by Mackode - Bartlomiej Makowski on 14/04/2022.
//

import CoreMedia
import Foundation

import CastlabsApple
import CastlabsSubtitles
import CastlabsThumbnails
import CastlabsYoubora
import PRESTOplay

#if !os(tvOS)
import CastlabsDownloader
#endif

// MARK: - VideoModule

@objc(VideoModule)
class VideoModule: NSObject {

    // MARK: Lifecycle

    override init() {
        super.init()
        VideoModule.shared = self
    }

    // MARK: Internal

    static var shared: VideoModule?

    private(set) var isAuthenticated = false
    private(set) var initData: NSDictionary?

    @objc
    static func requiresMainQueueSetup() -> Bool {
        true
    }

    @objc
    func initialize(
        _ sdkConfig: NSDictionary,
        companionSdkInfo: NSDictionary,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        initData = sdkConfig

        guard false == isAuthenticated else {
            resolve("successful")
            return
        }

        #if os(tvOS)
        var plugins: [CLPluginProtocol] = [HLSPlugin(), ThumbnailsPlugin()]
        #else
        var plugins: [CLPluginProtocol] = [
            HLSPlugin(),
            ThumbnailsPlugin(),
            DownloaderPlugin(),
            SubtitlesPlugin(),
        ]
        #endif

        // swiftlint:disable force_cast
        let companion = CompanionSDK(
            name: companionSdkInfo["name"] as! String,
            version: companionSdkInfo["version"] as! String
        )
        // swiftlint:enable force_cast

        if let analytics = sdkConfig["analytics"] as? NSDictionary {
            if
                let youbora = analytics["youbora"] as? NSDictionary,
                let accountCode = youbora["accountCode"] as? String
            {
                let youboraSettings = YouboraSettings(accountCode: accountCode)
                plugins.append(YouboraPlugin(youboraSettings))
            } else {
                let message = "Couldn't find accountCode for Youbora analytics"
                reject("error", message, nil)
                return
            }
        }

        var result: LicenseCheck = .invalid_license
        let group = DispatchGroup()
        group.enter()
        DispatchQueue.main.async {
            result = PRESTOplaySDK.shared.register(self.licenseKey, plugins, companion)
            group.leave()
        }
        group.wait()

        isAuthenticated = (.ok == result)
        if isAuthenticated {
            resolve("successful")
        } else {
            print("Authentication failed, \(result.rawValue)")
            reject("error", result.rawValue, nil)
        }
    }

    @objc
    func getPlayerState(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        if let state = videoView.getPlaybackState() {
            resolve(PRESTOUtils.mapState(state))
            return
        }
        resolve("")
    }

    @objc
    func getDuration(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        resolve(videoView.getDuration())
    }

    @objc
    func isLive(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        resolve(videoView.isLive())
    }

    @objc
    func getPosition(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        guard var currentPosition = videoView.getPosition() else {
            reject("error", "Unable to get position", nil)
            return
        }

        // recalculate position in case of live content
        if videoView.isLive() {
            guard
                videoView.getLiveStartTime() != nil,
                let lastSeekWindow = videoView.getSeekableTimeRanges().last else
            {
                reject("error", "Unable to get live position", nil)
                return
            }
            currentPosition = lastSeekWindow.duration.seconds
        }
        resolve(currentPosition)
    }

    @objc
    func getProgressPercentage(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        var progress = 0.0
        if let duration = videoView.getDuration() {
            if duration > 0, let position = videoView.getPosition() {
                progress = position / Double(duration) * 100.0
            }
        }
        resolve(Int(progress))
    }

    @objc
    func getBufferedPercentage(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        resolve(Int(videoView.getBufferedPercentage() ?? 0.0))
    }

    @objc
    func setPosition(
        _ instanceId: NSNumber!,
        position: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard 0.0 <= position.doubleValue else { return }
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        videoView.seek(CMTime(seconds: position.doubleValue, preferredTimescale: 1))
        videoView.onSeekToEvent(position.doubleValue)
        resolve("")
    }

    @objc
    func getVolume(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        resolve(videoView.getPlaybackVolume())
    }

    @objc
    func setVolume(
        _ instanceId: NSNumber!,
        volume: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        videoView.setPlaybackVolume(volume.doubleValue)
        resolve("")
    }

    @objc
    func getStats(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        var statsMap = [String: Any]()

        if let stats = videoView.getStats() {
            statsMap = PRESTOUtils.mapStats(stats)
        }
        resolve(statsMap)
    }

    @objc
    func getLiveStartTime(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        guard let liveStartTime = videoView.getLiveStartTime() else {
            resolve("")
            return
        }
        resolve(liveStartTime)
    }

    @objc
    func getVideoQuality(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        if let videoRendition = videoView.getRenderedVideoRendition() {
            resolve(PRESTOUtils.mapVideoQuality(videoRendition))
            return
        }
        resolve("")
    }

    @objc
    func getVideoQualityMode(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        resolve(videoView.videoQualityMode)
    }

    @objc
    func setVideoQuality(
        _ instanceId: NSNumber!,
        index: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        if 0 <= index.intValue {
            videoView.setVideoRendition(videoView.videoRenditions[index.intValue])
        } else {
            videoView.setVideoRendition(nil)
        }
        resolve("")
    }

    @objc
    func getVideoQualities(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        var videoQualities: [[String: Any]] = []
        videoView.videoRenditions.forEach { videoRendition in
            let videoQuality = PRESTOUtils.mapVideoQuality(videoRendition)
            videoQualities.append(videoQuality)
        }
        resolve(videoQualities)
    }

    @objc
    func getAudioTrack(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        if let audioTrack = videoView.getAudioTrack() {
            resolve(PRESTOUtils.mapAudioTrack(audioTrack))
            return
        }
        resolve("")
    }

    @objc
    func setAudioTrack(
        _ instanceId: NSNumber!,
        index: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        if 0 <= index.intValue {
            if let tracks = videoView.getTracks() {
                DispatchQueue.main.async {
                    videoView.setAudioTrack(tracks.audio[index.intValue])
                }
            }
        }
        resolve("")
    }

    @objc
    func getAudioTracks(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        var audioTracks: [[String: Any]] = []
        if let tracks = videoView.getTracks() {
            tracks.audio.forEach { audioTrack in
                let audioTrack = PRESTOUtils.mapAudioTrack(audioTrack)
                audioTracks.append(audioTrack)
            }
        }
        resolve(audioTracks)
    }

    @objc
    func getSubtitleTrack(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        if let textTrack = videoView.getTextTrack() {
            resolve(PRESTOUtils.mapSubtitleTrack(textTrack))
            return
        }
        resolve("")
    }

    @objc
    func setSubtitleTrack(
        _ instanceId: NSNumber!,
        index: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        DispatchQueue.main.async {
            if 0 <= index.intValue {
                videoView.setTextTrack(videoView.getTracks()?.text[index.intValue])
            } else {
                videoView.setTextTrack(nil)
            }
        }
        resolve("")
    }

    @objc
    func getSubtitleTracks(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        var subtitleTracks: [[String: Any]] = []
        videoView.getTracks()?.text.forEach { textTrack in
            let subtitleTrack = PRESTOUtils.mapSubtitleTrack(textTrack)
            subtitleTracks.append(subtitleTrack)
        }
        resolve(subtitleTracks)
    }

    @objc
    func setSubtitleStyle(
        _ instanceId: NSNumber!,
        style: [String: Any]!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        videoView.setSubtitlesStyle(PRESTOUtils.mapSubtitlesStyle(style))
        resolve("")
    }

    @objc
    func getThumbnailData(
        _ instanceId: NSNumber!,
        position: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        if
            let thumbnailView = ThumbnailInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue)
        {
            thumbnailView.updateThumbnails(position.doubleValue, videoView)
        }
        resolve("")
    }

    @objc
    func setThumbnail(
        _ instanceId: NSNumber!,
        thumbnailId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        if
            let thumbnailView = ThumbnailInstanceManager.shared
                .getInstance(instanceId: thumbnailId.intValue)
        {
            thumbnailView.createThumbnails(videoView, instanceId.intValue, thumbnailId.intValue)
        }
        resolve("")
    }

    @objc
    func getPlaybackRate(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        resolve(videoView.getPlaybackRate())
    }

    @objc
    func setPlaybackRate(
        _ instanceId: NSNumber!,
        newPlaybackRate: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        videoView.setPlaybackRate(newPlaybackRate.doubleValue)
        resolve("")
    }

    @objc
    func isPictureInPicturePossible(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        var possible = false
        if #available(tvOS 14.0, *) {
            possible = videoView.pipController?.isPictureInPicturePossible ?? false
        }
        resolve(possible)
    }

    @objc
    func togglePictureInPicture(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard #available(tvOS 14.0, *) else { return }
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        guard let pipController = videoView.pipController else { return }
        guard true == pipController.isPictureInPicturePossible else { return }

        if true == pipController.isPictureInPictureActive {
            pipController.stopPictureInPicture()
        } else {
            pipController.startPictureInPicture()
        }
        resolve("")
    }

    @objc
    func isAirplayPossible(
        _ instanceId: NSNumber!,
        resolver resolve: RCTPromiseResolveBlock,
        rejecter reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }
        resolve(videoView.isNativePlayer())
    }

    @objc
    func playPlaylistItem(
        _ instanceId: NSNumber!,
        index: NSNumber!,
        resolve: RCTPromiseResolveBlock,
        reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }

        videoView.playPlaylistItem(index: index.intValue)

        resolve(nil)
    }

    @objc
    func addPlaylistItem(
        _ instanceId: NSNumber!,
        index: NSNumber!,
        jsPlayerConfiguration: String!,
        resolve: RCTPromiseResolveBlock,
        reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }

        videoView.addPlaylistItem(
            index: index.intValue,
            jsonPlayerConfiguration: jsPlayerConfiguration
        )

        resolve(nil)
    }

    @objc
    func removePlaylistItem(
        _ instanceId: NSNumber!,
        index: NSNumber!,
        resolve: RCTPromiseResolveBlock,
        reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }

        videoView.removePlaylistItem(index: index.intValue)

        resolve(nil)
    }


    @objc
    func play(
        _ instanceId: NSNumber!,
        resolve: RCTPromiseResolveBlock,
        reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }

        videoView.play()

        resolve(nil)
    }

    @objc
    func pause(
        _ instanceId: NSNumber!,
        resolve: RCTPromiseResolveBlock,
        reject: RCTPromiseRejectBlock
    ) {
        guard
            let videoView = VideoInstanceManager.shared
                .getInstance(instanceId: instanceId.intValue) else
        {
            reject("error", "Unable to instantiate VideoView", nil)
            return
        }

        videoView.pause()

        resolve(nil)
    }

    // MARK: Private

    private var licenseKey: String {
        let licenseData = initData?["licenseKey"] as? NSDictionary
        #if os(tvOS)
        return licenseData?["tvOS"] as? String ?? ""
        #else
        return licenseData?["iOS"] as? String ?? ""
        #endif
    }
}
