//
//  VideoView.swift
//  react-native-prestoplay-module
//
//  Created by Mackode - Bartlomiej Makowski on 25/02/2022.
//

import AVKit
import Foundation
import UIKit

import CastlabsApple
import CastlabsYoubora
import PRESTOplay

import React

// MARK: - VideoView

@objc
public class VideoView: UIView {

    // MARK: Lifecycle

    override init(frame: CGRect) {
        super.init(frame: frame)
    }

    required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    // MARK: Public

    public override func layoutSubviews() {
        player?.update(size: frame.size)
        onViewCreatedEvent()
    }

    override public func willMove(toSuperview newSuperview: UIView?) {
        guard nil == newSuperview
        else { return } // Remove from superview - unmount in react-native screen
        if #available(tvOS 14.0, *) {
            guard false == self.pipController?.isPictureInPictureActive ?? false else { return }
        }

        if #available(tvOS 14.0, *) {
            pipController?.delegate = nil
        }
        privatePipController = nil

        stop()
        removePlayerViewAndDetach()

        playerView = nil
        previousMappedState = nil

        player = nil
        playlist = nil
        subtitles = nil
        youbora = nil

        currentMediaLoadRequest = nil
        currentPlayerConfiguration = nil
    }

    // MARK: Internal

    var videoQualityMode = "adaptive" // or "manual"

    @objc var onViewCreated: RCTDirectEventBlock?
    @objc var onStateChanged: RCTDirectEventBlock?
    @objc var onError: RCTDirectEventBlock?
    @objc var onSeekTo: RCTDirectEventBlock?
    @objc var onSeekCompleted: RCTDirectEventBlock?
    @objc var onFullyBuffered: RCTDirectEventBlock?
    @objc var onThumbnailLoaded: RCTDirectEventBlock?
    @objc var onPlayerConfigChanged: RCTDirectEventBlock?
    @objc var onPlayerModelChanged: RCTDirectEventBlock?
    @objc var onPlaylistItemChanged: RCTDirectEventBlock?
    @objc var onPlaylistEnded: RCTDirectEventBlock?

    var videoRenditions: [VideoRendition] {
        var tmp: [VideoRendition] = []
        guard let tracks = getTracks() else { return tmp }
        tracks.video.forEach { videoTrack in
            videoTrack.renditions.forEach { videoRendition in
                tmp.append(videoRendition)
            }
        }
        return tmp
    }

    var tracks: Tracks? {
        player?.tracks
    }

    @available(tvOS 14.0, *)
    var pipController: AVPictureInPictureController? {
        privatePipController as? AVPictureInPictureController
    }

    @objc
    func setMediaLoadRequest(_ json: String) {
        DispatchQueue.main.async { [weak self] in
            guard let self = self else { return }

            guard
                let mediaLoadRequest = BridgeDeserializer.toMediaLoadRequest(json: json)
            else {
                print("Invalid player config: \(json)")
                let rnError = RNError(
                    severity: .fatal,
                    type: ErrorType.player_open_error.rawValue,
                    message: "Invalid player config"
                )
                onErrorEvent(rnError.dictionary)
                return
            }

            if true == self.openStream(mediaLoadRequest) {
                self.onPlayerConfigChangedEvent()
            }
        }
    }

    @objc
    func onViewCreatedEvent() {
        VideoInstanceManager.shared.addInstance(instanceId: hashValue, videoView: self)
        onViewCreated?([
            "instanceId": hashValue,
        ])
    }

    @objc
    func onThumbnailLoadedEvent(_ requestedPosition: Float64, _ width: Int, _ height: Int) {
        onThumbnailLoaded?([
            "requestedPosition": requestedPosition,
            "actualPosition": getPosition() ?? 0.0,
            "width": width,
            "height": height,
        ])
    }

    @objc
    func onStateChangedEvent(_ state: Int) {
        onStateChanged?([
            "playbackState": state,
        ])
    }

    @objc
    func onErrorEvent(_ errorInfo: [String: Any]) {
        onError?(errorInfo)
    }

    @objc
    func onSeekToEvent(_ newPosition: Float64) {
        seeking = true
        onSeekTo?([
            "newPosition": newPosition,
        ])
    }

    @objc
    func onSeekCompletedEvent() {
        seeking = false
        onSeekCompleted?([:])
    }

    @objc
    func onFullyBufferedEvent() {
        onFullyBuffered?([:])
    }

    @objc
    func isNativePlayer() -> Bool {
        guard let player = getPlayer() else { return false }
        return (player.engine == .apple)
    }

    @objc
    func onPlayerConfigChangedEvent() {
        onPlayerConfigChanged?([:])
    }

    @objc
    func onPlayerModelChangedEvent() {
        onPlayerModelChanged?([:])
    }

    @objc
    func onPlaylistItemChangedEvent(_ newIndex: Int) {
        onPlaylistItemChanged?([
            "newItemIndex": newIndex,
        ])
    }

    @objc
    func onPlaylistEndedEvent() {
        onPlaylistEnded?([:])
    }

    // MARK: Private

    private var playerView: UIView?
    private var player: PlayerAPI?
    private var playlist: PlaylistAPI?
    private var subtitles: SubtitlesAPI?
    private var youbora: AnalyticsAPI?

    private var currentMediaLoadRequest: MediaLoadRequest?
    private var currentPlayerConfiguration: PlayerConfiguration?
    private var seeking = false
    private var playWhenReady = false
    private var subtitlesAdded = false
    private var previousMappedState: Int?
    private var playbackState: PlaybackState = .idle
    private var playerLock = NSLock()

    private var privatePipController: AnyObject?
}

// MARK: -

extension VideoView {
    func openStream(_ mediaLoadRequest: MediaLoadRequest) -> Bool {
        // TODO: iOS SDK doesn't support a custom start item in a playlist
        var firstItem = mediaLoadRequest.items[0]
        let playerConfigurations = mediaLoadRequest.items.map { $0.playerConfiguration }
        let autoPlay = firstItem.playerConfiguration.autoPlay ?? false

        if false == mediaLoadRequest.usePlaylist {
            let download = PRESTOplaySDK.shared.downloader().getDownloads().first { download in
                firstItem.playerConfiguration.contentUrl == download.localUrl
            }

            if let download {
                firstItem = MediaLoadRequestItem(
                    playerConfiguration: try! PlayerConfiguration(with: download),
                    youboraMetadata: firstItem.youboraMetadata,
                    ignoreSilentSwitch: firstItem.ignoreSilentSwitch
                )
            }
        }

        if nil == player {
            let player = createPlayer()
            let playlist = createPlaylist(player: player)
            let subtitles = createSubtitles(player: player)
            let youbora = createYoubora(player: player)

            setupSubtitles(
                subtitles: subtitles,
                playerConfiguration: firstItem.playerConfiguration
            )

            setupYoubora(
                youbora: youbora,
                metadata: firstItem.youboraMetadata
            )

            setupIgnoreSilentSwitch(
                ignoreSilentSwitch: firstItem.ignoreSilentSwitch
            )

            if mediaLoadRequest.usePlaylist {
                playlist?.load(items: playerConfigurations)
            } else {
                player.load(config: firstItem.playerConfiguration)
            }

            player.open(autoplay: false)

            if #available(tvOS 14.0, *) {
                privatePipController = player.getPictureInPictureController()
                pipController?.delegate = self
            }

            playerLock.lock()

            self.player = player
            self.playlist = playlist
            self.subtitles = subtitles
            self.youbora = youbora

            currentMediaLoadRequest = mediaLoadRequest
            currentPlayerConfiguration = firstItem.playerConfiguration
            playWhenReady = autoPlay

            playerLock.unlock()
        } else {
            if let player = player as? PlayerComponentProtocol {
                playerLock.lock()

                setupSubtitles(
                    subtitles: subtitles,
                    playerConfiguration: firstItem.playerConfiguration
                )

                setupYoubora(
                    youbora: youbora,
                    metadata: firstItem.youboraMetadata
                )

                setupIgnoreSilentSwitch(
                    ignoreSilentSwitch: firstItem.ignoreSilentSwitch
                )

                if mediaLoadRequest.usePlaylist {
                    playlist?.load(items: playerConfigurations)
                } else {
                    // remove all items
                    if let playlist = playlist {
                        while playlist.getSize() > 0 {
                            playlist.removeItem(at: 0)
                        }
                    }

                    player.load(config: firstItem.playerConfiguration)
                }

                currentMediaLoadRequest = mediaLoadRequest
                currentPlayerConfiguration = firstItem.playerConfiguration
                playWhenReady = autoPlay

                playerLock.unlock()
            } else {
                let rnError = RNError(
                    severity: .fatal,
                    type: ErrorType.player_open_error.rawValue,
                    message: "Player is not initialized"
                )
                onErrorEvent(rnError.dictionary)
                return false
            }
        }

        return true
    }

    func createPlayer() -> PlayerAPI {
        var player = PRESTOplaySDK.shared.player()
        playerView = createPlayerViewAndAttach(to: player)

        player.onTrackModel = { [weak self] in
            guard let self = self else { return }
            self.onPlayerModelChangedEvent()
        }

        player.onState = { [weak self] old, state in
            guard let self = self else { return }
            guard let player = self.player else { return }
            guard old != state else { return }

            switch state {
            case .ready:
                if self.playWhenReady {
                    play()
                }
            case .playing:
                if self.seeking == true {
                    self.onSeekCompletedEvent()
                }
            case .buffering:
                if let buffer = player.bufferFullness, buffer >= 0.99 {
                    self.onFullyBufferedEvent()
                }
            case .error:
                let rnError = RNError(clError: player.error)
                self.onErrorEvent(rnError.dictionary)
            default:
                break
            }

            if let currentPlaybackState = player.playbackState {
                let mappedState = PRESTOUtils.mapState(currentPlaybackState)
                if previousMappedState != mappedState {
                    self.onStateChangedEvent(mappedState!)
                    previousMappedState = mappedState
                }

                playbackState = currentPlaybackState
            }
        }

        PRESTOplaySDK.shared.onError = { [weak self] _, error in
            guard let self else { return }
            let rnError = RNError(clError: error)
            onErrorEvent(rnError.dictionary)
        }

        return player
    }

    func createPlaylist(player: PlayerAPI) -> PlaylistAPI? {
        var playlist = PRESTOplaySDK.shared.playlist(
            for: player,
            mode: .nativeQueue,
            loopList: false
        )

        playlist?.onItemChange = { [weak self] in
            guard
                let self = self,
                let newIndex = self.playlist?.getCurrentItemIndex(),
                let newMediaLoadRequestItem = self.currentMediaLoadRequest?.items[newIndex] else
            {
                return
            }

            let newPlayerConfiguration = newMediaLoadRequestItem.playerConfiguration

            self.setupSubtitles(
                subtitles: self.subtitles,
                playerConfiguration: newPlayerConfiguration
            )

            self.setupYoubora(
                youbora: self.youbora,
                metadata: newMediaLoadRequestItem.youboraMetadata
            )

            self.setupIgnoreSilentSwitch(
                ignoreSilentSwitch: newMediaLoadRequestItem.ignoreSilentSwitch
            )

            self.currentPlayerConfiguration = newPlayerConfiguration

            self.onPlaylistItemChangedEvent(newIndex)
        }

        playlist?.onPlaylistEnded = { [weak self] in
            guard let self = self else { return }
            self.onPlaylistEndedEvent()
        }

        return playlist
    }

    func createSubtitles(player: PlayerAPI) -> SubtitlesAPI? {
        PRESTOplaySDK.shared.subtitles(for: player)
    }

    func createYoubora(player: PlayerAPI) -> AnalyticsAPI? {
        PRESTOplaySDK.shared.analytics(for: player, .youbora)
    }

    func createPlayerViewAndAttach(to player: PlayerAPI) -> UIView {
        let view = UIView(frame: frame)
        insertSubview(view, at: 0)
        player.attach(to: view.layer)
        return view
    }

    func removePlayerViewAndDetach() {
        getPlayer()?.detachFromView()
        playerView?.removeFromSuperview()
    }

    func setupIgnoreSilentSwitch(ignoreSilentSwitch: String?) {
        /// ignore silent switch is intentionally implemented in open source part of the sdk
        /// as there are many use cases like mixing audio, which should not be forced
        /// by PRESTOplay SDK v4 and should be considered case by case by the SDK user
        do {
            switch ignoreSilentSwitch {
            case "ignore":
                try AVAudioSession.sharedInstance().setCategory(.playback)
            case "obey":
                try AVAudioSession.sharedInstance().setCategory(.ambient)
            default:
                print("AVAudioSession will use default silent switch behaviour")
            }
        } catch { }
    }

    func seek(_ time: CMTime) {
        DispatchQueue.main.async { [weak self] in
            guard let self else { return }
            self.player?.seek(time)
        }
    }

    func setupSubtitles(
        subtitles: SubtitlesAPI?,
        playerConfiguration: PlayerConfiguration
    ) {
        if true == subtitlesAdded {
            // RN-285: Subtitles module fails silently if the reset is executed when no tracks
            subtitles?.reset()
            subtitlesAdded = false
        }

        let sideloadTracks = playerConfiguration.sideloadedTracks?.filter { $0.trackType == .text }

        guard let sideloadTracks else { return }

        for sideloadTrack in sideloadTracks {
            let track = TextTrack(
                id: "",
                format: textFormat(fromMimeType: sideloadTrack.mimeType),
                isPlatformRendered: false,
                label: sideloadTrack.label ?? "",
                language: sideloadTrack.language ?? "",
                sourceUrl: sideloadTrack.url
            )

            subtitles?.addTextTrack(track) { [weak self] _, err in
                if let err {
                    let rnError = RNError(clError: err)
                    self?.onErrorEvent(rnError.dictionary)
                }
            }

            subtitlesAdded = true
        }
    }

    func setupYoubora(youbora: AnalyticsAPI?, metadata: YouboraMetadata?) {
        if var youbora {
            youbora.metadata = metadata
        }
    }
}

// MARK: - PlayerAPI

extension VideoView {
    func getPlayer() -> PlayerAPI? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        return player
    }

    func getPlayerConfiguration() -> PlayerConfiguration? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        return currentPlayerConfiguration
    }

    func getPlaybackState() -> PlaybackState? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return nil }
        return player.playbackState
    }

    func play() {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }

        guard let player else { return }

        if playbackState != .playing {
            player.play()
            playWhenReady = true
        }
    }

    func pause() {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }

        guard let player else { return }

        if playbackState != .paused, playbackState != .pausing {
            player.pause()
            playWhenReady = false
        }
    }

    func stop() {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return }
        player.stop()
    }

    func getPosition() -> Float64? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return nil }
        return player.position
    }

    func getTracks() -> Tracks? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return nil }
        return player.tracks
    }

    func getBufferedPercentage() -> Float? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return nil }
        return player.bufferedPercentage
    }

    func getRenderedVideoRendition() -> VideoRendition? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return nil }
        return player.getRenderedVideoRendition()
    }

    func getAudioTrack() -> AudioTrack? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return nil }
        return player.getAudioTrack()
    }

    func getTextTrack() -> TextTrack? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return nil }
        return player.getTextTrack()
    }

    func getDuration() -> TimeInterval? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return nil }
        return player.duration
    }

    func isLive() -> Bool {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return false }
        return player.isLive
    }

    func getStats() -> Stats? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return nil }
        return player.getStats()
    }

    func getLiveStartTime() -> TimeInterval? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return .infinity }
        return player.getLiveStartTime()
    }

    func getSeekableTimeRanges() -> [CMTimeRange] {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return [] }
        return player.seekableTimeRanges
    }

    func getPlaybackVolume() -> Float64 {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return 0.0 }
        return player.playbackVolume
    }

    func setPlaybackVolume(_ volume: Float64) {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard var player else { return }
        player.playbackVolume = volume
    }

    func setAudioTrack(_ track: AudioTrack) {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return }
        player.setAudioTrack(track)
    }

    func setTextTrack(_ track: TextTrack?) {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return }
        player.setTextTrack(track)
    }

    func setVideoRendition(_ rendition: VideoRendition?) {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return }
        player.setVideoRendition(rendition)
    }

    func setSubtitlesStyle(_ subtitlesStyle: SubtitlesStyle) {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard player != nil else { return }
        subtitles?.subtitlesStyle = subtitlesStyle
    }

    func playPlaylistItem(index: Int) {
        playlist?.playItem(at: index)
    }

    func addPlaylistItem(index: Int, jsonPlayerConfiguration: String) {
        guard
            let mediaLoadRequestItem = BridgeDeserializer
                .toMediaLoadRequestItem(json: jsonPlayerConfiguration) else { return }
        playlist?.addItem(mediaLoadRequestItem.playerConfiguration, at: index)
        currentMediaLoadRequest?.items.append(mediaLoadRequestItem)
    }

    func removePlaylistItem(index: Int) {
        playlist?.removeItem(at: index)
        currentMediaLoadRequest?.items.remove(at: index)
    }

    func getPlaybackRate() -> Float64? {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard let player else { return nil }
        return player.playbackRate
    }

    func setPlaybackRate(_ newPlaybackRate: Float64) {
        playerLock.lock()
        defer {
            playerLock.unlock()
        }
        guard var player else { return }
        player.playbackRate = newPlaybackRate
    }
}

// MARK: AVPictureInPictureControllerDelegate

// Send PiP events to PictureInPictureEventModule.
@available(tvOS 14.0, *)
extension VideoView: AVPictureInPictureControllerDelegate {
    public func pictureInPictureControllerWillStartPictureInPicture(
        _: AVPictureInPictureController
    ) {
        NotificationCenter.default.post(
            name: NSNotification.Name.PICTURE_IN_PICTURE_WILL_START,
            object: nil
        )
    }

    // swiftlint:disable line_length
    public func pictureInPictureController(
        _: AVPictureInPictureController,
        restoreUserInterfaceForPictureInPictureStopWithCompletionHandler completionHandler: @escaping (
            Bool
        ) -> Void
    ) {
        NotificationCenter.default.post(
            name: NSNotification.Name.RESTORE_USER_INTERFACE_FOR_PICTURE_IN_PICTURE_STOP,
            object: nil
        )
        completionHandler(true)
    }

    // swiftlint:enable

    public func pictureInPictureControllerDidStopPictureInPicture(_: AVPictureInPictureController) {
        NotificationCenter.default.post(
            name: NSNotification.Name.PICTURE_IN_PICTURE_DID_STOP,
            object: nil
        )
    }
}
