import { EmitterSubscription } from 'react-native';
import { AudioTrack, SubtitleTrack } from '../index';
import TVStream from '../types/TVStream';
import PlayerOptions from '../types/PlayerOptions';
import Action from '../types/Action';
import ActionUiProperty from '../types/ActionUiProperty';
import ChannelMetadata from '../types/ChannelMetadata';
import PopupMenu from '../types/PopupMenu';
import WillSeekCallback from '../types/WillSeekCallback';
import DidSeekCallback from '../types/DidSeekCallback';
import GenericEventCallback from '../types/GenericEventCallback';
import SubtitleTrackChangedCallback from '../types/SubtitleTrackChangedCallback';
import AudioTrackChangedCallback from '../types/AudioTrackChangedCallback';
import ErrorEventCallback from '../types/ErrorEventCallback';
import StreamChangedCallback from '../types/StreamChangedCallback';
/**
 * The AVPlayerViewController proxy, which provides playback capabilities using the default system UI.
 *
 * @example
 * ```typescript
 * import {CLAppleTVPlayer} from '@castlabs/react-native-prestoplay';
 * ```
 * @remarks Not supported on Android.
 */
declare class CLAppleTVPlayer {
    /**
     * Triggered on playback ended.
     * Close the player or change the stream here.
     */
    static didPlayToEnd?: GenericEventCallback;
    /**
     * Triggered on player closed.
     */
    static playerIsClosed?: GenericEventCallback;
    /**
     * Triggered on seeking started.
     */
    static willSeek?: WillSeekCallback;
    /**
     * Triggered on seeking completed.
     */
    static didSeek?: DidSeekCallback;
    /**
     * Triggered on playback status changed.
     */
    static timeControlStatus?: GenericEventCallback;
    /**
     * Triggered when the user skips to the next channel.
     *
     * This callback is only triggered for live streams and when a single stream is loaded.
     * It is not triggered when a playlist is in use, i.e., when multiple streams are queued.
     * To replace the current stream with the next one, call the {@link CLAppleTVPlayer.openPlayer} method.
     */
    static skipToNextChannel?: GenericEventCallback;
    /**
     * Triggered when the user skips to the previous channel.
     *
     * This callback is only triggered for live streams and when a single stream is loaded.
     * It is not triggered when a playlist is in use, i.e., when multiple streams are queued.
     * To replace the current stream with the next one, call the {@link CLAppleTVPlayer.openPlayer} method.
     */
    static skipToPreviousChannel?: GenericEventCallback;
    /**
     * Triggered on error thrown.
     */
    static onError?: ErrorEventCallback;
    /**
     * Triggered on audio track changed.
     */
    static audioTrackChanged?: AudioTrackChangedCallback;
    /**
     * Triggered on subtitle track changed.
     */
    static subtitleTrackChanged?: SubtitleTrackChangedCallback;
    /**
     * Triggered on stream changed.
     */
    static streamChanged?: StreamChangedCallback;
    private static nativeEventEmitter;
    private static _initialize;
    private static registerPlayerEvent;
    /**
     * Subscribes to the action event.
     * @example
     * ```typescript
     * CLAppleTVPlayer.addActionListener((event: any) => {
     *   switch (event.id) {
     *     case ActionId.Speed:
     *       CLAppleTVPlayer.setRate(Number(event.data));
     *       break;
     *     case ActionId.Skip00:
     *       CLAppleTVPlayer.seekToSecond(15);
     *       break;
     *   }
     * });
     * ```
     * @param listener the listener
     * @returns the event subscription
     */
    static addActionListener(listener: (event: any) => void): EmitterSubscription;
    /**
     * Unsubscribes from the action event.
     * @param subscription the event subscription
     */
    static removeActionListener(subscription: EmitterSubscription): void;
    /**
     * Opens the player.
     *
     * If a single stream is loaded, the {@link CLAppleTVPlayer.skipToNextChannel}
     * and {@link CLAppleTVPlayer.skipToPreviousChannel} callbacks are fired
     * when the user skips to the next or previous channel. Also, when a single
     * stream is loaded, you can replace it with another by calling
     * the {@link CLAppleTVPlayer.openPlayer} method again. This is particularly
     * useful if you want to replace the current stream with a different one.
     *
     * If multiple streams are loaded, the player will automatically switch
     * to the next stream when the current stream ends. In addition,
     * the {@link CLAppleTVPlayer.streamChanged} callback will be fired when
     * the player switches to the next stream. When all streams are live,
     * the user can use the TV remote to switch between channels. However,
     * the {@link CLAppleTVPlayer.skipToNextChannel} and
     * {@link CLAppleTVPlayer.skipToPreviousChannel} callbacks will not fire.
     *
     * If loopStream is false, the player will ignore next or previous channel
     * requests if the current stream is the last or first item in the list.
     * The player will still display the channel's interstitial view,
     * but will not change the stream.
     *
     * @param streams the stream configuration
     * @param options the player options
     * @param transportBarActions the transport bar actions
     * @param infoTabActions the info tab actions
     * @param contextualActions the contextual actions
     * @param loopStreams loops through the streams list. Default is `true`.
     * @param onError the error callback
     * @param onSuccess the success callback
     */
    static openPlayer(streams: TVStream[], options?: PlayerOptions, transportBarActions?: Array<Action | PopupMenu>, infoTabActions?: Action[], contextualActions?: Action[], loopStreams?: Boolean, onError?: (error: any) => void, onSuccess?: () => void): void;
    /**
     * Starts playback.
     */
    static play(): void;
    /**
     * Pauses playback.
     */
    static pause(): void;
    /**
     * Closes the player.
     */
    static closePlayer(): void;
    /**
     * Sets the metadata to be displayed in the title view and info tab.
     * @param metadata the channel metadata
     */
    static setMetadata(metadata: ChannelMetadata): void;
    /**
     * Sets player options.
     * @param options the player options
     */
    static setOptions(options: PlayerOptions): void;
    /**
     * Sets transport bar actions.
     * @param transportBarActions the transport bar actions
     */
    static setTransportBarActions(transportBarActions: Array<Action | PopupMenu>): void;
    /**
     * Sets info tab actions.
     * @param infoTabActions the info tab actions
     */
    static setInfoTabActions(infoTabActions: Action[]): void;
    /**
     * Sets contextual actions.
     * @param contextualActions the contextual actions
     */
    static setContextualActions(contextualActions: Action[]): void;
    /**
     * Changes the state of the action and UI appearance.
     * @param id the action identifier
     * @param state the action state
     * @param uiProp the action title and image resource name in Xcode project
     */
    static setActionStateAndUi(id: string, state: boolean, uiProp: ActionUiProperty): void;
    /**
     * Sets the new playback rate.
     * @param rate the playback rate
     */
    static setRate(rate: number): void;
    /**
     * Moves playhead to the given position.
     * @param second the new position in seconds
     */
    static seekToSecond(second: number): void;
    /**
     * Returns the current playhead.
     * @returns the current position in seconds
     */
    static getPosition(): Promise<number>;
    /**
     * Returns the live start time.
     * A live HLS stream has to have the `EXT-X-PROGRAM-DATE-TIME` defined.
     * @returns The live start time since the epoch in seconds; `-1` if it is unknown or not applicable.
     */
    static getLiveStartTime(): Promise<number>;
    /**
     * Returns the current audio track.
     * @returns {Promise<AudioTrack>} the current audio track
     */
    static getAudioTrack(): Promise<AudioTrack>;
    /**
     * Returns the current subtitle track.
     * @returns {Promise<SubtitleTrack>} the current subtitle track
     */
    static getSubtitleTrack(): Promise<SubtitleTrack>;
    /**
     * Returns the index of the current stream.
     * @returns {Promise<number>} the current stream index
     */
    static getCurrentStreamIndex(): Promise<number>;
    private static toNativeTVStream;
}
export default CLAppleTVPlayer;
