import { Component, ReactNode } from 'react';
import CLThumbnail from './CLThumbnail';
import SDKConfiguration from '../types/SDKConfiguration';
import CLVideoProps from '../types/CLVideoProps';
import PlayerState from '../types/PlayerState';
import VideoQuality from '../types/VideoQuality';
import AudioTrack from '../types/AudioTrack';
import SubtitleTrack from '../types/SubtitleTrack';
import SubtitleStyle from '../types/SubtitleStyle';
import PlayerConfiguration from '../types/PlayerConfiguration';
import Stats from '../types/Stats';
/**
 * @example
 * ```typescript
 * import {CLVideo} from '@castlabs/react-native-prestoplay';
 * ```
 * @noInheritDoc
 */
declare class CLVideo extends Component<CLVideoProps> {
    private static _isAuthenticated;
    private _instanceId;
    private _playWhenReady;
    /**
     * Initializes the CLVideo component.
     *
     * This method should be called once before using the CLVideo component.
     * Licence keys are associated with your application ID.
     * Please contact support for more information on how to obtain licence keys.
     *
     * @example
     * ```typescript
     * CLVideo.initialize({
     *   licenseKey: {
     *     android: 'my-android-key',
     *     iOS: 'my-ios-key',
     *     tvOS: 'my-ios-key',
     *   }
     * });
     * ```
     * @param sdkConfig The configuration of the PRESTOplay SDK.
     * @returns If the promise is resolved, the CLVideo component is fully initialized and ready to use.
     */
    static initialize(sdkConfig: SDKConfiguration): Promise<void>;
    /**
     * Checks if the component has been authenticated.
     *
     * @returns True if the component has been authenticated with given license keys.
     */
    static get isAuthenticated(): boolean;
    constructor(props: CLVideoProps);
    shouldComponentUpdate(nextProps: Readonly<CLVideoProps>, nextState: Readonly<any>, nextContext: any): boolean;
    componentDidUpdate(prevProps: Readonly<CLVideoProps>, prevState: Readonly<any>): void;
    get instanceId(): number | undefined;
    getPlayerConfiguration(): PlayerConfiguration | undefined;
    render(): ReactNode;
    /**
     * Called when the view is created
     * @param {NativeSyntheticEvent<ViewCreatedData>} event
     */
    private onViewCreated;
    /**
     * Called when the state of the player is changed
     * @param {NativeSyntheticEvent<StateChangedData>} event
     */
    private onStateChanged;
    /**
     * On iOS, when a call is received, the playback state
     * automatically changes to paused. When the call ends,
     * the playback state remains paused. However, if the player is
     * in picture-in-picture mode (both Android and iOS), the
     * playback state changes to playing. Therefore, it's necessary
     * to synchronize `playWhenReady` value with the `playbackState`.
     * @param {number} playbackState
     */
    private syncPlaybackState;
    /**
     * Called on player error
     * @param {NativeSyntheticEvent<ErrorData>} event
     */
    private onError;
    /**
     * Called when the player is sought to another position
     * @param {NativeSyntheticEvent<SeekToData>} event
     */
    private onSeekTo;
    /**
     * Called when seeking is completed
     */
    private onSeekCompleted;
    /**
     * Called when the player is fully buffered
     */
    private onFullyBuffered;
    /**
     * Called when the thumbnail is loaded. The actual image data is not sent in the
     * data parameter. Instead, the bitmap is loaded in the thumbnail component which
     * is set to this video component
     * @param {NativeSyntheticEvent<ThumbnailData>} event
     */
    private onThumbnailLoaded;
    /**
     * Called when the player model is changed. The player model is changed on the
     * playback initial start-up or when the media period is changed.
     */
    private onPlayerModelChanged;
    private onPlaylistItemChanged;
    private onPlaylistEnded;
    /**
     * Returns the current player state.
     * @returns The current player state
     */
    getPlayerState(): Promise<PlayerState>;
    /**
     * Returns the total duration of content.
     * @returns The total content duration in seconds
     */
    getDuration(): Promise<number>;
    /**
     * Returns true if the stream is live.
     * @returns Weather the stream is live or not
     */
    isLive(): Promise<boolean>;
    /**
     * Returns the current position.
     * @returns The current position in seconds.
     */
    getPosition(): Promise<number>;
    /**
     * Sets the new position.
     * @param newPosition The new position in seconds.
     */
    setPosition(newPosition: number): void;
    /**
     * Returns the current volume.
     * @returns The current volume, a value between 0 and 1.
     */
    getVolume(): Promise<number>;
    /**
     * Sets the new volume.
     * @param newVolume The new volume, a value between 0 and 1.
     */
    setVolume(newVolume: number): void;
    /**
     * Returns the current video quality.
     * @returns The current video quality.
     */
    getVideoQuality(): Promise<VideoQuality>;
    /**
     * Returns the current video quality mode.
     * @returns {@link C.VIDEO_QUALITY_ADAPTIVE} or {@link C.VIDEO_QUALITY_MANUAL}
     */
    getVideoQualityMode(): Promise<string>;
    /**
     * Sets the current video quality.
     * @param index The index of the video quality in the {@link getVideoQualities} array
     */
    setVideoQuality(index: number): void;
    /**
     * Returns all video qualities.
     * @returns All video qualities.
     */
    getVideoQualities(): Promise<VideoQuality[]>;
    /**
     * Returns the current audio track.
     * @returns The current audio track.
     */
    getAudioTrack(): Promise<AudioTrack>;
    /**
     * Sets the current audio track.
     * @param index - The index of the audio track in the {@link getAudioTracks()} array.
     */
    setAudioTrack(index: number): void;
    /**
     * Returns all audio tracks.
     * @returns All audio tracks.
     */
    getAudioTracks(): Promise<AudioTrack[]>;
    /**
     * Returns the current text track.
     * @returns The current text track.
     */
    getSubtitleTrack(): Promise<SubtitleTrack>;
    /**
     * Sets the current text track
     * @param index The index of the text track in the {@link getSubtitleTracks()} array.
     */
    setSubtitleTrack(index: number): void;
    /**
     * Returns all text tracks.
     * @returns All text tracks.
     */
    getSubtitleTracks(): Promise<SubtitleTrack[]>;
    /**
     * Sets the style of the text track.
     * @param style The style of the text track.
     */
    setSubtitleStyle(style: SubtitleStyle): void;
    /**
     * This method doesn't return anything but will trigger {@link CLVideoProps.onThumbnailLoaded}
     * event if the thumbnail is loaded. It also sets the bitmap of the thumbnail component
     * if the component is set to this video component.
     *
     * @param position Position in seconds
     */
    getThumbnailData(position: number): void;
    /**
     * Registers the {@link CLThumbnail} component.
     * @param thumbnail The CLThumbnail component.
     */
    setThumbnail(thumbnail: CLThumbnail): void;
    /**
     * Returns the playback progress.
     * @returns The playback progress as percentage.
     */
    getProgressPercentage(): Promise<number>;
    /**
     * Returns the buffering progress.
     * @returns The buffering progress as percentage.
     */
    getBufferedPercentage(): Promise<number>;
    /**
     * Plays the video.
     */
    play(): Promise<void>;
    /**
     * Pauses the video.
     */
    pause(): Promise<void>;
    /**
     * Returns true if the player will transition into playing state when enough data is loaded. If false the player will transition into paused state.
     * Note that the player can have intermediate states until it reaches the playing state.
     * @returns {boolean} True if the video will play when enough data, false otherwise.
     */
    isPlayWhenReady(): boolean;
    /**
     * Checks whether the picture-in-picture mode is available or not.
     * @return `true` if picture-in-picture mode is available; Otherwise `false`.
     */
    isPictureInPicturePossible(): Promise<boolean>;
    /**
     * Activates or deactivates the picture-in-picture mode.
     */
    togglePictureInPicture(): void;
    /**
     * Returns the live start time.
     * A live HLS stream has to have the `EXT-X-PROGRAM-DATE-TIME` defined.
     * @returns The live start time since the epoch in seconds; `-1` if it is unknown or not applicable.
     */
    getLiveStartTime(): Promise<number>;
    /**
     * Checks whether the AirPlay is available or not.
     * @return `true` if AirPlay is available; Otherwise `false`.
     */
    isAirplayPossible(): Promise<boolean>;
    /**
     * Returns statistics and information about the current state of the player.
     * @return The statistics.
     */
    getStats(): Promise<Stats>;
    /**
     * Returns speed of the playback.
     * @return The current speed of the playback, where 1 means "normal" speed.
     */
    getPlaybackRate(): Promise<number>;
    /**
     * Sets speed of the playback, where 1 means "normal" speed.
     *
     * On iOS and tvOS, the playback rate can only be increased if
     * the current position is not near the live edge.
     * This behaviour in AVPlayer is designed to prevent stalling.
     *
     * @param newPlaybackRate The new playback rate, where 1 means "normal" speed.
     */
    setPlaybackRate(newPlaybackRate: number): void;
    private get playlist();
    private get nativeVideoModule();
    private get ready();
    private get notReady();
    private failIfNotReady;
}
export default CLVideo;
