import React, { Component } from 'react';
import { NativeModules, requireNativeComponent, StyleSheet, View, } from 'react-native';
import PlayerState from '../types/PlayerState';
import Playlist from '../utils/Playlist';
import MediaLoadRequest from '../types/MediaLoadRequest';
import ErrorMessage from './ErrorMessage';
import PlayerNotReadyError from '../errors/PlayerNotReadyError';
const VideoView = requireNativeComponent('VideoView');
/**
 * @example
 * ```typescript
 * import {CLVideo} from '@castlabs/react-native-prestoplay';
 * ```
 * @noInheritDoc
 */
class CLVideo extends Component {
    static _isAuthenticated = false;
    _instanceId;
    _playWhenReady = false;
    /**
     * Initializes the CLVideo component.
     *
     * This method should be called once before using the CLVideo component.
     * Licence keys are associated with your application ID.
     * Please contact support for more information on how to obtain licence keys.
     *
     * @example
     * ```typescript
     * CLVideo.initialize({
     *   licenseKey: {
     *     android: 'my-android-key',
     *     iOS: 'my-ios-key',
     *     tvOS: 'my-ios-key',
     *   }
     * });
     * ```
     * @param sdkConfig The configuration of the PRESTOplay SDK.
     * @returns If the promise is resolved, the CLVideo component is fully initialized and ready to use.
     */
    static initialize(sdkConfig) {
        // The actual version is injected by the build system
        const companionSdkInfo = {
            name: 'react-native',
            version: '0.16.9',
        };
        return NativeModules.VideoModule.initialize(sdkConfig, companionSdkInfo).then(() => {
            CLVideo._isAuthenticated = true;
        }, (error) => {
            console.log('Initialization failed. ' + error);
            CLVideo._isAuthenticated = false;
        });
    }
    /**
     * Checks if the component has been authenticated.
     *
     * @returns True if the component has been authenticated with given license keys.
     */
    static get isAuthenticated() {
        return CLVideo._isAuthenticated;
    }
    constructor(props) {
        super(props);
        this.onViewCreated = this.onViewCreated.bind(this);
        this.onStateChanged = this.onStateChanged.bind(this);
        this.onError = this.onError.bind(this);
        this.onSeekTo = this.onSeekTo.bind(this);
        this.onSeekCompleted = this.onSeekCompleted.bind(this);
        this.onFullyBuffered = this.onFullyBuffered.bind(this);
        this.onThumbnailLoaded = this.onThumbnailLoaded.bind(this);
        this.onPlayerModelChanged = this.onPlayerModelChanged.bind(this);
        this.onPlaylistItemChanged = this.onPlaylistItemChanged.bind(this);
        this.onPlaylistEnded = this.onPlaylistEnded.bind(this);
    }
    shouldComponentUpdate(
    /* eslint-disable @typescript-eslint/no-unused-vars */
    nextProps, nextState, nextContext) {
        return this.props.media !== nextProps.media;
    }
    componentDidUpdate(
    /* eslint-disable @typescript-eslint/no-unused-vars */
    prevProps, prevState) {
        this._playWhenReady = this.getPlayerConfiguration()?.autoPlay || false;
    }
    get instanceId() {
        return this._instanceId;
    }
    getPlayerConfiguration() {
        return this.props.media instanceof Playlist
            ? this.props.media.current
            : this.props.media;
    }
    render() {
        if (!CLVideo.isAuthenticated) {
            return <ErrorMessage message="PRESTOplay SDK not authenticated"/>;
        }
        const mediaLoadRequest = MediaLoadRequest.fromMedia(this.props.media);
        // TODO remove playWhenReady param in glue code
        // TODO remove loop param in glue code
        return (<View style={[styles.container, this.props.style]}>
        <VideoView 
        /* @ts-ignore */
        style={styles.video} mediaLoadRequest={mediaLoadRequest.toJSON()} onViewCreated={this.onViewCreated} onStateChanged={this.onStateChanged} onError={this.onError} onSeekTo={this.onSeekTo} onSeekCompleted={this.onSeekCompleted} onFullyBuffered={this.onFullyBuffered} onThumbnailLoaded={this.onThumbnailLoaded} onPlayerModelChanged={this.onPlayerModelChanged} onPlaylistItemChanged={this.onPlaylistItemChanged} onPlaylistEnded={this.onPlaylistEnded}/>
      </View>);
    }
    //=================================
    //============  Events  ===========
    //=================================
    /**
     * Called when the view is created
     * @param {NativeSyntheticEvent<ViewCreatedData>} event
     */
    onViewCreated(event) {
        const instanceId = event.nativeEvent.instanceId;
        this._instanceId = instanceId;
        this.props.onViewCreated?.(instanceId);
        this.playlist?.onVideoCreated(instanceId);
    }
    /**
     * Called when the state of the player is changed
     * @param {NativeSyntheticEvent<StateChangedData>} event
     */
    async onStateChanged(event) {
        const playbackState = event.nativeEvent.playbackState;
        await this.syncPlaybackState(playbackState);
        if (playbackState === PlayerState.Finished) {
            await this.pause();
        }
        this.props.onStateChanged?.(playbackState);
    }
    /**
     * On iOS, when a call is received, the playback state
     * automatically changes to paused. When the call ends,
     * the playback state remains paused. However, if the player is
     * in picture-in-picture mode (both Android and iOS), the
     * playback state changes to playing. Therefore, it's necessary
     * to synchronize `playWhenReady` value with the `playbackState`.
     * @param {number} playbackState
     */
    async syncPlaybackState(playbackState) {
        switch (playbackState) {
            case PlayerState.Pausing:
                if (this._playWhenReady) {
                    await this.pause();
                }
                break;
            case PlayerState.Playing:
                if (!this._playWhenReady) {
                    await this.play();
                }
                break;
        }
    }
    /**
     * Called on player error
     * @param {NativeSyntheticEvent<ErrorData>} event
     */
    onError(event) {
        this.props.onError?.(event.nativeEvent);
    }
    /**
     * Called when the player is sought to another position
     * @param {NativeSyntheticEvent<SeekToData>} event
     */
    onSeekTo(event) {
        this.props.onSeekTo?.(event.nativeEvent.newPosition);
    }
    /**
     * Called when seeking is completed
     */
    onSeekCompleted() {
        this.props.onSeekCompleted?.();
    }
    /**
     * Called when the player is fully buffered
     */
    onFullyBuffered() {
        this.props.onFullyBuffered?.();
    }
    /**
     * Called when the thumbnail is loaded. The actual image data is not sent in the
     * data parameter. Instead, the bitmap is loaded in the thumbnail component which
     * is set to this video component
     * @param {NativeSyntheticEvent<ThumbnailData>} event
     */
    onThumbnailLoaded(event) {
        this.props.onThumbnailLoaded?.(event.nativeEvent);
    }
    /**
     * Called when the player model is changed. The player model is changed on the
     * playback initial start-up or when the media period is changed.
     */
    onPlayerModelChanged() {
        // TODO is it in use?
        this.props.onPlayerModelChanged?.();
    }
    onPlaylistItemChanged(event) {
        // If the new index is 0, the iOS will pass undefined.
        // In this case, we set the index back to 0.
        this.playlist?.onItemChanged(event.nativeEvent.newItemIndex || 0);
    }
    onPlaylistEnded() {
        this.playlist?.onEnded();
    }
    //=================================
    //============  Methods  ==========
    //=================================
    /**
     * Returns the current player state.
     * @returns The current player state
     */
    getPlayerState() {
        return this.failIfNotReady(() => this.nativeVideoModule.getPlayerState(this.instanceId));
    }
    /**
     * Returns the total duration of content.
     * @returns The total content duration in seconds
     */
    getDuration() {
        return this.failIfNotReady(() => this.nativeVideoModule.getDuration(this.instanceId));
    }
    /**
     * Returns true if the stream is live.
     * @returns Weather the stream is live or not
     */
    isLive() {
        return this.failIfNotReady(() => this.nativeVideoModule.isLive(this.instanceId));
    }
    /**
     * Returns the current position.
     * @returns The current position in seconds.
     */
    getPosition() {
        return this.failIfNotReady(() => this.nativeVideoModule.getPosition(this.instanceId));
    }
    /**
     * Sets the new position.
     * @param newPosition The new position in seconds.
     */
    setPosition(newPosition) {
        if (this.ready) {
            this.nativeVideoModule.setPosition(this.instanceId, newPosition);
        }
    }
    /**
     * Returns the current volume.
     * @returns The current volume, a value between 0 and 1.
     */
    getVolume() {
        return this.failIfNotReady(() => this.nativeVideoModule.getVolume(this.instanceId));
    }
    /**
     * Sets the new volume.
     * @param newVolume The new volume, a value between 0 and 1.
     */
    setVolume(newVolume) {
        if (this.ready) {
            this.nativeVideoModule.setVolume(this.instanceId, newVolume);
        }
    }
    /**
     * Returns the current video quality.
     * @returns The current video quality.
     */
    getVideoQuality() {
        return this.failIfNotReady(() => this.nativeVideoModule.getVideoQuality(this.instanceId));
    }
    /**
     * Returns the current video quality mode.
     * @returns {@link C.VIDEO_QUALITY_ADAPTIVE} or {@link C.VIDEO_QUALITY_MANUAL}
     */
    getVideoQualityMode() {
        return this.failIfNotReady(() => this.nativeVideoModule.getVideoQualityMode(this.instanceId));
    }
    /**
     * Sets the current video quality.
     * @param index The index of the video quality in the {@link getVideoQualities} array
     */
    setVideoQuality(index) {
        if (this.ready) {
            this.nativeVideoModule.setVideoQuality(this.instanceId, index);
        }
    }
    /**
     * Returns all video qualities.
     * @returns All video qualities.
     */
    getVideoQualities() {
        return this.failIfNotReady(() => this.nativeVideoModule.getVideoQualities(this.instanceId));
    }
    /**
     * Returns the current audio track.
     * @returns The current audio track.
     */
    getAudioTrack() {
        return this.failIfNotReady(() => this.nativeVideoModule.getAudioTrack(this.instanceId));
    }
    /**
     * Sets the current audio track.
     * @param index - The index of the audio track in the {@link getAudioTracks()} array.
     */
    setAudioTrack(index) {
        if (this.ready) {
            this.nativeVideoModule.setAudioTrack(this.instanceId, index);
        }
    }
    /**
     * Returns all audio tracks.
     * @returns All audio tracks.
     */
    getAudioTracks() {
        return this.failIfNotReady(() => this.nativeVideoModule.getAudioTracks(this.instanceId));
    }
    /**
     * Returns the current text track.
     * @returns The current text track.
     */
    getSubtitleTrack() {
        // TODO rename to getTextTrack
        return this.failIfNotReady(() => this.nativeVideoModule.getSubtitleTrack(this.instanceId));
    }
    /**
     * Sets the current text track
     * @param index The index of the text track in the {@link getSubtitleTracks()} array.
     */
    setSubtitleTrack(index) {
        // TODO rename to setTextTrack
        if (this.ready) {
            this.nativeVideoModule.setSubtitleTrack(this.instanceId, index);
        }
    }
    /**
     * Returns all text tracks.
     * @returns All text tracks.
     */
    getSubtitleTracks() {
        // TODO rename to getTextTracks
        return this.failIfNotReady(() => this.nativeVideoModule.getSubtitleTracks(this.instanceId));
    }
    /**
     * Sets the style of the text track.
     * @param style The style of the text track.
     */
    setSubtitleStyle(style) {
        // TODO rename to setTextStyle
        if (this.ready) {
            this.nativeVideoModule.setSubtitleStyle(this.instanceId, style);
        }
    }
    /**
     * This method doesn't return anything but will trigger {@link CLVideoProps.onThumbnailLoaded}
     * event if the thumbnail is loaded. It also sets the bitmap of the thumbnail component
     * if the component is set to this video component.
     *
     * @param position Position in seconds
     */
    getThumbnailData(position) {
        // TODO rename to loadThumbnail
        if (this.ready) {
            this.nativeVideoModule.getThumbnailData(this.instanceId, position);
        }
    }
    /**
     * Registers the {@link CLThumbnail} component.
     * @param thumbnail The CLThumbnail component.
     */
    setThumbnail(thumbnail) {
        if (this.ready) {
            this.nativeVideoModule.setThumbnail(this.instanceId, thumbnail.instanceId);
        }
    }
    /**
     * Returns the playback progress.
     * @returns The playback progress as percentage.
     */
    getProgressPercentage() {
        return this.failIfNotReady(() => this.nativeVideoModule.getProgressPercentage(this.instanceId));
    }
    /**
     * Returns the buffering progress.
     * @returns The buffering progress as percentage.
     */
    getBufferedPercentage() {
        return this.failIfNotReady(() => this.nativeVideoModule.getBufferedPercentage(this.instanceId));
    }
    /**
     * Plays the video.
     */
    async play() {
        // TODO add pause method to glue code
        const state = await this.getPlayerState();
        if (state === PlayerState.Finished) {
            this.setPosition(0);
        }
        if (this.ready) {
            this._playWhenReady = true;
            await this.nativeVideoModule.play(this.instanceId);
        }
    }
    /**
     * Pauses the video.
     */
    async pause() {
        // TODO add pause method to glue code
        if (this.ready) {
            this._playWhenReady = false;
            await this.nativeVideoModule.pause(this.instanceId);
        }
    }
    /**
     * Returns true if the player will transition into playing state when enough data is loaded. If false the player will transition into paused state.
     * Note that the player can have intermediate states until it reaches the playing state.
     * @returns {boolean} True if the video will play when enough data, false otherwise.
     */
    isPlayWhenReady() {
        return this._playWhenReady;
    }
    /**
     * Checks whether the picture-in-picture mode is available or not.
     * @return `true` if picture-in-picture mode is available; Otherwise `false`.
     */
    isPictureInPicturePossible() {
        return this.failIfNotReady(() => this.nativeVideoModule.isPictureInPicturePossible(this.instanceId));
    }
    /**
     * Activates or deactivates the picture-in-picture mode.
     */
    togglePictureInPicture() {
        if (this.ready) {
            this.nativeVideoModule.togglePictureInPicture(this.instanceId);
        }
    }
    /**
     * Returns the live start time.
     * A live HLS stream has to have the `EXT-X-PROGRAM-DATE-TIME` defined.
     * @returns The live start time since the epoch in seconds; `-1` if it is unknown or not applicable.
     */
    getLiveStartTime() {
        if (this.notReady) {
            return Promise.reject(-1);
        }
        return this.nativeVideoModule.getLiveStartTime(this.instanceId);
    }
    /**
     * Checks whether the AirPlay is available or not.
     * @return `true` if AirPlay is available; Otherwise `false`.
     */
    isAirplayPossible() {
        return this.failIfNotReady(() => this.nativeVideoModule.isAirplayPossible(this.instanceId));
    }
    /**
     * Returns statistics and information about the current state of the player.
     * @return The statistics.
     */
    getStats() {
        return this.failIfNotReady(() => this.nativeVideoModule.getStats(this.instanceId));
    }
    /**
     * Returns speed of the playback.
     * @return The current speed of the playback, where 1 means "normal" speed.
     */
    getPlaybackRate() {
        return this.failIfNotReady(() => this.nativeVideoModule.getPlaybackRate(this.instanceId));
    }
    /**
     * Sets speed of the playback, where 1 means "normal" speed.
     *
     * On iOS and tvOS, the playback rate can only be increased if
     * the current position is not near the live edge.
     * This behaviour in AVPlayer is designed to prevent stalling.
     *
     * @param newPlaybackRate The new playback rate, where 1 means "normal" speed.
     */
    setPlaybackRate(newPlaybackRate) {
        this.nativeVideoModule.setPlaybackRate(this.instanceId, newPlaybackRate);
    }
    get playlist() {
        return this.props.media instanceof Playlist ? this.props.media : undefined;
    }
    get nativeVideoModule() {
        return NativeModules.VideoModule;
    }
    get ready() {
        return this.instanceId !== undefined;
    }
    get notReady() {
        return this.instanceId === undefined;
    }
    failIfNotReady(callback) {
        if (this.notReady) {
            return Promise.reject(new PlayerNotReadyError());
        }
        return callback();
    }
}
const styles = StyleSheet.create({
    container: {
        width: '100%',
        height: '100%',
    },
    video: {
        flex: 1,
    },
});
export default CLVideo;
