import Metadata from './Metadata';
import AnalyticsMetadata from './AnalyticsMetadata';
import SideloadedTrack from './SideloadedTrack';
import DrmType from './DrmType';
import DrmSystem from './DrmSystem';
import DrmTodayConfiguration from './DrmTodayConfiguration';
import DrmTodayOnboardConfiguration from './DrmTodayOnboardConfiguration';
import CustomDrmConfiguration from './CustomDrmConfiguration';
import LiveConfiguration from './LiveConfiguration';
import SilentSwitchBehaviour from './SilentSwitchBehaviour';
import Source from './Source';
import NetworkConfiguration from './NetworkConfiguration';
import AndroidPlayerConfiguration from './AndroidPlayerConfiguration';
import SecondaryDisplayBehaviour from './SecondaryDisplayBehaviour';
export default interface PlayerConfiguration {
    /**
     *  Sets whether the player should start playback as soon as enough content
     *  has loaded to allow it to do so without interruption.
     */
    autoPlay?: boolean;
    /**
     * The source to play.
     *
     * If more than one source is available for the same content, the player
     * will select the source best suited to the current device.
     *
     * We recommend providing multiple sources for the same content when using
     * Chromecast. Chromecast doesn't support protected HLS. Therefore, you'll
     * need to provide protected HLS and DASH sources on iOS.
     */
    source: string | Source | Source[];
    /** The start time of the stream in milliseconds. */
    startTimeMs?: number;
    /** Sets whether this is live content. */
    live?: boolean;
    /** Sets whether this is 360 content. */
    threesixty?: boolean;
    /**
     * The language code for the preferred text language.
     * If this is specified, subtitles will be enabled at startup.
     */
    preferredTextLanguage?: string;
    /**
     * The language code for the preferred audio language.
     */
    preferredAudioLanguage?: string;
    /** The metadata of the stream. */
    metaData?: Metadata;
    /** The analytics metadata of the stream. */
    analyticsMetaData?: AnalyticsMetadata;
    /** The side-loaded thumbnail or text tracks */
    sideloadedTracks?: SideloadedTrack[];
    /** The DRM type. */
    drmType?: DrmType;
    /** The DRM system. */
    drmSystem?: DrmSystem;
    /**
     * The DRM configuration.
     *
     * If the `drmType` is `DRMTODAY`, the configuration should be `DrmTodayConfiguration`.
     * If the `drmType` is `DRMTODAY_ONBOARD`, the configuration should be `DrmTodayOnboardConfiguration`.
     * If the `drmType` is `CUSTOM`, the configuration should be `CustomDrmConfiguration`.
     */
    drmConfiguration?: DrmTodayConfiguration | DrmTodayOnboardConfiguration | CustomDrmConfiguration;
    /** The live configuration to tweak live latency and others. */
    liveConfiguration?: LiveConfiguration;
    /**
     * The network configuration to tweak timeouts and retry options.
     */
    networkConfiguration?: NetworkConfiguration;
    /**
     * Sets the manifest header parameters.
     * @remarks Not supported on iOS/tvOS.
     */
    contentParameters?: Object;
    /**
     * Sets the query parameters to add to content (manifest) requests.
     * @remarks Not supported on iOS/tvOS.
     */
    contentQueryParameters?: Object;
    /**
     * Sets the segment header parameters.
     * @remarks Not supported on iOS/tvOS.
     */
    segmentParameters?: Object;
    /**
     * Sets the segment query parameters.
     * @remarks Not supported on iOS/tvOS.
     */
    segmentQueryParameters?: Object;
    /**
     * Sets the user ID of the stream.
     */
    userId?: string;
    /**
     * Sets whether to use DRM init data coming from segments or from the manifest.
     * @remarks Limited to DASH streams. Not supported on iOS/tvOS.
     */
    forceInStreamDrmInitData?: boolean;
    /**
     * Specifies whether to ignore or obey the physical silent switch on Apple devices.
     * If not defined, a default system behaviour is left.
     * @remarks Not supported on Android.
     */
    ignoreSilentSwitch?: SilentSwitchBehaviour;
    /**
     * The behaviour of the player when a secondary display is connected.
     * Defaults to {@link SecondaryDisplayBehaviour.Never}.
     */
    secondaryDisplayBehaviour?: SecondaryDisplayBehaviour;
    /**
     * Player options specific to Android platform.
     */
    android?: AndroidPlayerConfiguration;
}
