import { GenericCallback } from '../index';
import DownloadProgress from '../types/DownloadProgress';
import DownloadErrorCallback from '../types/DownloadErrorCallback';
import DownloadProgressCallback from '../types/DownloadProgressCallback';
import DownloadConfig from '../types/DownloadConfig';
import TrackSet from '../types/TrackSet';
import TrackSelection from '../types/TrackSelection';
import DownloadState from '../types/DownloadState';
/**
 * @example
 * ```typescript
 * import {CLVideo} from '@castlabs/react-native-prestoplay';
 * ```
 */
declare class CLDownloader {
    instanceId: string | null;
    private startCallback;
    private stopCallback;
    private progressCallback;
    private errorCallback;
    private completeCallback;
    /**
     * Creates a new download instance and selects it.
     * @param config The download configuration
     * @returns The available tracks for download.
     */
    prepare(config: DownloadConfig): Promise<TrackSet>;
    /**
     * Selects the download instance by ID.
     * A promise is rejected if the download doesn't exist.
     * @see {@link prepare}
     * @param id The ID of the existing download.
     * @returns The download progress.
     */
    prepareById(id: string): Promise<DownloadProgress>;
    /**
     * Starts downloading selected tracks.
     * @param trackSelection The selected tracks.
     */
    start(trackSelection: TrackSelection): void;
    /**
     * Resumes the selected download.
     */
    resume(): void;
    /**
     * Pauses the selected download.
     */
    pause(): void;
    /**
     * Deletes the selected download.
     */
    delete(): void;
    /**
     * Returns the selected download state.
     * @returns The state of the download.
     */
    getState(): Promise<DownloadState>;
    /**
     * Returns the local manifest file path. It could be used for offline playing.
     * @returns The local file path of the manifest.
     */
    getLocalManifestUrl(): Promise<string>;
    /**
     * Returns the available tracks to download.
     * @returns The available tracks.
     */
    getTracks(): Promise<TrackSet>;
    /**
     * Returns the progress of the selected download.
     * @returns The progress of the selected download.
     */
    getProgress(): Promise<DownloadProgress>;
    /**
     * Binds an event handler to the download started event.
     * @param callback A function to execute each time the event is triggered.
     */
    onStart(callback: GenericCallback): void;
    /**
     * Binds an event handler to the download stopped event.
     * @param callback A function to execute each time the event is triggered.
     */
    onStop(callback: GenericCallback): void;
    /**
     * Binds an event handler to the download progress event.
     * @param callback A function to execute each time the event is triggered.
     */
    onProgress(callback: DownloadProgressCallback): void;
    /**
     * Binds an event handler to the download error event.
     * @param callback A function to execute each time the event is triggered.
     */
    onError(callback: DownloadErrorCallback): void;
    /**
     * Binds an event handler to the download completed event.
     * @param callback A function to execute each time the event is triggered.
     */
    onComplete(callback: GenericCallback): void;
    private static instances;
    private static nativeEventEmitter;
    private static events;
    private static callDownloaderCallback;
    private static addInstance;
    private static addListeners;
    private static addListener;
}
export default CLDownloader;
