import { NativeEventEmitter, NativeModules, } from 'react-native';
const DownloadModule = NativeModules.DownloadModule;
/**
 * @example
 * ```typescript
 * import {CLVideo} from '@castlabs/react-native-prestoplay';
 * ```
 */
class CLDownloader {
    instanceId = null;
    startCallback = null;
    stopCallback = null;
    progressCallback = null;
    errorCallback = null;
    completeCallback = null;
    //=================================
    //============  Methods  ==========
    //=================================
    /**
     * Creates a new download instance and selects it.
     * @param config The download configuration
     * @returns The available tracks for download.
     */
    prepare(config) {
        this.instanceId = config.id;
        return DownloadModule.prepareDownload({
            id: config.id,
            url: config.url,
            drm: JSON.stringify(config.drm),
        });
    }
    /**
     * Selects the download instance by ID.
     * A promise is rejected if the download doesn't exist.
     * @see {@link prepare}
     * @param id The ID of the existing download.
     * @returns The download progress.
     */
    prepareById(id) {
        this.instanceId = id;
        CLDownloader.addInstance(this);
        return DownloadModule.prepareById(id);
    }
    /**
     * Starts downloading selected tracks.
     * @param trackSelection The selected tracks.
     */
    start(trackSelection) {
        if (this.instanceId == null) {
            return;
        }
        DownloadModule.startDownload(this.instanceId, trackSelection);
    }
    /**
     * Resumes the selected download.
     */
    resume() {
        if (this.instanceId == null) {
            return;
        }
        DownloadModule.resumeDownload(this.instanceId);
    }
    /**
     * Pauses the selected download.
     */
    pause() {
        if (this.instanceId == null) {
            return;
        }
        DownloadModule.pauseDownload(this.instanceId);
    }
    /**
     * Deletes the selected download.
     */
    delete() {
        if (this.instanceId == null) {
            return;
        }
        DownloadModule.deleteDownload(this.instanceId);
        this.instanceId = null;
    }
    /**
     * Returns the selected download state.
     * @returns The state of the download.
     */
    getState() {
        if (this.instanceId == null) {
            return Promise.reject();
        }
        return DownloadModule.getState(this.instanceId);
    }
    /**
     * Returns the local manifest file path. It could be used for offline playing.
     * @returns The local file path of the manifest.
     */
    getLocalManifestUrl() {
        if (this.instanceId == null) {
            return Promise.reject();
        }
        return DownloadModule.getLocalManifestUrl(this.instanceId);
    }
    /**
     * Returns the available tracks to download.
     * @returns The available tracks.
     */
    getTracks() {
        if (this.instanceId == null) {
            return Promise.reject();
        }
        return DownloadModule.getTracks(this.instanceId);
    }
    /**
     * Returns the progress of the selected download.
     * @returns The progress of the selected download.
     */
    getProgress() {
        if (this.instanceId == null) {
            return Promise.reject();
        }
        return DownloadModule.getProgress(this.instanceId);
    }
    //=================================
    //============  Events  ===========
    //=================================
    /**
     * Binds an event handler to the download started event.
     * @param callback A function to execute each time the event is triggered.
     */
    onStart(callback) {
        if (this.instanceId == null) {
            return;
        }
        this.startCallback = callback;
    }
    /**
     * Binds an event handler to the download stopped event.
     * @param callback A function to execute each time the event is triggered.
     */
    onStop(callback) {
        if (this.instanceId == null) {
            return;
        }
        this.stopCallback = callback;
    }
    /**
     * Binds an event handler to the download progress event.
     * @param callback A function to execute each time the event is triggered.
     */
    onProgress(callback) {
        if (this.instanceId == null) {
            return;
        }
        this.progressCallback = callback;
    }
    /**
     * Binds an event handler to the download error event.
     * @param callback A function to execute each time the event is triggered.
     */
    onError(callback) {
        if (this.instanceId == null) {
            return;
        }
        this.errorCallback = callback;
    }
    /**
     * Binds an event handler to the download completed event.
     * @param callback A function to execute each time the event is triggered.
     */
    onComplete(callback) {
        if (this.instanceId == null) {
            return;
        }
        this.completeCallback = callback;
    }
    //=================================
    //============  Static  ===========
    //=================================
    static instances = [];
    static nativeEventEmitter = new NativeEventEmitter(DownloadModule);
    static events = {
        'CLDownloader-Start': (event) => {
            CLDownloader.callDownloaderCallback(event.downloadId, (instance) => {
                instance.startCallback ? instance.startCallback() : null;
            });
        },
        'CLDownloader-Stop': (event) => {
            CLDownloader.callDownloaderCallback(event.downloadId, (instance) => {
                instance.stopCallback ? instance.stopCallback() : null;
            });
        },
        'CLDownloader-Progress': (event) => {
            CLDownloader.callDownloaderCallback(event.downloadId, (instance) => {
                instance.progressCallback ? instance.progressCallback(event) : null;
            });
        },
        'CLDownloader-Error': (event) => {
            CLDownloader.callDownloaderCallback(event.downloadId, (instance) => {
                instance.errorCallback ? instance.errorCallback(event) : null;
            });
        },
        'CLDownloader-Complete': (event) => {
            CLDownloader.callDownloaderCallback(event.downloadId, (instance) => {
                instance.completeCallback ? instance.completeCallback() : null;
            });
        },
    };
    static callDownloaderCallback(identifier, callback) {
        CLDownloader.instances.forEach(instance => {
            if (identifier === instance.instanceId) {
                callback(instance);
            }
        });
    }
    static addInstance(instance) {
        CLDownloader.addListeners();
        CLDownloader.instances.push(instance);
    }
    static addListeners() {
        if (CLDownloader.instances.length > 0) {
            return;
        }
        for (const key in CLDownloader.events) {
            const value = CLDownloader.events[key];
            CLDownloader.addListener(key, value);
        }
    }
    static addListener(name, callback) {
        let subscription = SubscriptionManager.get(name);
        if (subscription != null) {
            subscription.remove();
        }
        subscription = CLDownloader.nativeEventEmitter.addListener(name, callback);
        SubscriptionManager.set(name, subscription);
    }
}
class SubscriptionManager {
    static keys = [];
    static values = [];
    static set(key, value) {
        let n = this.keys.indexOf(key);
        if (n === -1) {
            this.keys.push(key);
            this.values.push(value);
        }
        else {
            this.values[n] = value;
        }
    }
    static get(key) {
        let n = this.keys.indexOf(key);
        if (n === -1) {
            return null;
        }
        else {
            return this.values[n];
        }
    }
}
export default CLDownloader;
