import { MediaLoadRequest } from 'react-native-google-cast';
import PlayerConfiguration from '../types/PlayerConfiguration';
import DrmTodayConfiguration from '../types/DrmTodayConfiguration';
import { CustomDrmConfiguration } from '../index';
import DrmTodayOnboardConfiguration from '../types/DrmTodayOnboardConfiguration';
import CLVideo from '../components/CLVideo';
/**
 * Converts a {@link CLVideo} component or a {@link PlayerConfiguration} object into a
 * {@link https://github.com/react-native-google-cast/react-native-google-cast/blob/main/src/types/MediaLoadRequest.ts MediaLoadRequest}
 * object that is used to load media to Chromecast. The MediaLoadRequest is part of the
 * {@link https://github.com/react-native-google-cast/react-native-google-cast react-native-google-cast} library.
 */
export default class MediaLoadRequestUtils {
    /**
     * Converts a {@link CLVideo} component to a MediaLoadRequest object,
     * convenient when Chromecast is not connected and there is local playback in progress.
     *
     * This MediaLoadRequest contains the player configuration as well as the current player state, position, audio track and text track.
     * An optional DRM configuration can be provided to override the one in the player configuration.
     * This is particularly useful if the DRM configuration contains authentication tokens that may have expired at the time of casting.
     *
     * Before passing the MediaLoadRequest object to the `loadMedia`,
     * you can add your custom data to the `mediaLoadRequest.mediaInfo.customData` object.
     * Later, you can retrieve this data from the session by accessing the `mediaStatus.mediaInfo.customData` object.
     *
     * @exmple
     * ```javascript
     * const mediaLoadRequest = await MediaLoadRequestUtils.fromVideoComponent(videoComponent);
     * // Add your custom data to the `mediaLoadRequest.mediaInfo.customData` object
     * await castSession.getClient().loadMedia(mediaLoadRequest);
     * ```
     *
     * @param videoComponent The video component
     * @param drmConfiguration The optional DRM configuration
     * @returns The media load request
     */
    static fromVideoComponent(videoComponent: CLVideo, drmConfiguration?: DrmTodayConfiguration | DrmTodayOnboardConfiguration | CustomDrmConfiguration): Promise<MediaLoadRequest>;
    /**
     * Converts a {@link PlayerConfiguration} component to a MediaLoadRequest object,
     * convenient when Chromecast is connected and there is no local playback.
     *
     * An optional DRM configuration can be provided to override the one in the player configuration.
     * This is particularly useful if the DRM configuration contains authentication tokens that may have expired at the time of casting.
     *
     * Before passing the MediaLoadRequest object to the `loadMedia`,
     * you can add your custom data to the `mediaLoadRequest.mediaInfo.customData` object.
     * Later, you can retrieve this data from the session by accessing the `mediaStatus.mediaInfo.customData` object.
     *
     * @exmple
     * ```javascript
     * const mediaLoadRequest = await MediaLoadRequestUtils.fromPlayerConfiguration(playerConfiguration);
     * // Add your custom data to the `mediaLoadRequest.mediaInfo.customData` object
     * await castSession.getClient().loadMedia(mediaLoadRequest);
     * ```
     *
     * @param playerConfiguration The player configuration
     * @param drmConfiguration The optional DRM configuration
     * @returns The MediaLoadRequest object
     */
    static fromPlayerConfiguration(playerConfiguration: PlayerConfiguration, drmConfiguration?: DrmTodayConfiguration | DrmTodayOnboardConfiguration | CustomDrmConfiguration): MediaLoadRequest;
    private static toStartTime;
    private static toContentId;
    private static toContentType;
    private static toSource;
    private static toStreamType;
    private static toDrmConfiguration;
}
