import { PLAYER_CONFIGURATION_CAST_KEY } from './Constants';
/**
 * Converts a MediaStatus object to a {@link PlayerConfiguration} object.
 * The MediaStatus is part of the
 * {@link https://github.com/react-native-google-cast/react-native-google-cast react-native-google-cast} library.
 */
export default class MediaStatusUtils {
    // TODO rename to CastMediaStatusUtils
    /**
     * Converts a MediaStatus object to a {@link PlayerConfiguration} object.
     *
     * The player configuration contains the current player state, position, audio track and text track.
  
     * An optional DRM configuration can be provided to override the one in the player configuration.
     * This is particularly useful if the DRM configuration contains authentication tokens that may be already expired.
     *
     * @example
     * ```jsx
     * const playerConfiguration = MediaStatusUtils.toPlayerConfiguration(
     *    this.mediaStatus,
     * );
     * // ...
     * <CLVideo media={playerConfiguration} />
     * ```
     * @param mediaStatus The media status
     * @param drmConfiguration The optional DRM configuration
     * @returns The player configuration
     */
    static toPlayerConfiguration(mediaStatus, drmConfiguration) {
        const self = MediaStatusUtils;
        const customData = mediaStatus.mediaInfo?.customData ?? {};
        if (!customData[PLAYER_CONFIGURATION_CAST_KEY]) {
            return null;
        }
        const playerConfiguration = customData[PLAYER_CONFIGURATION_CAST_KEY];
        return Object.assign({}, playerConfiguration, {
            startTimeMs: self.toStartTime(mediaStatus),
            preferredAudioLanguage: self.toPreferredAudioLanguage(mediaStatus),
            preferredTextLanguage: self.toPreferredTextLanguage(mediaStatus),
            drmConfiguration: drmConfiguration ?? playerConfiguration.drmConfiguration,
        });
    }
    static toPreferredAudioLanguage(mediaStatus) {
        const tracks = mediaStatus.mediaInfo?.mediaTracks ?? [];
        const activeTrackIds = mediaStatus.activeTrackIds ?? [];
        const activeAudioTrack = tracks
            .filter(track => track.type === 'audio')
            .find(track => activeTrackIds.includes(track.id));
        return activeAudioTrack?.language;
    }
    static toPreferredTextLanguage(mediaStatus) {
        const tracks = mediaStatus.mediaInfo?.mediaTracks ?? [];
        const activeTrackIds = mediaStatus.activeTrackIds ?? [];
        const activeTextTrack = tracks
            .filter(track => track.type === 'text')
            .find(track => activeTrackIds.includes(track.id));
        return activeTextTrack?.language;
    }
    static toStartTime(mediaStatus) {
        return Math.floor(mediaStatus.streamPosition * 1000);
    }
}
