import { PlayerConfiguration } from '../index';
import PlaylistEndedCallback from '../types/PlaylistEndedCallback';
import PlaylistItemChangedCallback from '../types/PlaylistItemChangedCallback';
/**
 * A list of items to play.
 *
 *  Downloaded media doesn't work with playlists on Apple devices.
 *  We recommend using the single player configuration for downloaded media.
 */
export default class Playlist {
    private _videoId;
    private _currentIndex;
    private readonly _items;
    private readonly _endedCallback;
    private readonly _itemChangedCallback;
    /**
     * Create a new playlist.
     *
     * @param items The items in the playlist.
     * @param onItemChanged The callback to trigger when the item is changed.
     * @param onEnded The callback to trigger when the playlist ends.
     */
    constructor(items: PlayerConfiguration[], onItemChanged?: PlaylistItemChangedCallback, onEnded?: PlaylistEndedCallback);
    /**
     * Triggered by the video component when the native video is created.
     * @param videoId The the native video ID.
     * @ignore
     */
    onVideoCreated(videoId: number): void;
    /**
     * Triggered by the video component when the item is changed.
     * @param newIndex The index of the new item.
     * @ignore
     */
    onItemChanged(newIndex: number): void;
    /**
     * Triggered by the video component when the playlist ends.
     * @ignore
     */
    onEnded(): void;
    /**
     * Play the next item in the playlist.
     * @throws {PlaylistNotReadyError} If the playlist is not ready.
     * @throws {PlaylistItemNotFoundError} If the next item doesn't exist.
     * @returns A promise that resolves when the next item is played.
     */
    playNext(): Promise<void>;
    /**
     * Play the previous item in the playlist.
     * @throws {PlaylistNotReadyError} If the playlist is not ready.
     * @throws {PlaylistItemNotFoundError} If the previous item doesn't exist.
     * @returns A promise that resolves when the previous item is played.
     */
    playPrev(): Promise<void>;
    /**
     * Play an item in the playlist.
     * @param at The index of the item to play.
     * @throws {PlaylistNotReadyError} If the playlist is not ready.
     * @throws {PlaylistItemNotFoundError} If the item doesn't exist.
     * @returns A promise that resolves when the item is played.
     */
    playItem(at: number): Promise<void>;
    /**
     * Check if there is a next item in the playlist.
     * @returns True if there is a next item, false otherwise.
     */
    hasNext(): boolean;
    /**
     * Check if there is a previous item in the playlist.
     * @returns True if there is a previous item, false otherwise.
     */
    hasPrev(): boolean;
    /**
     * The index of the current item in the playlist.
     */
    get currentIndex(): number;
    /**
     * The current item in the playlist.
     */
    get current(): PlayerConfiguration | undefined;
    /**
     * The items in the playlist.
     */
    get items(): ReadonlyArray<PlayerConfiguration>;
    /**
     * Add an item to the playlist.
     * @param playerConfiguration The player configuration for the item.
     * @param at The index to insert the item at.
     * @returns A promise that resolves when the item is added.
     */
    insert(playerConfiguration: PlayerConfiguration, at?: number): Promise<void>;
    /**
     * Remove an item from the playlist.
     * @param at The index of the item to remove.
     * @throws {PlaylistItemNotFoundError} If the item doesn't exist.
     * @returns A promise that resolves when the item is removed.
     */
    remove(at: number): Promise<void>;
    private get ready();
    private get nativeVideoModule();
}
