import PlaylistNotReadyError from '../errors/PlaylistNotReadyError';
import PlaylistItemNotFoundError from '../errors/PlaylistItemNotFoundError';
import { NativeModules } from 'react-native';
/**
 * A list of items to play.
 *
 *  Downloaded media doesn't work with playlists on Apple devices.
 *  We recommend using the single player configuration for downloaded media.
 */
export default class Playlist {
    _videoId;
    _currentIndex = 0;
    _items = [];
    _endedCallback;
    _itemChangedCallback;
    /**
     * Create a new playlist.
     *
     * @param items The items in the playlist.
     * @param onItemChanged The callback to trigger when the item is changed.
     * @param onEnded The callback to trigger when the playlist ends.
     */
    constructor(items, onItemChanged, onEnded) {
        this._items = [...items];
        this._currentIndex = 0;
        this._itemChangedCallback = onItemChanged;
        this._endedCallback = onEnded;
    }
    /**
     * Triggered by the video component when the native video is created.
     * @param videoId The the native video ID.
     * @ignore
     */
    onVideoCreated(videoId) {
        this._videoId = videoId;
    }
    /**
     * Triggered by the video component when the item is changed.
     * @param newIndex The index of the new item.
     * @ignore
     */
    onItemChanged(newIndex) {
        this._currentIndex = newIndex;
        this._itemChangedCallback?.(this.currentIndex);
    }
    /**
     * Triggered by the video component when the playlist ends.
     * @ignore
     */
    onEnded() {
        this._endedCallback?.();
    }
    /**
     * Play the next item in the playlist.
     * @throws {PlaylistNotReadyError} If the playlist is not ready.
     * @throws {PlaylistItemNotFoundError} If the next item doesn't exist.
     * @returns A promise that resolves when the next item is played.
     */
    async playNext() {
        await this.playItem(this.currentIndex + 1);
    }
    /**
     * Play the previous item in the playlist.
     * @throws {PlaylistNotReadyError} If the playlist is not ready.
     * @throws {PlaylistItemNotFoundError} If the previous item doesn't exist.
     * @returns A promise that resolves when the previous item is played.
     */
    async playPrev() {
        await this.playItem(this.currentIndex - 1);
    }
    /**
     * Play an item in the playlist.
     * @param at The index of the item to play.
     * @throws {PlaylistNotReadyError} If the playlist is not ready.
     * @throws {PlaylistItemNotFoundError} If the item doesn't exist.
     * @returns A promise that resolves when the item is played.
     */
    async playItem(at) {
        if (!this.ready) {
            throw new PlaylistNotReadyError();
        }
        if (at < 0 || at >= this._items.length) {
            throw new PlaylistItemNotFoundError();
        }
        await this.nativeVideoModule.playPlaylistItem(this._videoId, at);
        this._currentIndex = at;
    }
    /**
     * Check if there is a next item in the playlist.
     * @returns True if there is a next item, false otherwise.
     */
    hasNext() {
        return this._currentIndex < this._items.length - 1;
    }
    /**
     * Check if there is a previous item in the playlist.
     * @returns True if there is a previous item, false otherwise.
     */
    hasPrev() {
        return this._currentIndex > 0;
    }
    /**
     * The index of the current item in the playlist.
     */
    get currentIndex() {
        return this._currentIndex;
    }
    /**
     * The current item in the playlist.
     */
    get current() {
        return this._items[this._currentIndex];
    }
    /**
     * The items in the playlist.
     */
    get items() {
        return this._items;
    }
    /**
     * Add an item to the playlist.
     * @param playerConfiguration The player configuration for the item.
     * @param at The index to insert the item at.
     * @returns A promise that resolves when the item is added.
     */
    async insert(playerConfiguration, at) {
        if (at === undefined) {
            at = this._items.length;
        }
        await this.nativeVideoModule.addPlaylistItem(this._videoId, at, JSON.stringify(playerConfiguration));
        this._items.splice(at, 0, playerConfiguration);
    }
    /**
     * Remove an item from the playlist.
     * @param at The index of the item to remove.
     * @throws {PlaylistItemNotFoundError} If the item doesn't exist.
     * @returns A promise that resolves when the item is removed.
     */
    async remove(at) {
        if (at < 0 || at >= this._items.length) {
            throw new PlaylistItemNotFoundError();
        }
        await this.nativeVideoModule.removePlaylistItem(this._videoId, at);
        this._items.splice(at, 1);
    }
    get ready() {
        return this._videoId !== undefined;
    }
    get nativeVideoModule() {
        return NativeModules.VideoModule;
    }
}
