import {
  NativeEventEmitter,
  EmitterSubscription,
  NativeModules,
} from 'react-native';
import {CLVideo, AudioTrack, SubtitleTrack} from '../index';
import TVStream from '../types/TVStream';
import PlayerOptions from '../types/PlayerOptions';
import Action from '../types/Action';
import ActionUiProperty from '../types/ActionUiProperty';
import ChannelMetadata from '../types/ChannelMetadata';
import PopupMenu from '../types/PopupMenu';
import WillSeekCallback from '../types/WillSeekCallback';
import DidSeekCallback from '../types/DidSeekCallback';
import GenericEventCallback from '../types/GenericEventCallback';
import SubtitleTrackChangedCallback from '../types/SubtitleTrackChangedCallback';
import AudioTrackChangedCallback from '../types/AudioTrackChangedCallback';
import ErrorEventCallback from '../types/ErrorEventCallback';
import DrmSystem from '../types/DrmSystem';
import SourceUtils from '../utils/SourceUtils';
import Objects from '../utils/Objects';
import StreamChangedCallback from '../types/StreamChangedCallback';

const {TVPlayerModule} = NativeModules;

const PlayerEvent = {
  DidPlayToEnd: 'did-play-to-end',
  PlayerIsClosed: 'player-is-closed',
  WillSeek: 'will-seek',
  DidSeek: 'did-seek',
  TimeControlStatus: 'time-control-status',
  SkipToNextChannel: 'skip-to-next-channel',
  SkipToPreviousChannel: 'skip-to-previous-channel',
  PlaybackStateError: 'playback-state-error',
  AudioTrackChanged: 'audio-track-changed',
  SubtitleTrackChanged: 'subtitle-track-changed',
  StreamChanged: 'stream-changed',
};

interface NativeTVStream {
  playerConfig: string;
  channelMetadata: string;
  nextChannelInfo?: string;
}

/**
 * The AVPlayerViewController proxy, which provides playback capabilities using the default system UI.
 *
 * @example
 * ```typescript
 * import {CLAppleTVPlayer} from '@castlabs/react-native-prestoplay';
 * ```
 * @remarks Not supported on Android.
 */
class CLAppleTVPlayer {
  /**
   * Triggered on playback ended.
   * Close the player or change the stream here.
   */
  public static didPlayToEnd?: GenericEventCallback;

  /**
   * Triggered on player closed.
   */
  public static playerIsClosed?: GenericEventCallback;

  /**
   * Triggered on seeking started.
   */
  public static willSeek?: WillSeekCallback;

  /**
   * Triggered on seeking completed.
   */
  public static didSeek?: DidSeekCallback;

  /**
   * Triggered on playback status changed.
   */
  public static timeControlStatus?: GenericEventCallback;

  /**
   * Triggered when the user skips to the next channel.
   *
   * This callback is only triggered for live streams and when a single stream is loaded.
   * It is not triggered when a playlist is in use, i.e., when multiple streams are queued.
   * To replace the current stream with the next one, call the {@link CLAppleTVPlayer.openPlayer} method.
   */
  public static skipToNextChannel?: GenericEventCallback;

  /**
   * Triggered when the user skips to the previous channel.
   *
   * This callback is only triggered for live streams and when a single stream is loaded.
   * It is not triggered when a playlist is in use, i.e., when multiple streams are queued.
   * To replace the current stream with the next one, call the {@link CLAppleTVPlayer.openPlayer} method.
   */
  public static skipToPreviousChannel?: GenericEventCallback;

  /**
   * Triggered on error thrown.
   */
  public static onError?: ErrorEventCallback;

  /**
   * Triggered on audio track changed.
   */
  public static audioTrackChanged?: AudioTrackChangedCallback;

  /**
   * Triggered on subtitle track changed.
   */
  public static subtitleTrackChanged?: SubtitleTrackChangedCallback;

  /**
   * Triggered on stream changed.
   */
  public static streamChanged?: StreamChangedCallback;

  private static nativeEventEmitter = new NativeEventEmitter(TVPlayerModule);

  // noinspection JSVoidFunctionReturnValueUsed
  private static _initialize = (() => {
    this.registerPlayerEvent();
  })();

  private static registerPlayerEvent() {
    const listener = (event: any) => {
      console.log('onPlayerEvent: ', event);
      if (event == null) {
        return;
      }

      switch (event.id) {
        case PlayerEvent.DidPlayToEnd:
          this.didPlayToEnd?.(event);
          break;
        case PlayerEvent.PlayerIsClosed:
          this.playerIsClosed?.(event);
          break;
        case PlayerEvent.WillSeek:
          this.willSeek?.(event);
          break;
        case PlayerEvent.DidSeek:
          this.didSeek?.(event);
          break;
        case PlayerEvent.TimeControlStatus:
          this.timeControlStatus?.(event);
          break;
        case PlayerEvent.SkipToNextChannel:
          this.skipToNextChannel?.(event);
          break;
        case PlayerEvent.SkipToPreviousChannel:
          this.skipToPreviousChannel?.(event);
          break;
        case PlayerEvent.PlaybackStateError:
          this.onError?.(event);
          break;
        case PlayerEvent.AudioTrackChanged:
          this.audioTrackChanged?.(event);
          break;
        case PlayerEvent.SubtitleTrackChanged:
          this.subtitleTrackChanged?.(event);
          break;
        case PlayerEvent.StreamChanged:
          this.streamChanged?.(event);
          break;
      }
    };
    this.nativeEventEmitter.addListener('onPlayerEvent', listener);
  }

  /**
   * Subscribes to the action event.
   * @example
   * ```typescript
   * CLAppleTVPlayer.addActionListener((event: any) => {
   *   switch (event.id) {
   *     case ActionId.Speed:
   *       CLAppleTVPlayer.setRate(Number(event.data));
   *       break;
   *     case ActionId.Skip00:
   *       CLAppleTVPlayer.seekToSecond(15);
   *       break;
   *   }
   * });
   * ```
   * @param listener the listener
   * @returns the event subscription
   */
  public static addActionListener(
    listener: (event: any) => void,
  ): EmitterSubscription {
    return this.nativeEventEmitter.addListener('onActionEvent', listener);
  }

  /**
   * Unsubscribes from the action event.
   * @param subscription the event subscription
   */
  public static removeActionListener(subscription: EmitterSubscription) {
    subscription.remove();
  }

  /**
   * Opens the player.
   *
   * If a single stream is loaded, the {@link CLAppleTVPlayer.skipToNextChannel}
   * and {@link CLAppleTVPlayer.skipToPreviousChannel} callbacks are fired
   * when the user skips to the next or previous channel. Also, when a single
   * stream is loaded, you can replace it with another by calling
   * the {@link CLAppleTVPlayer.openPlayer} method again. This is particularly
   * useful if you want to replace the current stream with a different one.
   *
   * If multiple streams are loaded, the player will automatically switch
   * to the next stream when the current stream ends. In addition,
   * the {@link CLAppleTVPlayer.streamChanged} callback will be fired when
   * the player switches to the next stream. When all streams are live,
   * the user can use the TV remote to switch between channels. However,
   * the {@link CLAppleTVPlayer.skipToNextChannel} and
   * {@link CLAppleTVPlayer.skipToPreviousChannel} callbacks will not fire.
   *
   * If loopStream is false, the player will ignore next or previous channel
   * requests if the current stream is the last or first item in the list.
   * The player will still display the channel's interstitial view,
   * but will not change the stream.
   *
   * @param streams the stream configuration
   * @param options the player options
   * @param transportBarActions the transport bar actions
   * @param infoTabActions the info tab actions
   * @param contextualActions the contextual actions
   * @param loopStreams loops through the streams list. Default is `true`.
   * @param onError the error callback
   * @param onSuccess the success callback
   */
  public static openPlayer(
    streams: TVStream[],
    options?: PlayerOptions,
    transportBarActions?: Array<Action | PopupMenu>,
    infoTabActions?: Action[],
    contextualActions?: Action[],
    loopStreams?: Boolean,
    onError?: (error: any) => void,
    onSuccess?: () => void,
  ) {
    if (CLVideo.isAuthenticated) {
      const nativeStreams = streams.map(stream =>
        CLAppleTVPlayer.toNativeTVStream(stream),
      );
      TVPlayerModule.openPlayer(
        nativeStreams,
        options,
        transportBarActions,
        infoTabActions,
        contextualActions,
        loopStreams,
        (error: any) => {
          if (error) {
            onError?.(error);
          } else {
            onSuccess?.();
          }
        },
      );
    } else {
      onError?.('Authentication failed.');
    }
  }

  /**
   * Starts playback.
   */
  public static play() {
    TVPlayerModule.play();
  }

  /**
   * Pauses playback.
   */
  public static pause() {
    TVPlayerModule.pause();
  }

  /**
   * Closes the player.
   */
  public static closePlayer() {
    TVPlayerModule.closePlayer();
  }

  /**
   * Sets the metadata to be displayed in the title view and info tab.
   * @param metadata the channel metadata
   */
  public static setMetadata(metadata: ChannelMetadata) {
    TVPlayerModule.setMetadata(JSON.stringify(metadata));
  }

  /**
   * Sets player options.
   * @param options the player options
   */
  public static setOptions(options: PlayerOptions) {
    TVPlayerModule.setOptions(options);
  }

  /**
   * Sets transport bar actions.
   * @param transportBarActions the transport bar actions
   */
  public static setTransportBarActions(
    transportBarActions: Array<Action | PopupMenu>,
  ) {
    TVPlayerModule.setTransportBarActions(transportBarActions);
  }

  /**
   * Sets info tab actions.
   * @param infoTabActions the info tab actions
   */
  public static setInfoTabActions(infoTabActions: Action[]) {
    TVPlayerModule.setInfoTabActions(infoTabActions);
  }

  /**
   * Sets contextual actions.
   * @param contextualActions the contextual actions
   */
  public static setContextualActions(contextualActions: Action[]) {
    TVPlayerModule.setContextualActions(contextualActions);
  }

  /**
   * Changes the state of the action and UI appearance.
   * @param id the action identifier
   * @param state the action state
   * @param uiProp the action title and image resource name in Xcode project
   */
  public static setActionStateAndUi(
    id: string,
    state: boolean,
    uiProp: ActionUiProperty,
  ) {
    TVPlayerModule.setActionStateAndUi(id, state, uiProp);
  }

  /**
   * Sets the new playback rate.
   * @param rate the playback rate
   */
  public static setRate(rate: number) {
    TVPlayerModule.setRate(rate);
  }

  /**
   * Moves playhead to the given position.
   * @param second the new position in seconds
   */
  public static seekToSecond(second: number) {
    TVPlayerModule.seekToSecond(second);
  }

  /**
   * Returns the current playhead.
   * @returns the current position in seconds
   */
  public static async getPosition(): Promise<number> {
    return TVPlayerModule.getPosition();
  }

  /**
   * Returns the live start time.
   * A live HLS stream has to have the `EXT-X-PROGRAM-DATE-TIME` defined.
   * @returns The live start time since the epoch in seconds; `-1` if it is unknown or not applicable.
   */
  public static async getLiveStartTime(): Promise<number> {
    return TVPlayerModule.getLiveStartTime();
  }

  /**
   * Returns the current audio track.
   * @returns {Promise<AudioTrack>} the current audio track
   */
  public static async getAudioTrack(): Promise<AudioTrack> {
    return TVPlayerModule.getAudioTrack();
  }

  /**
   * Returns the current subtitle track.
   * @returns {Promise<SubtitleTrack>} the current subtitle track
   */
  public static async getSubtitleTrack(): Promise<SubtitleTrack> {
    return TVPlayerModule.getSubtitleTrack();
  }

  /**
   * Returns the index of the current stream.
   * @returns {Promise<number>} the current stream index
   */
  public static async getCurrentStreamIndex(): Promise<number> {
    return TVPlayerModule.getCurrentStreamIndex();
  }

  private static toNativeTVStream(stream: TVStream): NativeTVStream {
    const playerConfig = Objects.cloneDeep(stream.playerConfig);

    playerConfig.drmSystem = DrmSystem.FAIRPLAY;
    playerConfig.source = SourceUtils.findMatchedSource(
      SourceUtils.getSources(playerConfig),
      SourceUtils.IOS_FILTERS,
    );

    return {
      playerConfig: JSON.stringify(playerConfig),
      channelMetadata: JSON.stringify(stream.channelMetadata),
      nextChannelInfo: stream.nextChannelInfo
        ? JSON.stringify(stream.nextChannelInfo)
        : undefined,
    };
  }
}

export default CLAppleTVPlayer;
