import {
  NativeEventEmitter,
  NativeEventSubscription,
  NativeModules,
} from 'react-native';
import {GenericCallback} from '../index';
import DownloadProgress from '../types/DownloadProgress';
import DownloadErrorCallback from '../types/DownloadErrorCallback';
import DownloadProgressCallback from '../types/DownloadProgressCallback';
import DownloadConfig from '../types/DownloadConfig';
import TrackSet from '../types/TrackSet';
import TrackSelection from '../types/TrackSelection';
import DownloadState from '../types/DownloadState';

const DownloadModule = NativeModules.DownloadModule;

/**
 * @example
 * ```typescript
 * import {CLVideo} from '@castlabs/react-native-prestoplay';
 * ```
 */
class CLDownloader {
  public instanceId: string | null = null;

  private startCallback: (() => void) | null = null;
  private stopCallback: (() => void) | null = null;
  private progressCallback: ((event: any) => void) | null = null;
  private errorCallback: ((event: any) => void) | null = null;
  private completeCallback: (() => void) | null = null;

  //=================================
  //============  Methods  ==========
  //=================================

  /**
   * Creates a new download instance and selects it.
   * @param config The download configuration
   * @returns The available tracks for download.
   */
  public prepare(config: DownloadConfig): Promise<TrackSet> {
    this.instanceId = config.id;
    return DownloadModule.prepareDownload({
      id: config.id,
      url: config.url,
      drm: JSON.stringify(config.drm),
    });
  }

  /**
   * Selects the download instance by ID.
   * A promise is rejected if the download doesn't exist.
   * @see {@link prepare}
   * @param id The ID of the existing download.
   * @returns The download progress.
   */
  public prepareById(id: string): Promise<DownloadProgress> {
    this.instanceId = id;

    CLDownloader.addInstance(this);
    return DownloadModule.prepareById(id);
  }

  /**
   * Starts downloading selected tracks.
   * @param trackSelection The selected tracks.
   */
  public start(trackSelection: TrackSelection) {
    if (this.instanceId == null) {
      return;
    }
    DownloadModule.startDownload(this.instanceId, trackSelection);
  }

  /**
   * Resumes the selected download.
   */
  public resume() {
    if (this.instanceId == null) {
      return;
    }
    DownloadModule.resumeDownload(this.instanceId);
  }

  /**
   * Pauses the selected download.
   */
  public pause() {
    if (this.instanceId == null) {
      return;
    }
    DownloadModule.pauseDownload(this.instanceId);
  }

  /**
   * Deletes the selected download.
   */
  public delete() {
    if (this.instanceId == null) {
      return;
    }
    DownloadModule.deleteDownload(this.instanceId);
    this.instanceId = null;
  }

  /**
   * Returns the selected download state.
   * @returns The state of the download.
   */
  public getState(): Promise<DownloadState> {
    if (this.instanceId == null) {
      return Promise.reject();
    }
    return DownloadModule.getState(this.instanceId);
  }

  /**
   * Returns the local manifest file path. It could be used for offline playing.
   * @returns The local file path of the manifest.
   */
  public getLocalManifestUrl(): Promise<string> {
    if (this.instanceId == null) {
      return Promise.reject();
    }
    return DownloadModule.getLocalManifestUrl(this.instanceId);
  }

  /**
   * Returns the available tracks to download.
   * @returns The available tracks.
   */
  public getTracks(): Promise<TrackSet> {
    if (this.instanceId == null) {
      return Promise.reject();
    }
    return DownloadModule.getTracks(this.instanceId);
  }

  /**
   * Returns the progress of the selected download.
   * @returns The progress of the selected download.
   */
  public getProgress(): Promise<DownloadProgress> {
    if (this.instanceId == null) {
      return Promise.reject();
    }
    return DownloadModule.getProgress(this.instanceId);
  }

  //=================================
  //============  Events  ===========
  //=================================

  /**
   * Binds an event handler to the download started event.
   * @param callback A function to execute each time the event is triggered.
   */
  public onStart(callback: GenericCallback) {
    if (this.instanceId == null) {
      return;
    }
    this.startCallback = callback;
  }

  /**
   * Binds an event handler to the download stopped event.
   * @param callback A function to execute each time the event is triggered.
   */
  public onStop(callback: GenericCallback) {
    if (this.instanceId == null) {
      return;
    }
    this.stopCallback = callback;
  }

  /**
   * Binds an event handler to the download progress event.
   * @param callback A function to execute each time the event is triggered.
   */
  public onProgress(callback: DownloadProgressCallback) {
    if (this.instanceId == null) {
      return;
    }
    this.progressCallback = callback;
  }

  /**
   * Binds an event handler to the download error event.
   * @param callback A function to execute each time the event is triggered.
   */
  public onError(callback: DownloadErrorCallback) {
    if (this.instanceId == null) {
      return;
    }
    this.errorCallback = callback;
  }

  /**
   * Binds an event handler to the download completed event.
   * @param callback A function to execute each time the event is triggered.
   */
  public onComplete(callback: GenericCallback) {
    if (this.instanceId == null) {
      return;
    }
    this.completeCallback = callback;
  }

  //=================================
  //============  Static  ===========
  //=================================

  private static instances: CLDownloader[] = [];
  private static nativeEventEmitter = new NativeEventEmitter(DownloadModule);
  private static events: Record<string, (event: any) => void> = {
    'CLDownloader-Start': (event: any) => {
      CLDownloader.callDownloaderCallback(
        event.downloadId,
        (instance: CLDownloader) => {
          instance.startCallback ? instance.startCallback() : null;
        },
      );
    },
    'CLDownloader-Stop': (event: any) => {
      CLDownloader.callDownloaderCallback(
        event.downloadId,
        (instance: CLDownloader) => {
          instance.stopCallback ? instance.stopCallback() : null;
        },
      );
    },
    'CLDownloader-Progress': (event: any) => {
      CLDownloader.callDownloaderCallback(
        event.downloadId,
        (instance: CLDownloader) => {
          instance.progressCallback ? instance.progressCallback(event) : null;
        },
      );
    },
    'CLDownloader-Error': (event: any) => {
      CLDownloader.callDownloaderCallback(
        event.downloadId,
        (instance: CLDownloader) => {
          instance.errorCallback ? instance.errorCallback(event) : null;
        },
      );
    },
    'CLDownloader-Complete': (event: any) => {
      CLDownloader.callDownloaderCallback(
        event.downloadId,
        (instance: CLDownloader) => {
          instance.completeCallback ? instance.completeCallback() : null;
        },
      );
    },
  };

  private static callDownloaderCallback(
    identifier: string,
    callback: (instance: CLDownloader) => void,
  ) {
    CLDownloader.instances.forEach(instance => {
      if (identifier === instance.instanceId) {
        callback(instance);
      }
    });
  }

  private static addInstance(instance: CLDownloader) {
    CLDownloader.addListeners();
    CLDownloader.instances.push(instance);
  }

  private static addListeners() {
    if (CLDownloader.instances.length > 0) {
      return;
    }
    for (const key in CLDownloader.events) {
      const value = CLDownloader.events[key];
      CLDownloader.addListener(key, value);
    }
  }

  private static addListener(name: string, callback: (event: any) => void) {
    let subscription = SubscriptionManager.get(name);
    if (subscription != null) {
      subscription.remove();
    }
    subscription = CLDownloader.nativeEventEmitter.addListener(name, callback);
    SubscriptionManager.set(name, subscription);
  }
}

class SubscriptionManager {
  private static keys: String[] = [];
  private static values: NativeEventSubscription[] = [];

  public static set(key: string, value: NativeEventSubscription) {
    let n = this.keys.indexOf(key);
    if (n === -1) {
      this.keys.push(key);
      this.values.push(value);
    } else {
      this.values[n] = value;
    }
  }

  public static get(key: string): NativeEventSubscription | null {
    let n = this.keys.indexOf(key);
    if (n === -1) {
      return null;
    } else {
      return this.values[n];
    }
  }
}

export default CLDownloader;
