import {MediaLoadRequest} from 'react-native-google-cast';
import MediaStreamType from 'react-native-google-cast/src/types/MediaStreamType';
import ContentType from '../types/ContentType';
import SourceUtils from './SourceUtils';
import PlayerConfiguration from '../types/PlayerConfiguration';
import DrmType from '../types/DrmType';
import DrmTodayConfiguration from '../types/DrmTodayConfiguration';
import DrmTodayEnvironment from '../types/DrmTodayEnvironment';
import {CustomDrmConfiguration} from '../index';
import DrmTodayOnboardConfiguration from '../types/DrmTodayOnboardConfiguration';
import {PLAYER_CONFIGURATION_CAST_KEY} from './Constants';
import CLVideo from '../components/CLVideo';
import PlayerState from '../types/PlayerState';
import PlayerConfigurationNotFoundError from '../errors/PlayerConfigurationNotFoundError';

/**
 * Converts a {@link CLVideo} component or a {@link PlayerConfiguration} object into a
 * {@link https://github.com/react-native-google-cast/react-native-google-cast/blob/main/src/types/MediaLoadRequest.ts MediaLoadRequest}
 * object that is used to load media to Chromecast. The MediaLoadRequest is part of the
 * {@link https://github.com/react-native-google-cast/react-native-google-cast react-native-google-cast} library.
 */
export default class MediaLoadRequestUtils {
  // TODO rename to CastMediaLoadRequestUtils

  /**
   * Converts a {@link CLVideo} component to a MediaLoadRequest object,
   * convenient when Chromecast is not connected and there is local playback in progress.
   *
   * This MediaLoadRequest contains the player configuration as well as the current player state, position, audio track and text track.
   * An optional DRM configuration can be provided to override the one in the player configuration.
   * This is particularly useful if the DRM configuration contains authentication tokens that may have expired at the time of casting.
   *
   * Before passing the MediaLoadRequest object to the `loadMedia`,
   * you can add your custom data to the `mediaLoadRequest.mediaInfo.customData` object.
   * Later, you can retrieve this data from the session by accessing the `mediaStatus.mediaInfo.customData` object.
   *
   * @exmple
   * ```javascript
   * const mediaLoadRequest = await MediaLoadRequestUtils.fromVideoComponent(videoComponent);
   * // Add your custom data to the `mediaLoadRequest.mediaInfo.customData` object
   * await castSession.getClient().loadMedia(mediaLoadRequest);
   * ```
   *
   * @param videoComponent The video component
   * @param drmConfiguration The optional DRM configuration
   * @returns The media load request
   */
  public static async fromVideoComponent(
    videoComponent: CLVideo,
    drmConfiguration?:
      | DrmTodayConfiguration
      | DrmTodayOnboardConfiguration
      | CustomDrmConfiguration,
  ): Promise<MediaLoadRequest> {
    const self = MediaLoadRequestUtils;

    const state = await videoComponent.getPlayerState();
    const position = await videoComponent.getPosition(); // seconds
    const audioTrack = await videoComponent.getAudioTrack();
    const textTrack = await videoComponent.getSubtitleTrack();

    const basePlayerConfiguration = videoComponent.getPlayerConfiguration();
    if (!basePlayerConfiguration) {
      throw new PlayerConfigurationNotFoundError();
    }

    const playerConfiguration = Object.assign({}, basePlayerConfiguration, {
      autoPlay: state === PlayerState.Playing,
      startTimeMs: position * 1000,
      preferredAudioLanguage: audioTrack?.language,
      preferredTextLanguage: textTrack?.language,
    });

    return self.fromPlayerConfiguration(playerConfiguration, drmConfiguration);
  }

  /**
   * Converts a {@link PlayerConfiguration} component to a MediaLoadRequest object,
   * convenient when Chromecast is connected and there is no local playback.
   *
   * An optional DRM configuration can be provided to override the one in the player configuration.
   * This is particularly useful if the DRM configuration contains authentication tokens that may have expired at the time of casting.
   *
   * Before passing the MediaLoadRequest object to the `loadMedia`,
   * you can add your custom data to the `mediaLoadRequest.mediaInfo.customData` object.
   * Later, you can retrieve this data from the session by accessing the `mediaStatus.mediaInfo.customData` object.
   *
   * @exmple
   * ```javascript
   * const mediaLoadRequest = await MediaLoadRequestUtils.fromPlayerConfiguration(playerConfiguration);
   * // Add your custom data to the `mediaLoadRequest.mediaInfo.customData` object
   * await castSession.getClient().loadMedia(mediaLoadRequest);
   * ```
   *
   * @param playerConfiguration The player configuration
   * @param drmConfiguration The optional DRM configuration
   * @returns The MediaLoadRequest object
   */
  public static fromPlayerConfiguration(
    playerConfiguration: PlayerConfiguration,
    drmConfiguration?:
      | DrmTodayConfiguration
      | DrmTodayOnboardConfiguration
      | CustomDrmConfiguration,
  ): MediaLoadRequest {
    const self = MediaLoadRequestUtils;

    playerConfiguration = Object.assign({}, playerConfiguration, {
      drmConfiguration:
        drmConfiguration ?? playerConfiguration.drmConfiguration,
    });

    return {
      autoplay: playerConfiguration.autoPlay,
      startTime: self.toStartTime(playerConfiguration),
      mediaInfo: {
        contentUrl: '',
        contentId: self.toContentId(playerConfiguration),
        contentType: self.toContentType(playerConfiguration),
        streamType: self.toStreamType(playerConfiguration),
        metadata: {
          type: 'movie',
        },
        customData: {
          playerConfiguration: {
            viewerId: playerConfiguration.userId,
            source: self.toSource(playerConfiguration),
            drm: self.toDrmConfiguration(playerConfiguration),
            preferredAudioLanguage: playerConfiguration.preferredAudioLanguage,
            preferredTextLanguage: playerConfiguration.preferredTextLanguage,
          },
          [PLAYER_CONFIGURATION_CAST_KEY]: playerConfiguration,
        },
      },
    };
  }

  private static toStartTime(
    playerConfiguration: PlayerConfiguration,
  ): number | undefined {
    return playerConfiguration.startTimeMs !== undefined
      ? playerConfiguration.startTimeMs / 1000
      : undefined;
  }

  private static toContentId(playerConfiguration: PlayerConfiguration): string {
    return SourceUtils.findMatchedSource(
      SourceUtils.getSources(playerConfiguration),
      SourceUtils.CHROMECAST_FILTERS,
    ).url;
  }

  private static toContentType(
    playerConfiguration: PlayerConfiguration,
  ): string {
    const type = SourceUtils.findMatchedSource(
      SourceUtils.getSources(playerConfiguration),
      SourceUtils.CHROMECAST_FILTERS,
    ).type;

    switch (type) {
      case ContentType.DASH:
        return 'application/dash+xml';
      case ContentType.SMOOTH:
        return 'application/vnd.ms-sstr+xml';
      case ContentType.HLS:
        return 'application/x-mpegurl';
      case ContentType.MP4:
        return 'video/mp4';
      default:
        throw new Error('Unknown content type');
    }
  }

  private static toSource(playerConfiguration: PlayerConfiguration): any {
    const source = SourceUtils.findMatchedSource(
      SourceUtils.getSources(playerConfiguration),
      SourceUtils.CHROMECAST_FILTERS,
    );

    return {
      url: source.url,
      type: this.toContentType(playerConfiguration),
      drmProtected: source.drmProtected,
      isLive: playerConfiguration.live ?? false,
    };
  }

  private static toStreamType(
    playerConfiguration: PlayerConfiguration,
  ): MediaStreamType | undefined {
    if (playerConfiguration.live === true) {
      return MediaStreamType.LIVE;
    } else {
      return MediaStreamType.BUFFERED;
    }
  }

  private static toDrmConfiguration(
    playerConfiguration: PlayerConfiguration,
  ): any | undefined {
    if (
      playerConfiguration.drmConfiguration &&
      playerConfiguration.drmType === DrmType.CUSTOM
    ) {
      const drmConfiguration =
        playerConfiguration.drmConfiguration as CustomDrmConfiguration;

      const headers = Object.assign(
        {},
        drmConfiguration.WIDEVINE?.licensingParameters,
        drmConfiguration.PLAYREADY?.licensingParameters,
      );

      return {
        env: 'HeaderDrm',
        customData: {
          widevineLicenseUrl: drmConfiguration?.WIDEVINE?.licensingUrl,
          playReadyLicenseUrl: drmConfiguration?.PLAYREADY?.licensingUrl,
          headers,
        },
      };
    } else if (
      playerConfiguration.drmConfiguration &&
      playerConfiguration.drmType === DrmType.DRMTODAY
    ) {
      const drmConfiguration =
        playerConfiguration.drmConfiguration as DrmTodayConfiguration;

      let env: string = '';
      switch (drmConfiguration.environment) {
        case DrmTodayEnvironment.STAGING:
          env = 'DRMtoday_STAGING';
          break;
        case DrmTodayEnvironment.TESTING:
          env = 'DRMtoday_TESTING';
          break;
        default:
          env = 'DRMtoday';
          break;
      }

      return {
        env,
        customData: {
          userId: drmConfiguration.userId,
          sessionId: drmConfiguration.sessionId,
          merchant: drmConfiguration.merchant,
          assetId: drmConfiguration.assetId,
          variantId: drmConfiguration.variantId,
          authToken: drmConfiguration.authToken,
        },
      };
    } else {
      return undefined;
    }
  }
}
