import PlayerConfiguration from '../types/PlayerConfiguration';
import {MediaStatus} from 'react-native-google-cast';
import {PLAYER_CONFIGURATION_CAST_KEY} from './Constants';
import CustomDrmConfiguration from '../types/CustomDrmConfiguration';
import DrmTodayConfiguration from '../types/DrmTodayConfiguration';
import DrmTodayOnboardConfiguration from '../types/DrmTodayOnboardConfiguration';

/**
 * Converts a MediaStatus object to a {@link PlayerConfiguration} object.
 * The MediaStatus is part of the
 * {@link https://github.com/react-native-google-cast/react-native-google-cast react-native-google-cast} library.
 */
export default class MediaStatusUtils {
  // TODO rename to CastMediaStatusUtils

  /**
   * Converts a MediaStatus object to a {@link PlayerConfiguration} object.
   *
   * The player configuration contains the current player state, position, audio track and text track.

   * An optional DRM configuration can be provided to override the one in the player configuration.
   * This is particularly useful if the DRM configuration contains authentication tokens that may be already expired.
   *
   * @example
   * ```jsx
   * const playerConfiguration = MediaStatusUtils.toPlayerConfiguration(
   *    this.mediaStatus,
   * );
   * // ...
   * <CLVideo media={playerConfiguration} />
   * ```
   * @param mediaStatus The media status
   * @param drmConfiguration The optional DRM configuration
   * @returns The player configuration
   */
  public static toPlayerConfiguration(
    mediaStatus: MediaStatus,
    drmConfiguration?:
      | DrmTodayConfiguration
      | DrmTodayOnboardConfiguration
      | CustomDrmConfiguration,
  ): PlayerConfiguration | null {
    const self = MediaStatusUtils;

    const customData: any = mediaStatus.mediaInfo?.customData ?? {};
    if (!customData[PLAYER_CONFIGURATION_CAST_KEY]) {
      return null;
    }

    const playerConfiguration = customData[
      PLAYER_CONFIGURATION_CAST_KEY
    ] as PlayerConfiguration;

    return Object.assign({}, playerConfiguration, {
      startTimeMs: self.toStartTime(mediaStatus),
      preferredAudioLanguage: self.toPreferredAudioLanguage(mediaStatus),
      preferredTextLanguage: self.toPreferredTextLanguage(mediaStatus),
      drmConfiguration:
        drmConfiguration ?? playerConfiguration.drmConfiguration,
    });
  }

  private static toPreferredAudioLanguage(
    mediaStatus: MediaStatus,
  ): string | undefined {
    const tracks = mediaStatus.mediaInfo?.mediaTracks ?? [];
    const activeTrackIds = mediaStatus.activeTrackIds ?? [];

    const activeAudioTrack = tracks
      .filter(track => track.type === 'audio')
      .find(track => activeTrackIds.includes(track.id));

    return activeAudioTrack?.language;
  }

  private static toPreferredTextLanguage(
    mediaStatus: MediaStatus,
  ): string | undefined {
    const tracks = mediaStatus.mediaInfo?.mediaTracks ?? [];
    const activeTrackIds = mediaStatus.activeTrackIds ?? [];

    const activeTextTrack = tracks
      .filter(track => track.type === 'text')
      .find(track => activeTrackIds.includes(track.id));

    return activeTextTrack?.language;
  }

  private static toStartTime(mediaStatus: MediaStatus): number {
    return Math.floor(mediaStatus.streamPosition * 1000);
  }
}
