import {PlayerConfiguration} from '../index';
import PlaylistEndedCallback from '../types/PlaylistEndedCallback';
import PlaylistItemChangedCallback from '../types/PlaylistItemChangedCallback';
import PlaylistNotReadyError from '../errors/PlaylistNotReadyError';
import PlaylistItemNotFoundError from '../errors/PlaylistItemNotFoundError';
import {NativeModules} from 'react-native';

/**
 * A list of items to play.
 *
 *  Downloaded media doesn't work with playlists on Apple devices.
 *  We recommend using the single player configuration for downloaded media.
 */
export default class Playlist {
  private _videoId: number | undefined;
  private _currentIndex = 0;

  private readonly _items: PlayerConfiguration[] = [];
  private readonly _endedCallback: PlaylistEndedCallback | undefined;
  private readonly _itemChangedCallback:
    | PlaylistItemChangedCallback
    | undefined;

  /**
   * Create a new playlist.
   *
   * @param items The items in the playlist.
   * @param onItemChanged The callback to trigger when the item is changed.
   * @param onEnded The callback to trigger when the playlist ends.
   */
  constructor(
    items: PlayerConfiguration[],
    onItemChanged?: PlaylistItemChangedCallback,
    onEnded?: PlaylistEndedCallback,
  ) {
    this._items = [...items];
    this._currentIndex = 0;

    this._itemChangedCallback = onItemChanged;
    this._endedCallback = onEnded;
  }

  /**
   * Triggered by the video component when the native video is created.
   * @param videoId The the native video ID.
   * @ignore
   */
  public onVideoCreated(videoId: number): void {
    this._videoId = videoId;
  }

  /**
   * Triggered by the video component when the item is changed.
   * @param newIndex The index of the new item.
   * @ignore
   */
  public onItemChanged(newIndex: number): void {
    this._currentIndex = newIndex;
    this._itemChangedCallback?.(this.currentIndex);
  }

  /**
   * Triggered by the video component when the playlist ends.
   * @ignore
   */
  public onEnded(): void {
    this._endedCallback?.();
  }

  /**
   * Play the next item in the playlist.
   * @throws {PlaylistNotReadyError} If the playlist is not ready.
   * @throws {PlaylistItemNotFoundError} If the next item doesn't exist.
   * @returns A promise that resolves when the next item is played.
   */
  public async playNext(): Promise<void> {
    await this.playItem(this.currentIndex + 1);
  }

  /**
   * Play the previous item in the playlist.
   * @throws {PlaylistNotReadyError} If the playlist is not ready.
   * @throws {PlaylistItemNotFoundError} If the previous item doesn't exist.
   * @returns A promise that resolves when the previous item is played.
   */
  public async playPrev(): Promise<void> {
    await this.playItem(this.currentIndex - 1);
  }

  /**
   * Play an item in the playlist.
   * @param at The index of the item to play.
   * @throws {PlaylistNotReadyError} If the playlist is not ready.
   * @throws {PlaylistItemNotFoundError} If the item doesn't exist.
   * @returns A promise that resolves when the item is played.
   */
  public async playItem(at: number): Promise<void> {
    if (!this.ready) {
      throw new PlaylistNotReadyError();
    }
    if (at < 0 || at >= this._items.length) {
      throw new PlaylistItemNotFoundError();
    }

    await this.nativeVideoModule.playPlaylistItem(this._videoId, at);

    this._currentIndex = at;
  }

  /**
   * Check if there is a next item in the playlist.
   * @returns True if there is a next item, false otherwise.
   */
  public hasNext(): boolean {
    return this._currentIndex < this._items.length - 1;
  }

  /**
   * Check if there is a previous item in the playlist.
   * @returns True if there is a previous item, false otherwise.
   */
  public hasPrev(): boolean {
    return this._currentIndex > 0;
  }

  /**
   * The index of the current item in the playlist.
   */
  public get currentIndex(): number {
    return this._currentIndex;
  }

  /**
   * The current item in the playlist.
   */
  public get current(): PlayerConfiguration | undefined {
    return this._items[this._currentIndex];
  }

  /**
   * The items in the playlist.
   */
  public get items(): ReadonlyArray<PlayerConfiguration> {
    return this._items;
  }

  /**
   * Add an item to the playlist.
   * @param playerConfiguration The player configuration for the item.
   * @param at The index to insert the item at.
   * @returns A promise that resolves when the item is added.
   */
  public async insert(
    playerConfiguration: PlayerConfiguration,
    at?: number,
  ): Promise<void> {
    if (at === undefined) {
      at = this._items.length;
    }

    await this.nativeVideoModule.addPlaylistItem(
      this._videoId,
      at,
      JSON.stringify(playerConfiguration),
    );

    this._items.splice(at, 0, playerConfiguration);
  }

  /**
   * Remove an item from the playlist.
   * @param at The index of the item to remove.
   * @throws {PlaylistItemNotFoundError} If the item doesn't exist.
   * @returns A promise that resolves when the item is removed.
   */
  public async remove(at: number): Promise<void> {
    if (at < 0 || at >= this._items.length) {
      throw new PlaylistItemNotFoundError();
    }
    await this.nativeVideoModule.removePlaylistItem(this._videoId, at);
    this._items.splice(at, 1);
  }

  private get ready(): boolean {
    return this._videoId !== undefined;
  }

  private get nativeVideoModule() {
    return NativeModules.VideoModule;
  }
}
